@@ -26,6 +26,13 @@ pref("mail.rights.override", true);
 pref("mail.rights.override", true);
 #endif
 
+// At startup, should we check to see if the installation
+// date is older than some threshold
+pref("app.update.checkInstallTime", true);
+
+// The number of days a binary is permitted to be old without checking is defined in
+// thunderbird-branding.js (app.update.checkInstallTime.days)
+
 // The minimum delay in seconds for the timer to fire between the notification
 // of each consumer of the timer manager.
 // minimum=30 seconds, default=120 seconds, and maximum=300 seconds
@@ -43,6 +50,44 @@ pref("app.update.timerFirstInterval", 30000);
 
 // Enables some extra Application Update Logging (can reduce performance)
 pref("app.update.log", false);
+// Causes Application Update Logging to be sent to a file in the profile
+// directory. This preference is automatically disabled on application start to
+// prevent it from being left on accidentally. Turning this pref on enables
+// logging, even if app.update.log is false.
+pref("app.update.log.file", false);
+
+// The number of general background check failures to allow before notifying the
+// user of the failure. User initiated update checks always notify the user of
+// the failure.
+pref("app.update.backgroundMaxErrors", 10);
+
+// Ids of the links to the "What's new" update documentation
+pref("app.update.link.updateAvailableWhatsNew", "update-available-whats-new");
+pref("app.update.link.updateManualWhatsNew", "update-manual-whats-new");
+
+// How many times we should let downloads fail before prompting the user to
+// download a fresh installer.
+pref("app.update.download.promptMaxAttempts", 2);
+
+// How many times we should let an elevation prompt fail before prompting the user to
+// download a fresh installer.
+pref("app.update.elevation.promptMaxAttempts", 2);
+
+#ifdef NIGHTLY_BUILD
+  // Whether to delay popup notifications when an update is available and
+  // suppress them when an update is installed and waiting for user to restart.
+  // If set to true, these notifications will immediately be shown as banners in
+  // the app menu and as badges on the app menu button. Update available
+  // notifications will not create popup prompts until a week has passed without
+  // the user installing the update. Update restart notifications will not
+  // create popup prompts at all. This doesn't affect update notifications
+  // triggered by errors/failures or manual install prompts.
+  pref("app.update.suppressPrompts", false);
+#endif
+
+// If set to true, a message will be displayed in the hamburger menu while
+// an update is being downloaded.
+pref("app.update.notifyDuringDownload", false);
 
 // If set to true, the Update Service will automatically download updates when
 // user can apply updates. This pref is no longer used on Windows, except as the
@@ -101,7 +146,7 @@ pref("datareporting.healthreport.infoURL", "https://www.mozilla.org/thunderbird/
 pref("datareporting.policy.dataSubmissionEnabled", true);
 pref("datareporting.policy.dataSubmissionPolicyAcceptedVersion", 0);
 pref("datareporting.policy.dataSubmissionPolicyBypassNotification", false);
-pref("datareporting.policy.currentPolicyVersion", 1);
+pref("datareporting.policy.currentPolicyVersion", 2);
 pref("datareporting.policy.firstRunURL", "https://www.mozilla.org/thunderbird/legal/privacy/");
 #endif
 
@@ -141,11 +186,6 @@ pref("extensions.update.autoUpdateDefault", true);
 
 pref("extensions.systemAddon.update.enabled", true);  // See bug 1462160.
 
-pref("extensions.hotfix.id", "thunderbird-hotfix@mozilla.org");
-pref("extensions.hotfix.cert.checkAttributes", true);
-pref("extensions.hotfix.certs.1.sha1Fingerprint", "91:53:98:0C:C1:86:DF:47:8F:35:22:9E:11:C9:A7:31:04:49:A1:AA");
-pref("extensions.hotfix.certs.2.sha1Fingerprint", "39:E7:2B:7A:5B:CF:37:78:F9:5D:4A:E0:53:2D:2F:3D:68:53:C5:60");
-
 // Disable add-ons installed into the shared user and shared system areas by
 // default. This does not include the application directory. See the SCOPE
 // constants in AddonManager.jsm for values to use here
@@ -239,7 +279,6 @@ pref("mail.spellcheck.inline", true);
 
 pref("mail.folder.views.version", 0);
 
-pref("mail.folderpane.showColumns", false);
 // Force the unit shown for the size of all folders. If empty, the unit
 // is determined automatically for each folder. Allowed values: KB/MB/<empty string>
 pref("mail.folderpane.sizeUnits", "");
@@ -255,7 +294,6 @@ pref("mail.last_msg_movecopy_was_move", true);
 //Set the font color for links to something lighter
 pref("browser.anchor_color", "#0B6CDA");
 
-pref("browser.preferences.instantApply", true);
 #ifdef XP_MACOSX
 pref("browser.preferences.animateFadeIn", true);
 #else
@@ -295,7 +333,7 @@ pref("mail.showCondensedAddresses", true); // show the friendly display name for
 
 pref("mailnews.attachments.display.start_expanded", false);
 // hidden pref for changing how we present attachments in the message pane
-pref("mail.pane_config.dynamic",            0);
+pref("mail.pane_config.dynamic", 2);
 pref("mailnews.reuse_thread_window2",     true);
 pref("editor.singleLine.pasteNewlines", 4);  // substitute commas for new lines in single line text boxes
 pref("editor.CR_creates_new_p", true);
@@ -336,10 +374,6 @@ pref("mail.forward_message_mode", 2);
 
 pref("mailnews.send.loglevel", "Warn");
 
-pref("mailnews.nntp.jsmodule", true);
-
-pref("mailnews.pop3.jsmodule", true);
-
 pref("mail.import.in_new_tab", true);
 
 // End core mailnews.js pref overrides
@@ -462,7 +496,6 @@ pref("dom.serviceWorkers.enabled", true);
 // Generic browser related prefs.
 /////////////////////////////////////////////////////////////////
 pref("browser.send_pings", false);
-pref("browser.chrome.toolbar_tips",         true);
 pref("browser.xul.error_pages.expert_bad_cert", false);
 
 // Attachment download manager settings
@@ -481,13 +514,7 @@ pref("browser.helperApps.deleteTempFileOnExit", true);
 #endif
 
 // Not used in Thunderbird.
-#ifdef NIGHTLY_BUILD
 pref("browser.startup.homepage.abouthome_cache.enabled", true);
-#else
-pref("browser.startup.homepage.abouthome_cache.enabled", false);
-#endif
-// Whether to start the private browsing mode at application startup. Not used in Thunderbird.
-pref("browser.privatebrowsing.autostart", false);
 
 pref("spellchecker.dictionary", "");
 // Dictionary download preference
@@ -503,6 +530,11 @@ pref("profile.force.migration", "");
 pref("alerts.totalOpenTime", 10000);
 #endif
 
+// Disable new windows notifications until they are fully supported by Thunderbird (bug 1838139).
+#ifdef XP_WIN
+pref("alerts.useSystemBackend", false);
+#endif
+
 // analyze urls in mail messages for scams
 pref("mail.phishing.detection.enabled", true);
 // If phishing detection is enabled, allow fine grained control
@@ -528,21 +560,13 @@ pref("mail.tabs.loadInBackground", true);
 
 // Tabs
 pref("mail.tabs.tabMinWidth", 100);
-pref("mail.tabs.tabMaxWidth", 210);
+pref("mail.tabs.tabMaxWidth", 250);
 pref("mail.tabs.tabClipWidth", 140);
-pref("mail.tabs.autoHide", false);
+pref("mail.tabs.autoHide", true);
 pref("mail.tabs.closeWindowWithLastTab", true);
 
 // Allow the tabs to be in the titlebar on supported systems
-#ifdef UNIX_BUT_NOT_MAC
-pref("mail.tabs.drawInTitlebar", false);
-#else
 pref("mail.tabs.drawInTitlebar", true);
-#endif
-
-// Offer additional drag space to the user. The drag space
-// will only be shown if browser.tabs.drawInTitlebar is true.
-pref("mail.tabs.extraDragSpace", false);
 
 // The breakpad report server to link to in about:crashes
 pref("breakpad.reportURL", "https://crash-stats.mozilla.com/report/index/");
@@ -781,6 +805,12 @@ pref("places.frecency.defaultVisitBonus", 0);
 pref("places.frecency.unvisitedBookmarkBonus", 140);
 pref("places.frecency.unvisitedTypedBonus", 200);
 
+// Enables alternative frecency calculation for origins.
+pref("places.frecency.origins.alternative.featureGate", false);
+
+// The default Places log level.
+pref("places.loglevel", "Error");
+
 // Windows taskbar support
 #ifdef XP_WIN
 pref("mail.taskbar.lists.enabled", true);
@@ -821,7 +851,7 @@ pref("mail.ignore_thread.learn_more_url", "https://support.thunderbird.net/kb/ig
 
 // Density control for the entire Thunderbird UI.
 // The possible values are 0=compact, 1=normal, 2=touch.
-pref("mail.uidensity", 0);
+pref("mail.uidensity", 1);
 
 // Font size control for the entire Thunderbird UI. The value represents the
 // pixel value which will be applied as inline style to the root element of the
@@ -920,6 +950,13 @@ pref("devtools.browsertoolbox.fission", true);
 pref("devtools.browsertoolbox.fission", false);
 #endif
 
+// When the Multiprocess Browser Toolbox is enabled, you can configure the scope of it:
+// - "everything" will enable debugging absolutely everything in the browser
+//   All processes, all documents, all workers, all add-ons.
+// - "parent-process" will restrict debugging to the parent process
+//   All privileged javascript, documents and workers running in the parent process.
+pref("devtools.browsertoolbox.scope", "everything");
+
 // Toolbox Button preferences
 pref("devtools.command-button-pick.enabled", true);
 pref("devtools.command-button-frames.enabled", true);
@@ -970,6 +1007,10 @@ pref("devtools.gridinspector.showInfiniteLines", false);
 // Max number of grid highlighters that can be displayed
 pref("devtools.gridinspector.maxHighlighters", 3);
 
+// Whether or not simplified highlighters should be used when
+// prefers-reduced-motion is enabled.
+pref("devtools.inspector.simple-highlighters-reduced-motion", false);
+
 // Whether or not the box model panel is opened in the layout view
 pref("devtools.layout.boxmodel.opened", true);
 // Whether or not the flexbox panel is opened in the layout view
@@ -1177,6 +1218,9 @@ pref("devtools.webconsole.groupWarningMessages", true);
 // Saved state of the Display content messages checkbox in the browser console.
 pref("devtools.browserconsole.contentMessages", true);
 
+// Enable network monitoring the browser toolbox console/browser console.
+pref("devtools.browserconsole.enableNetworkMonitoring", false);
+
 // Enable client-side mapping service for source maps
 pref("devtools.source-map.client-service.enabled", true);
 
@@ -1324,10 +1368,6 @@ pref("mail.compose.warn_public_recipients.threshold", 15);
 // too many public recipients notification is shown.
 pref("mail.compose.warn_public_recipients.aggressive", false);
 
-// Temporary pref while both 3-pane UIs exist.
-// Opens folders in about:3pane and messages in about:message if true.
-pref("mail.useNewMailTabs", false);
-
 // The URL of most things that can be printed is useless information.
 // Hide it and move the title to the center.
 pref("print.print_headerleft", "");
@@ -1337,14 +1377,52 @@ pref("print.print_headerright", "");
 // Enable Masonry Layout for AddressBook.
 pref("layout.css.grid-template-masonry-value.enabled", true);
 
-// End of year donation appeal.
-pref("app.donation.eoy.version", 1);
-pref("app.donation.eoy.version.viewed", 0);
-pref("app.donation.eoy.url", "https://www.thunderbird.net/thunderbird/102.0/eoy/");
+#ifdef NIGHTLY_BUILD
+// If set to false, FxAccounts and Sync will be unavailable.
+// A restart is mandatory after flipping that preference.
+pref("identity.fxaccounts.enabled", true);
+pref("identity.fxaccounts.log.sensitive", true);
+pref("services.sync.log.appender.console", "Info");
+// Auto-config URL for FxA self-hosters, makes an HTTP request to
+// [identity.fxaccounts.autoconfig.uri]/.well-known/fxa-client-configuration
+// This is now the prefered way of pointing to a custom FxA server, instead
+// of making changes to "identity.fxaccounts.*.uri".
+pref("identity.fxaccounts.autoconfig.uri", "https://accounts.stage.mozaws.net");
+// The remote FxA root content URL. Must use HTTPS.
+pref("identity.fxaccounts.remote.root", "https://accounts.stage.mozaws.net");
+// The value of the context query parameter passed in FxA requests.
+pref("identity.fxaccounts.contextParam", "fx_desktop_v3");
+// Token server used by the FxA Sync identity.
+pref("identity.sync.tokenserver.uri", "https://token.stage.mozaws.net/1.0/sync/1.5");
+// Adds stage server to the white list, because we need it.
+pref("webchannel.allowObject.urlWhitelist", "https://content.cdn.mozilla.net https://support.mozilla.org https://install.mozilla.org https://accounts.stage.mozaws.net");
+// Adds Firefox/10x.0 to the User-Agent string, because we need it.
+// TODO: Fix this.
+pref("general.useragent.compatMode.firefox", true);
 
-// Beta appeal
-pref("app.beta_appeal.version", 1);
-pref("app.beta_appeal.version.viewed", 0);
-pref("app.beta_appeal.url", "https://www.thunderbird.net/thunderbird/102.0/beta-appeal/?locale=%LOCALE%&version=%VERSION%&channel=%CHANNEL%&os=%OS%&buildid=%APPBUILDID%");
-// June 21st expiry.
-pref("app.beta_appeal.expiry", 1687323600);
+// Enable the sync engines we want, and disable the ones we don't want.
+pref("services.sync.engine.accounts", true);
+pref("services.sync.engine.addons", false);
+pref("services.sync.engine.addressbooks", true);
+pref("services.sync.engine.addresses", false);
+pref("services.sync.engine.calendars", true);
+pref("services.sync.engine.creditcards", false);
+pref("services.sync.engine.identities", true);
+pref("services.sync.engine.prefs", false);
+#endif
+
+// Donation appeal.
+pref("app.donation.eoy.version", 2);
+pref("app.donation.eoy.version.viewed", 0);
+pref("app.donation.eoy.url", "https://www.thunderbird.net/thunderbird/115.0/appeal/");
+
+// IMAP-JS disabled, Bug 1707547.
+pref("mailnews.imap.jsmodule", false);
+
+// Unified toolbar
+
+// 0: icons beside text
+// 1: icons above text
+// 2: icons only
+// 3: text only
+pref("toolbar.unifiedtoolbar.buttonstyle", 0);
