@@ -13,26 +13,26 @@
  */
 
 /* import-globals-from ../../extensions/mailviews/content/msgViewPickerOverlay.js */
-/* import-globals-from commandglue.js */
 /* import-globals-from customizeToolbar.js */
-/* import-globals-from mailWindow.js */
 /* import-globals-from utilityOverlay.js */
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+/* globals gChatTab */ // From globals chat-messenger.js
+/* globals currentAttachments */ // From msgHdrView.js
+
+var { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
-XPCOMUtils.defineLazyGetter(this, "gViewSourceUtils", function() {
+XPCOMUtils.defineLazyGetter(this, "gViewSourceUtils", function () {
   let scope = {};
   Services.scriptloader.loadSubScript(
     "chrome://global/content/viewSourceUtils.js",
     scope
   );
-  scope.gViewSourceUtils.viewSource = async function(aArgs) {
+  scope.gViewSourceUtils.viewSource = async function (aArgs) {
     // Check if external view source is enabled. If so, try it. If it fails,
     // fallback to internal view source.
     if (Services.prefs.getBoolPref("view_source.editor.external")) {
@@ -52,6 +52,18 @@ XPCOMUtils.defineLazyGetter(this, "gViewSourceUtils", function() {
   return scope.gViewSourceUtils;
 });
 
+Object.defineProperty(this, "BrowserConsoleManager", {
+  get() {
+    let { loader } = ChromeUtils.importESModule(
+      "resource://devtools/shared/loader/Loader.sys.mjs"
+    );
+    return loader.require("devtools/client/webconsole/browser-console-manager")
+      .BrowserConsoleManager;
+  },
+  configurable: true,
+  enumerable: true,
+});
+
 var gCustomizeSheet = false;
 
 function overlayRestoreDefaultSet() {
@@ -169,6 +181,12 @@ function overlayRepositionDialog() {
 }
 
 function CustomizeMailToolbar(toolboxId, customizePopupId) {
+  if (toolboxId === "mail-toolbox" && window.tabmail) {
+    // Open the unified toolbar customization panel only for mail.
+    document.querySelector("unified-toolbar").showCustomization();
+    return;
+  }
+
   // Disable the toolbar context menu items
   var menubar = document.getElementById("mail-menubar");
   for (var i = 0; i < menubar.children.length; ++i) {
@@ -187,18 +205,12 @@ function CustomizeMailToolbar(toolboxId, customizePopupId) {
 
   let externalToolbars = [];
   if (toolbox.getAttribute("id") == "mail-toolbox") {
-    if (document.getElementById("tabbar-toolbar")) {
-      externalToolbars.push(document.getElementById("tabbar-toolbar"));
-    }
     if (
       AppConstants.platform != "macosx" &&
       document.getElementById("toolbar-menubar")
     ) {
       externalToolbars.push(document.getElementById("toolbar-menubar"));
     }
-    if (document.getElementById("folderPaneHeader")) {
-      externalToolbars.push(document.getElementById("folderPaneHeader"));
-    }
   }
 
   if (gCustomizeSheet) {
@@ -225,7 +237,7 @@ function CustomizeMailToolbar(toolboxId, customizePopupId) {
     panel.style.visibility = "hidden";
     toolbox.addEventListener(
       "beforecustomization",
-      function() {
+      function () {
         panel.style.removeProperty("visibility");
       },
       { capture: false, once: true }
@@ -264,19 +276,9 @@ function MailToolboxCustomizeDone(aEvent, customizePopupId) {
     ViewPickerOnLoad();
   }
 
-  // make sure the folder location picker is initialized
-  if (document.getElementById("folder-location-container")) {
-    FolderPaneSelectionChange();
-  }
-
   var customizePopup = document.getElementById(customizePopupId);
   customizePopup.removeAttribute("disabled");
 
-  // make sure our toolbar buttons have the correct enabled state restored to them...
-  if (this.UpdateMailToolbar != undefined) {
-    UpdateMailToolbar(focus);
-  }
-
   let toolbox = document.querySelector('[doCustomization="true"]');
   if (toolbox) {
     toolbox.removeAttribute("doCustomization");
@@ -370,24 +372,27 @@ function onViewToolbarsPopupShowing(
   let firstMenuItem = insertPoint || popup.firstElementChild;
 
   for (let toolboxId of toolboxIds) {
+    let toolbars = [];
     let toolbox = document.getElementById(toolboxId);
 
-    // We consider child nodes that have a toolbarname attribute.
-    let toolbars = Array.from(toolbox.querySelectorAll("[toolbarname]"));
+    if (toolbox) {
+      // We consider child nodes that have a toolbarname attribute.
+      toolbars = toolbars.concat(
+        Array.from(toolbox.querySelectorAll("[toolbarname]"))
+      );
+    }
 
-    // On main Window add the folder pane toolbar to the list of toolbars that
-    // can be shown and hidden.
-    if (document.getElementById("tabmail")) {
-      if (toolbox.getAttribute("id") === "mail-toolbox") {
-        if (
-          AppConstants.platform != "macosx" &&
-          document.getElementById("toolbar-menubar")
-        ) {
-          toolbars.push(document.getElementById("toolbar-menubar"));
-        }
-        if (document.getElementById("folderPaneHeader")) {
-          toolbars.push(document.getElementById("folderPaneHeader"));
-        }
+    if (
+      toolboxId == "mail-toolbox" &&
+      toolbars.every(
+        toolbar => toolbar.getAttribute("id") !== "toolbar-menubar"
+      )
+    ) {
+      if (
+        AppConstants.platform != "macosx" &&
+        document.getElementById("toolbar-menubar")
+      ) {
+        toolbars.push(document.getElementById("toolbar-menubar"));
       }
     }
 
@@ -465,14 +470,14 @@ function toMessengerWindow() {
 
 function focusOnMail(tabNo, event) {
   // this is invoked by accel-<number>
-  // if the window isn't visible or focused, make it so
   var topWindow = Services.wm.getMostRecentWindow("mail:3pane");
   if (topWindow) {
-    if (topWindow != window) {
-      topWindow.focus();
-    } else {
-      document.getElementById("tabmail").selectTabByIndex(event, tabNo);
+    topWindow.focus();
+    const tabmail = document.getElementById("tabmail");
+    if (tabmail.globalOverlay) {
+      return;
     }
+    tabmail.selectTabByIndex(event, tabNo);
   } else {
     window.open(
       "chrome://messenger/content/messenger.xhtml",
@@ -487,6 +492,8 @@ function focusOnMail(tabNo, event) {
  *
  * @param {?object} openArgs - Arguments to pass to the address book.
  *   See `externalAction` in aboutAddressBook.js for details.
+ * @returns {?Window} The address book's window global, if the address book was
+ *   opened.
  */
 async function toAddressBook(openArgs) {
   let messengerWindow = toMessengerWindow();
@@ -506,6 +513,10 @@ async function toAddressBook(openArgs) {
     });
   }
 
+  if (messengerWindow.tabmail.globalOverlay) {
+    return null;
+  }
+
   return new Promise(resolve => {
     messengerWindow.tabmail.openTab("addressBookTab", {
       onLoad(event, browser) {
@@ -561,6 +572,7 @@ function showChatTab() {
 
 /**
  * Open about:import or importDialog.xhtml.
+ *
  * @param {"start"|"app"|"addressBook"|"calendar"|"export"} [tabId] - The tab
  *  to open in about:import.
  */
@@ -626,7 +638,7 @@ function openAddonsMgr(aView) {
     let emWindow;
     let browserWindow;
 
-    let receivePong = function(aSubject, aTopic, aData) {
+    let receivePong = function (aSubject, aTopic, aData) {
       let browserWin = aSubject.browsingContext.topChromeWindow;
       if (!emWindow || browserWin == window /* favor the current window */) {
         emWindow = aSubject;
@@ -672,6 +684,15 @@ function openActivityMgr() {
     .show(window);
 }
 
+/**
+ * Open the folder properties of current folder with the quota tab selected.
+ */
+function openFolderQuota() {
+  document
+    .getElementById("tabmail")
+    .currentAbout3Pane?.folderPane.editFolder("QuotaTab");
+}
+
 function openIMAccountMgr() {
   var win = Services.wm.getMostRecentWindow("Messenger:Accounts");
   if (win) {
@@ -709,6 +730,9 @@ function openIMAccountWizard() {
 }
 
 function openSavedFilesWnd() {
+  if (window.tabmail?.globalOverlay) {
+    return Promise.resolve();
+  }
   return openContentTab("about:downloads");
 }
 
@@ -738,13 +762,13 @@ function openAboutDialog() {
     return;
   }
 
-  let features;
+  let features = "chrome,centerscreen,";
   if (AppConstants.platform == "win") {
-    features = "chrome,centerscreen,dependent";
+    features += "dependent";
   } else if (AppConstants.platform == "macosx") {
-    features = "chrome,resizable=no,minimizable=no";
+    features += "resizable=no,minimizable=no";
   } else {
-    features = "chrome,centerscreen,dependent,dialog=no";
+    features += "dependent,dialog=no";
   }
 
   window.openDialog(
@@ -855,10 +879,7 @@ function safeModeRestart() {
     {}
   );
   if (rv == 0) {
-    let environment = Cc["@mozilla.org/process/environment;1"].getService(
-      Ci.nsIEnvironment
-    );
-    environment.set("MOZ_SAFE_MODE_RESTART", "1");
+    Services.env.set("MOZ_SAFE_MODE_RESTART", "1");
     let { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
     MailUtils.restartApplication();
   }
@@ -891,7 +912,7 @@ function getMostRecentMailWindow() {
  * dots and whitespace from filename extensions.
  *
  * @param aAttachment the AttachmentInfo object
- * @return a sanitized display name for the attachment
+ * @returns a sanitized display name for the attachment
  */
 function SanitizeAttachmentDisplayName(aAttachment) {
   let displayName = aAttachment.name.trim().replace(/\s+/g, " ");
@@ -957,6 +978,11 @@ function setupDataTransfer(event, attachments) {
       new nsFlavorDataProvider(),
       index
     );
+    event.dataTransfer.mozSetDataAt(
+      "application/x-moz-file-promise-dest-filename",
+      name.replace(/(.{74}).*(.{10})$/u, "$1...$2"),
+      index
+    );
     index++;
   }
 }
@@ -973,7 +999,7 @@ function updateTroubleshootMenuItem() {
     if (appSafeMode) {
       document.l10n.setAttributes(
         appSafeMode,
-        "appmenu-help-exit-troubleshoot-mode"
+        "appmenu-help-exit-troubleshoot-mode2"
       );
     }
   }
@@ -1020,11 +1046,14 @@ nsFlavorDataProvider.prototype = {
 
       // call our code for saving attachments
       if (attachment) {
-        var name = attachment.name || attachment.displayName;
-        var destFilePath = messenger.saveAttachmentToFolder(
+        let messenger = Cc["@mozilla.org/messenger;1"].createInstance(
+          Ci.nsIMessenger
+        );
+        let name = attachment.name || attachment.displayName;
+        let destFilePath = messenger.saveAttachmentToFolder(
           attachment.contentType,
           attachment.url,
-          encodeURIComponent(name),
+          name.replace(/(.{74}).*(.{10})$/u, "$1...$2"),
           attachment.uri,
           destDirectory
         );
