@@ -2,15 +2,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from mailWindow.js */
+/* globals goUpdateCommand */ // From globalOverlay.js
 
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+var { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
-var { PlacesUtils } = ChromeUtils.import(
-  "resource://gre/modules/PlacesUtils.jsm"
+var { PlacesUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/PlacesUtils.sys.mjs"
 );
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 var gShowBiDi = false;
 
@@ -70,6 +69,7 @@ function goUpdateFindTypeMenuItems() {
 
 /**
  * Gather all descendent text under given node.
+ *
  * @param {Node} root - The root node to gather text from.
  * @returns {string} The text data under the node.
  */
@@ -82,7 +82,7 @@ function gatherTextUnder(root) {
     if (node.nodeType == Node.TEXT_NODE) {
       // Add this text to our collection.
       text += " " + node.data;
-    } else if (node instanceof HTMLImageElement) {
+    } else if (HTMLImageElement.isInstance(node)) {
       // If it has an alt= attribute, add that.
       var altText = node.getAttribute("alt");
       if (altText && altText != "") {
@@ -154,7 +154,7 @@ function goToggleToolbar(id, elementID) {
   var toolbar = document.getElementById(id);
   var element = document.getElementById(elementID);
   if (toolbar) {
-    var isHidden = toolbar.hidden;
+    const isHidden = toolbar.getAttribute("hidden") === "true";
     toolbar.setAttribute("hidden", !isHidden);
     Services.xulStore.persist(toolbar, "hidden");
     if (element) {
@@ -193,10 +193,10 @@ function openUILink(url, event) {
           },
         ],
       })
-      .catch(Cu.reportError);
-    let messenger = Cc["@mozilla.org/messenger;1"].createInstance();
-    messenger = messenger.QueryInterface(Ci.nsIMessenger);
-    messenger.launchExternalURL(url);
+      .catch(console.error);
+    Cc["@mozilla.org/uriloader/external-protocol-service;1"]
+      .getService(Ci.nsIExternalProtocolService)
+      .loadURI(Services.io.newURI(url));
   }
 }
 
@@ -286,10 +286,10 @@ function openTab(tabType, tabParams, where) {
 /**
  * Open the specified URL as a content tab (or window)
  *
- * @param {String} url - The location to open.
- * @param {String} [where="tab"] - 'tab' to open in a new tab or 'window' to
+ * @param {string} url - The location to open.
+ * @param {string} [where="tab"] - 'tab' to open in a new tab or 'window' to
  *     open in a new window
- * @param {String} [linkHandler] - See specialTabs.contentTabType.openTab.
+ * @param {string} [linkHandler] - See specialTabs.contentTabType.openTab.
  */
 function openContentTab(url, where, linkHandler) {
   return openTab("contentTab", { url, linkHandler }, where);
@@ -355,7 +355,8 @@ function openPrivacyPolicy(where) {
  */
 function openTrustedLinkIn(url, where, params = {}) {
   if (!params.triggeringPrincipal) {
-    params.triggeringPrincipal = Services.scriptSecurityManager.getSystemPrincipal();
+    params.triggeringPrincipal =
+      Services.scriptSecurityManager.getSystemPrincipal();
   }
 
   openLinkIn(url, where, params);
@@ -380,9 +381,8 @@ function openWebLinkIn(url, where, params = {}) {
   }
 
   if (!params.triggeringPrincipal) {
-    params.triggeringPrincipal = Services.scriptSecurityManager.createNullPrincipal(
-      {}
-    );
+    params.triggeringPrincipal =
+      Services.scriptSecurityManager.createNullPrincipal({});
   }
   if (params.triggeringPrincipal.isSystemPrincipal) {
     throw new Error(
@@ -424,7 +424,7 @@ function openLinkIn(url, where, openParams) {
   }
 
   if ("switchToTabHavingURI" in window) {
-    switchToTabHavingURI(url, true);
+    window.switchToTabHavingURI(url, true);
     return;
   }
 
@@ -476,7 +476,7 @@ function openLinkExternally(url) {
         },
       ],
     })
-    .catch(Cu.reportError);
+    .catch(console.error);
 
   Cc["@mozilla.org/uriloader/external-protocol-service;1"]
     .getService(Ci.nsIExternalProtocolService)
@@ -512,16 +512,14 @@ function buildHelpMenu() {
     "helpTroubleshootMode"
   );
   if (helpTroubleshootModeItem) {
-    helpTroubleshootModeItem.disabled = !Services.policies.isAllowed(
-      "safeMode"
-    );
+    helpTroubleshootModeItem.disabled =
+      !Services.policies.isAllowed("safeMode");
   }
   let appmenu_troubleshootModeItem = document.getElementById(
     "appmenu_troubleshootMode"
   );
   if (appmenu_troubleshootModeItem) {
-    appmenu_troubleshootModeItem.disabled = !Services.policies.isAllowed(
-      "safeMode"
-    );
+    appmenu_troubleshootModeItem.disabled =
+      !Services.policies.isAllowed("safeMode");
   }
 }
