@@ -12,18 +12,17 @@
 // ------------------------------
 // Constants & Enumeration Values
 
-var { DownloadUtils } = ChromeUtils.import(
-  "resource://gre/modules/DownloadUtils.jsm"
+var { DownloadUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/DownloadUtils.sys.mjs"
 );
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+var { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { UpdateUtils } = ChromeUtils.import(
-  "resource://gre/modules/UpdateUtils.jsm"
+var { UpdateUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/UpdateUtils.sys.mjs"
 );
 var { TagUtils } = ChromeUtils.import("resource:///modules/TagUtils.jsm");
 
@@ -35,14 +34,8 @@ XPCOMUtils.defineLazyServiceGetters(this, {
   gMIMEService: ["@mozilla.org/mime;1", "nsIMIMEService"],
 });
 
-XPCOMUtils.defineLazyGetter(this, "gHasWinPackageId", () => {
-  let hasWinPackageId = false;
-  try {
-    hasWinPackageId = Services.sysinfo.getProperty("hasWinPackageId");
-  } catch (_ex) {
-    // The hasWinPackageId property doesn't exist; assume it would be false.
-  }
-  return hasWinPackageId;
+XPCOMUtils.defineLazyGetter(this, "gIsPackagedApp", () => {
+  return Services.sysinfo.getProperty("isPackagedApp");
 });
 
 const TYPE_PDF = "application/pdf";
@@ -65,6 +58,7 @@ Preferences.addAll([
   { id: "general.smoothScroll", type: "bool" },
   { id: "widget.gtk.overlay-scrollbars.enabled", type: "bool", inverted: true },
   { id: "mail.fixed_width_messages", type: "bool" },
+  { id: "mail.inline_attachments", type: "bool" },
   { id: "mail.quoted_style", type: "int" },
   { id: "mail.quoted_size", type: "int" },
   { id: "mail.citation_color", type: "string" },
@@ -93,8 +87,11 @@ Preferences.addAll([
   { id: "mail.purge_threshhold_mb", type: "int" },
   { id: "browser.cache.disk.capacity", type: "int" },
   { id: "browser.cache.disk.smart_size.enabled", inverted: true, type: "bool" },
+  { id: "privacy.clearOnShutdown.cache", type: "bool" },
   { id: "layers.acceleration.disabled", type: "bool", inverted: true },
   { id: "searchintegration.enable", type: "bool" },
+  { id: "mail.tabs.drawInTitlebar", type: "bool" },
+  { id: "mail.tabs.autoHide", type: "bool" },
 ]);
 if (AppConstants.platform == "win") {
   Preferences.add({ id: "mail.minimizeToTray", type: "bool" });
@@ -283,7 +280,7 @@ var gGeneralPane = {
     // is the one that gets displayed when the user first opens the dialog,
     // the dialog might stretch too much in an attempt to fit them all in.
     // XXX Shouldn't we perhaps just set a max-height on the richlistbox?
-    var _delayedPaneLoad = function(self) {
+    var _delayedPaneLoad = function (self) {
       self._loadAppHandlerData();
       self._rebuildVisibleTypes();
       self._sortVisibleTypes();
@@ -311,8 +308,10 @@ var gGeneralPane = {
     if (AppConstants.MOZ_UPDATER) {
       this.updateReadPrefs();
       gAppUpdater = new appUpdater(); // eslint-disable-line no-global-assign
-      let updateDisabled = gAppUpdater.updateDisabledByPolicy;
-      if (gAppUpdater.updateDisabledByPackage) {
+      let updateDisabled =
+        Services.policies && !Services.policies.isAllowed("appUpdate");
+
+      if (gIsPackagedApp) {
         // When we're running inside an app package, there's no point in
         // displaying any update content here, and it would get confusing if we
         // did, because our updater is not enabled.
@@ -441,6 +440,7 @@ var gGeneralPane = {
       window.addEventListener("unload", this);
 
       Services.obs.addObserver(this, AUTO_UPDATE_CHANGED_TOPIC);
+      Services.prefs.addObserver("mailnews.tags.", this);
     }
 
     Preferences.addSyncFromPrefListener(
@@ -554,8 +554,7 @@ var gGeneralPane = {
   },
 
   browseForSoundFile() {
-    const nsIFilePicker = Ci.nsIFilePicker;
-    var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
+    var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
 
     // if we already have a sound file, then use the path for that sound file
     // as the initial path in the dialog.
@@ -572,13 +571,13 @@ var gGeneralPane = {
       document
         .getElementById("bundlePreferences")
         .getString("soundFilePickerTitle"),
-      nsIFilePicker.modeOpen
+      Ci.nsIFilePicker.modeOpen
     );
     fp.appendFilters(Ci.nsIFilePicker.filterAudio);
     fp.appendFilters(Ci.nsIFilePicker.filterAll);
 
     fp.open(rv => {
-      if (rv != nsIFilePicker.returnOK || !fp.file) {
+      if (rv != Ci.nsIFilePicker.returnOK || !fp.file) {
         return;
       }
       // convert the nsIFile into a nsIFile url
@@ -616,9 +615,8 @@ var gGeneralPane = {
       // On OS X, if there is no selected custom sound then default one will
       // be played. We keep consistency by disabling the "Play sound" checkbox
       // if the user hasn't selected a custom sound file yet.
-      document.getElementById(
-        "newMailNotification"
-      ).disabled = !soundUrlLocation;
+      document.getElementById("newMailNotification").disabled =
+        !soundUrlLocation;
       document.getElementById("playSound").disabled = !soundUrlLocation;
       // The sound type radiogroup is hidden, but we have to keep the
       // play_sound.type pref set appropriately.
@@ -631,9 +629,8 @@ var gGeneralPane = {
     document.getElementById("mailnewsStartPageUrl").disabled = !Preferences.get(
       "mailnews.start_page.enabled"
     ).value;
-    document.getElementById(
-      "browseForStartPageUrl"
-    ).disabled = !Preferences.get("mailnews.start_page.enabled").value;
+    document.getElementById("browseForStartPageUrl").disabled =
+      !Preferences.get("mailnews.start_page.enabled").value;
   },
 
   updateShowAlert() {
@@ -676,7 +673,10 @@ var gGeneralPane = {
       self.updateRemoveButton();
 
       engineList.addEventListener("command", async () => {
-        await Services.search.setDefault(engineList.selectedItem.engine);
+        await Services.search.setDefault(
+          engineList.selectedItem.engine,
+          Ci.nsISearchService.CHANGE_REASON_USER
+        );
         self.updateRemoveButton();
       });
     });
@@ -700,6 +700,7 @@ var gGeneralPane = {
 
   /**
    * Look up OpenSearch Description URL.
+   *
    * @param url - the url to use as basis for discovery
    */
   async lookupOpenSearch(url) {
@@ -773,7 +774,10 @@ var gGeneralPane = {
     );
     engineList.selectedIndex =
       engineList.firstElementChild.childElementCount - 1;
-    await Services.search.setDefault(engineList.selectedItem.engine);
+    await Services.search.setDefault(
+      engineList.selectedItem.engine,
+      Ci.nsISearchService.CHANGE_REASON_USER
+    );
     this.updateRemoveButton();
   },
 
@@ -788,7 +792,10 @@ var gGeneralPane = {
         await Services.search.removeEngine(item.engine);
         item.remove();
         engineList.selectedIndex = 0;
-        await Services.search.setDefault(engineList.selectedItem.engine);
+        await Services.search.setDefault(
+          engineList.selectedItem.engine,
+          Ci.nsISearchService.CHANGE_REASON_USER
+        );
         this.updateRemoveButton();
         break;
       }
@@ -903,7 +910,7 @@ var gGeneralPane = {
           preference.setElementValue(element);
         }
       }
-    })().catch(Cu.reportError);
+    })().catch(console.error);
   },
 
   /**
@@ -971,9 +978,8 @@ var gGeneralPane = {
       }
     }
 
-    let defaultValue = element.firstElementChild.firstElementChild.getAttribute(
-      "value"
-    );
+    let defaultValue =
+      element.firstElementChild.firstElementChild.getAttribute("value");
     let languagePref = Preferences.get("font.language.group");
     let defaultType = this._readDefaultFontTypeForLanguage(languagePref.value);
     let listPref = Preferences.get(
@@ -1136,7 +1142,7 @@ var gGeneralPane = {
    * Bug 1693576), rather than maintaining a hardcoded list of RTL locales.
    *
    * @param {string} locale
-   * @return {"ltr" | "rtl"}
+   * @returns {"ltr" | "rtl"}
    */
   getLocaleDirection(locale) {
     if (
@@ -1374,10 +1380,6 @@ var gGeneralPane = {
     if (index >= 0) {
       var itemToRemove = this.mTagListBox.getItemAtIndex(index);
       MailServices.tags.deleteKey(itemToRemove.getAttribute("value"));
-      itemToRemove.remove();
-      var numItemsInListBox = this.mTagListBox.getRowCount();
-      this.mTagListBox.selectedIndex =
-        index < numItemsInListBox ? index : numItemsInListBox - 1;
     }
   },
 
@@ -1391,7 +1393,6 @@ var gGeneralPane = {
       var args = {
         result: "",
         keyToEdit: tagElToEdit.getAttribute("value"),
-        okCallback: editTagCallback,
       };
       gSubDialog.open(
         "chrome://messenger/content/preferences/tagDialog.xhtml",
@@ -1428,13 +1429,13 @@ var gGeneralPane = {
    * state of the automatic marking feature.
    */
   updateMarkAsReadOptions() {
-    let enableRadioGroup = Preferences.get("mailnews.mark_message_read.auto")
-      .value;
+    let enableRadioGroup = Preferences.get(
+      "mailnews.mark_message_read.auto"
+    ).value;
     let autoMarkAsPref = Preferences.get("mailnews.mark_message_read.delay");
     let autoMarkDisabled = !enableRadioGroup || autoMarkAsPref.locked;
-    document.getElementById(
-      "markAsReadAutoPreferences"
-    ).disabled = autoMarkDisabled;
+    document.getElementById("markAsReadAutoPreferences").disabled =
+      autoMarkDisabled;
     document.getElementById("secondsLabel").disabled = autoMarkDisabled;
     gGeneralPane.updateMarkAsReadTextbox();
   },
@@ -1444,10 +1445,12 @@ var gGeneralPane = {
    * Mark As Read On Delay feature.
    */
   updateMarkAsReadTextbox() {
-    let radioGroupEnabled = Preferences.get("mailnews.mark_message_read.auto")
-      .value;
-    let textBoxEnabled = Preferences.get("mailnews.mark_message_read.delay")
-      .value;
+    let radioGroupEnabled = Preferences.get(
+      "mailnews.mark_message_read.auto"
+    ).value;
+    let textBoxEnabled = Preferences.get(
+      "mailnews.mark_message_read.delay"
+    ).value;
     let intervalPref = Preferences.get(
       "mailnews.mark_message_read.delay.interval"
     );
@@ -1552,8 +1555,9 @@ var gGeneralPane = {
   readSmartSizeEnabled() {
     // The smart_size.enabled preference element is inverted="true", so its
     // value is the opposite of the actual pref value
-    var disabled = Preferences.get("browser.cache.disk.smart_size.enabled")
-      .value;
+    var disabled = Preferences.get(
+      "browser.cache.disk.smart_size.enabled"
+    ).value;
     this.updateCacheSizeUI(!disabled);
   },
 
@@ -1592,9 +1596,8 @@ var gGeneralPane = {
       Preferences.get("mail.purge_threshhold_mb").locked;
 
     document.getElementById("offlineCompactFolderMin").disabled = disabled;
-    document.getElementById(
-      "offlineCompactFolderAutomatically"
-    ).disabled = disabled;
+    document.getElementById("offlineCompactFolderAutomatically").disabled =
+      disabled;
   },
 
   /**
@@ -1623,7 +1626,7 @@ var gGeneralPane = {
     if (
       AppConstants.MOZ_UPDATER &&
       (!Services.policies || Services.policies.isAllowed("appUpdate")) &&
-      !gHasWinPackageId
+      !gIsPackagedApp
     ) {
       let radiogroup = document.getElementById("updateRadioGroup");
       radiogroup.disabled = true;
@@ -1632,7 +1635,7 @@ var gGeneralPane = {
         radiogroup.value = enabled;
         radiogroup.disabled = false;
       } catch (error) {
-        Cu.reportError(error);
+        console.error(error);
       }
     }
   },
@@ -1644,7 +1647,7 @@ var gGeneralPane = {
     if (
       AppConstants.MOZ_UPDATER &&
       (!Services.policies || Services.policies.isAllowed("appUpdate")) &&
-      !gHasWinPackageId
+      !gIsPackagedApp
     ) {
       let radiogroup = document.getElementById("updateRadioGroup");
       let updateAutoValue = radiogroup.value == "true";
@@ -1653,7 +1656,7 @@ var gGeneralPane = {
         await UpdateUtils.setAppUpdateAutoEnabled(updateAutoValue);
         radiogroup.disabled = false;
       } catch (error) {
-        Cu.reportError(error);
+        console.error(error);
         await this.updateReadPrefs();
         await this.reportUpdatePrefWriteError();
         return;
@@ -1700,17 +1703,13 @@ var gGeneralPane = {
       return;
     }
 
-    let [
-      title,
-      message,
-      okButton,
-      cancelButton,
-    ] = await document.l10n.formatValues([
-      { id: "update-in-progress-title" },
-      { id: "update-in-progress-message" },
-      { id: "update-in-progress-ok-button" },
-      { id: "update-in-progress-cancel-button" },
-    ]);
+    let [title, message, okButton, cancelButton] =
+      await document.l10n.formatValues([
+        { id: "update-in-progress-title" },
+        { id: "update-in-progress-message" },
+        { id: "update-in-progress-ok-button" },
+        { id: "update-in-progress-cancel-button" },
+      ]);
 
     // Continue is the cancel button which is BUTTON_POS_1 and is set as the
     // default so pressing escape or using a platform standard method of closing
@@ -1869,7 +1868,7 @@ var gGeneralPane = {
    * object.
    *
    * @param aHandlerInfo {nsIHandlerInfo} the type to get the extensions for.
-   * @return {string} the extensions for the type
+   * @returns {string} the extensions for the type
    */
   _typeDetails(aHandlerInfo) {
     let exts = [];
@@ -1904,8 +1903,9 @@ var gGeneralPane = {
 
   /**
    * Whether or not the given handler app is valid.
+   *
    * @param aHandlerApp {nsIHandlerApp} the handler app in question
-   * @return {boolean} whether or not it's valid
+   * @returns {boolean} whether or not it's valid
    */
   isValidHandlerApp(aHandlerApp) {
     if (!aHandlerApp) {
@@ -2070,6 +2070,7 @@ var gGeneralPane = {
     window.removeEventListener("unload", this);
 
     Services.obs.removeObserver(this, AUTO_UPDATE_CHANGED_TOPIC);
+    Services.prefs.removeObserver("mailnews.tags.", this);
   },
 
   // nsISupports
@@ -2078,12 +2079,26 @@ var gGeneralPane = {
 
   // nsIObserver
 
-  async observe(aSubject, aTopic, aData) {
-    if (aTopic == AUTO_UPDATE_CHANGED_TOPIC) {
-      if (aData != "true" && aData != "false") {
-        throw new Error("Invalid preference value for app.update.auto");
+  async observe(subject, topic, data) {
+    if (topic == AUTO_UPDATE_CHANGED_TOPIC) {
+      if (data != "true" && data != "false") {
+        throw new Error(`Invalid value for app.update.auto ${data}`);
+      }
+      document.getElementById("updateRadioGroup").value = data;
+    } else if (topic == "nsPref:changed" && data.startsWith("mailnews.tags.")) {
+      let selIndex = this.mTagListBox.selectedIndex;
+      this.mTagListBox.replaceChildren();
+      this.buildTagList();
+      let numItemsInListBox = this.mTagListBox.getRowCount();
+      this.mTagListBox.selectedIndex =
+        selIndex < numItemsInListBox ? selIndex : numItemsInListBox - 1;
+      if (data.endsWith(".color") && Services.prefs.prefHasUserValue(data)) {
+        let key = data.replace(/^mailnews\.tags\./, "").replace(/\.color$/, "");
+        let color = Services.prefs.getCharPref(`mailnews.tags.${key}.color`);
+        // Add to style sheet. We simply add the new color, the rule is added
+        // at the end and will overrule the previous rule.
+        TagUtils.addTagToAllDocumentSheets(key, color);
       }
-      document.getElementById("updateRadioGroup").value = aData;
     }
   },
 
@@ -2369,7 +2384,7 @@ class HandlerRow {
           if (internalMenuItem) {
             menu.selectedItem = internalMenuItem;
           } else {
-            Cu.reportError("No menu item defined to set!");
+            console.error("No menu item defined to set!");
           }
           break;
         case Ci.nsIHandlerInfo.useSystemDefault:
@@ -2419,7 +2434,7 @@ class HandlerRow {
     aEvent.stopPropagation();
 
     var handlerApp;
-    let onSelectionDone = function() {
+    let onSelectionDone = function () {
       // Rebuild the actions menu whether the user picked an app or canceled.
       // If they picked an app, we want to add the app to the menu and select it.
       // If they canceled, we want to go back to their previous selection.
@@ -2466,18 +2481,17 @@ class HandlerRow {
         params
       );
     } else {
-      const nsIFilePicker = Ci.nsIFilePicker;
-      let fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
+      let fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
       let winTitle = gGeneralPane._prefsBundle.getString("fpTitleChooseApp");
-      fp.init(window, winTitle, nsIFilePicker.modeOpen);
-      fp.appendFilters(nsIFilePicker.filterApps);
+      fp.init(window, winTitle, Ci.nsIFilePicker.modeOpen);
+      fp.appendFilters(Ci.nsIFilePicker.filterApps);
 
       // Prompt the user to pick an app.  If they pick one, and it's a valid
       // selection, then add it to the list of possible handlers.
 
       fp.open(rv => {
         if (
-          rv == nsIFilePicker.returnOK &&
+          rv == Ci.nsIFilePicker.returnOK &&
           fp.file &&
           gGeneralPane._isValidHandlerExecutable(fp.file)
         ) {
@@ -2911,39 +2925,14 @@ function addTagCallback(aName, aColor) {
 
   // Add to style sheet.
   let key = MailServices.tags.getKeyForTag(aName);
-  TagUtils.addTagToAllDocumentSheets(key, aColor);
-
-  var item = gGeneralPane.appendTagItem(aName, key, aColor);
-  var tagListBox = document.getElementById("tagList");
+  let tagListBox = document.getElementById("tagList");
+  let item = tagListBox.querySelector(`richlistitem[value=${key}]`);
   tagListBox.ensureElementIsVisible(item);
   tagListBox.selectItem(item);
   tagListBox.focus();
   return true;
 }
 
-function editTagCallback() {
-  // update the values of the selected item
-  let tagListEl = document.getElementById("tagList");
-  let index = tagListEl.selectedIndex;
-  if (index < 0) {
-    return false;
-  }
-
-  let tagElToEdit = tagListEl.getItemAtIndex(index);
-  let key = tagElToEdit.getAttribute("value");
-  let color = MailServices.tags.getColorForKey(key);
-  // update the color and label elements
-  tagElToEdit
-    .querySelector("label")
-    .setAttribute("value", MailServices.tags.getTagForKey(key));
-  tagElToEdit.style.color = color;
-
-  // Add to style sheet. We simply add the new color, the rule is added at the
-  // end and will overrule the previous rule.
-  TagUtils.addTagToAllDocumentSheets(key, color);
-  return true;
-}
-
 Preferences.get("mailnews.start_page.enabled").on(
   "change",
   gGeneralPane.updateStartPage
