@@ -7,7 +7,6 @@
 
 /* global GetEnigmailSvc, EnigRevokeKey */
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
@@ -15,9 +14,18 @@ var { RNP } = ChromeUtils.import("chrome://openpgp/content/modules/RNP.jsm");
 var { EnigmailKey } = ChromeUtils.import(
   "chrome://openpgp/content/modules/key.jsm"
 );
+var { EnigmailDialog } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/dialog.jsm"
+);
 var { EnigmailKeyRing } = ChromeUtils.import(
   "chrome://openpgp/content/modules/keyRing.jsm"
 );
+var { EnigmailKeyserverURIs } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/keyserverUris.jsm"
+);
+var { EnigmailKeyServer } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/keyserver.jsm"
+);
 var { EnigmailCryptoAPI } = ChromeUtils.import(
   "chrome://openpgp/content/modules/cryptoAPI.jsm"
 );
@@ -37,6 +45,7 @@ var gSignMessages = null;
 var gRequireEncrypt = null;
 var gDoNotEncrypt = null;
 var gAttachKey = null;
+var gSendAutocryptHeaders = null;
 var gEncryptSubject = null;
 var gEncryptDrafts = null;
 
@@ -53,11 +62,45 @@ var gTechPrefSMIME = null;
 
 function onInit() {
   initE2EEncryption(gIdentity);
+  Services.prefs.addObserver("mail.e2ee.auto_enable", autoEncryptPrefObserver);
+  Services.prefs.addObserver("mail.e2ee.auto_disable", autoEncryptPrefObserver);
 }
 
+window.addEventListener("unload", function () {
+  Services.prefs.removeObserver(
+    "mail.e2ee.auto_enable",
+    autoEncryptPrefObserver
+  );
+  Services.prefs.removeObserver(
+    "mail.e2ee.auto_disable",
+    autoEncryptPrefObserver
+  );
+});
+
 let gDisableEncryption;
 let gEnableEncryption;
 
+var autoEncryptPrefObserver = {
+  observe(subject, topic, prefName) {
+    if (topic == "nsPref:changed") {
+      if (
+        prefName == "mail.e2ee.auto_enable" ||
+        prefName == "mail.e2ee.auto_disable"
+      ) {
+        updateAutoEncryptRelated();
+      }
+    }
+  },
+};
+
+function updateAutoEncryptRelated() {
+  if (Services.prefs.getBoolPref("mail.e2ee.auto_enable")) {
+    document.getElementById("encryptionChoices").hidden = true;
+  } else {
+    document.getElementById("encryptionChoices").hidden = false;
+  }
+}
+
 async function initE2EEncryption(identity) {
   // Initialize all of our elements based on the current identity values...
   gEncryptionCertName = document.getElementById(kEncryptionCertPref);
@@ -67,6 +110,7 @@ async function initE2EEncryption(identity) {
   gDisableEncryption = document.getElementById("disable_encryption");
   gEnableEncryption = document.getElementById("enable_encryption");
   gAttachKey = document.getElementById("identity_attach_key");
+  gSendAutocryptHeaders = document.getElementById("identity_autocrypt_headers");
   gEncryptSubject = document.getElementById("identity_encrypt_subject");
   gEncryptDrafts = document.getElementById("identity_encrypt_drafts");
 
@@ -98,6 +142,8 @@ async function initE2EEncryption(identity) {
     gSignMessages.disabled = true;
 
     gAttachKey.checked = gAccount.defaultIdentity.attachPgpKey;
+    gSendAutocryptHeaders.checked =
+      gAccount.defaultIdentity.sendAutocryptHeaders;
     gEncryptSubject.checked = gAccount.defaultIdentity.protectSubject;
     gEncryptDrafts.checked = gAccount.defaultIdentity.autoEncryptDrafts;
     gSignMessages.checked = gAccount.defaultIdentity.signMail;
@@ -116,6 +162,7 @@ async function initE2EEncryption(identity) {
 
     gSignMessages.checked = identity.signMail;
     gAttachKey.checked = identity.attachPgpKey;
+    gSendAutocryptHeaders.checked = identity.sendAutocryptHeaders;
     gEncryptSubject.checked = identity.protectSubject;
     gEncryptDrafts.checked = identity.autoEncryptDrafts;
 
@@ -124,6 +171,8 @@ async function initE2EEncryption(identity) {
     enableSigningControls(enableSig);
   }
 
+  updateAutoEncryptRelated();
+
   // Always start with enabling select buttons.
   // This will keep the visibility of buttons in a sane state as user
   // jumps from security panel of one account to another.
@@ -256,6 +305,7 @@ function saveE2EEncryptionSettings(identity) {
   identity.setCharAttribute("signing_cert_dbkey", gSignCertName.dbKey);
 
   identity.attachPgpKey = gAttachKey.checked;
+  identity.sendAutocryptHeaders = gSendAutocryptHeaders.checked;
   identity.protectSubject = gEncryptSubject.checked;
   identity.autoEncryptDrafts = gEncryptDrafts.checked;
 }
@@ -450,14 +500,12 @@ function enableSigningControls(do_enable) {
 
 function enableSelectButtons() {
   gSignCertName.disabled = !gSignCertName.value;
-  document.getElementById(
-    "signingCertClearButton"
-  ).disabled = !gSignCertName.value;
+  document.getElementById("signingCertClearButton").disabled =
+    !gSignCertName.value;
 
   gEncryptionCertName.disabled = !gEncryptionCertName.value;
-  document.getElementById(
-    "encryptionCertClearButton"
-  ).disabled = !gEncryptionCertName.value;
+  document.getElementById("encryptionCertClearButton").disabled =
+    !gEncryptionCertName.value;
 }
 
 function smimeClearCert(smime_cert) {
@@ -591,7 +639,7 @@ async function keyExternalSuccess(keyId) {
  */
 function useOpenPGPKey(keyId) {
   // Rebuild the UI so that any new keys are listed.
-  gKeyId = keyId;
+  gKeyId = keyId.toUpperCase();
 
   // Update the identity with the key obtained from the key wizard.
   gIdentity.setUnicharAttribute("openpgp_key_id", keyId || "");
@@ -629,7 +677,7 @@ function closeNotification() {
 }
 
 /**
- * Refresh the UI on init or after a successful OpenPGP Key generation.
+ * Refresh the UI on init or after a successful OpenPGP key generation.
  */
 async function reloadOpenPgpUI() {
   let result = {};
@@ -732,8 +780,8 @@ async function reloadOpenPgpUI() {
     return b.keyCreated - a.keyCreated;
   });
 
-  // If the user has an external Key saved, and the pref is TRUE,
-  // we show it on top of the list.
+  // If the user has an external key saved, and the allow_external_gnupg
+  // pref is true, we show it on top of the list.
   if (externalKey) {
     let container = document.createXULElement("vbox");
     container.id = `openPgpOption${externalKey}`;
@@ -823,7 +871,10 @@ async function reloadOpenPgpUI() {
       if (Math.round(Date.now() / 1000) > key.expiryTime) {
         // Has expired.
         dateContainer.classList.add("key-expired");
-        dateIcon.setAttribute("src", "chrome://global/skin/icons/warning.svg");
+        dateIcon.setAttribute(
+          "src",
+          "chrome://messenger/skin/icons/new/compact/warning.svg"
+        );
         // Sets the title attribute.
         // The alt attribute is not set because the accessible name is already
         // set by the title.
@@ -844,7 +895,7 @@ async function reloadOpenPgpUI() {
           dateContainer.classList.add("key-is-expiring");
           dateIcon.setAttribute(
             "src",
-            "chrome://messenger/skin/icons/info.svg"
+            "chrome://messenger/skin/icons/new/compact/info.svg"
           );
           // Sets the title attribute.
           // The alt attribute is not set because the accessible name is already
@@ -868,6 +919,30 @@ async function reloadOpenPgpUI() {
     dateContainer.appendChild(description);
     dateContainer.appendChild(dateButton);
 
+    let publishContainer = null;
+
+    // If this key is the currently selected key, suggest publishing.
+    if (key.keyId == gKeyId) {
+      publishContainer = document.createXULElement("hbox");
+      publishContainer.setAttribute("align", "center");
+
+      let publishButton = document.createElement("button");
+      document.l10n.setAttributes(publishButton, "openpgp-key-publish");
+      publishButton.addEventListener("click", () => {
+        amE2eUploadKey(key);
+      });
+      publishButton.classList.add("button-small");
+
+      let description = document.createXULElement("description");
+      document.l10n.setAttributes(
+        description,
+        "openpgp-suggest-publishing-key"
+      );
+
+      publishContainer.appendChild(description);
+      publishContainer.appendChild(publishButton);
+    }
+
     let hiddenContainer = document.createXULElement("vbox");
     hiddenContainer.classList.add(
       "content-blocking-extra-information",
@@ -882,7 +957,7 @@ async function reloadOpenPgpUI() {
     let fingerprintImage = document.createElement("img");
     fingerprintImage.setAttribute(
       "src",
-      "chrome://messenger/skin/icons/fingerprint.svg"
+      "chrome://messenger/skin/icons/new/compact/fingerprint.svg"
     );
     fingerprintImage.setAttribute("alt", "");
 
@@ -913,7 +988,7 @@ async function reloadOpenPgpUI() {
     let createdImage = document.createElement("img");
     createdImage.setAttribute(
       "src",
-      "chrome://messenger/skin/shared/preferences/calendar.svg"
+      "chrome://messenger/skin/icons/new/compact/calendar.svg"
     );
     createdImage.setAttribute("alt", "");
 
@@ -987,7 +1062,7 @@ async function reloadOpenPgpUI() {
     document.l10n.setAttributes(backupItem, "openpgp-key-backup-key");
     backupItem.addEventListener("command", event => {
       event.stopPropagation();
-      openPgpExportSecretKey(`0x${key.keyId}`, `0x${key.fpr}`);
+      openPgpExportSecretKey(`0x${key.keyId}`, `${key.fpr}`);
     });
 
     let revokeItem = document.createXULElement("menuitem");
@@ -1021,6 +1096,9 @@ async function reloadOpenPgpUI() {
     hiddenContainer.appendChild(btnContainer);
 
     indent.appendChild(dateContainer);
+    if (publishContainer) {
+      indent.appendChild(publishContainer);
+    }
     indent.appendChild(hiddenContainer);
 
     container.appendChild(box);
@@ -1079,7 +1157,7 @@ function enigmailKeyDetails(keyId) {
 /**
  * Delete an OpenPGP Key.
  *
- * @param {Object} key - The selected OpenPGP Key.
+ * @param {object} key - The selected OpenPGP Key.
  */
 async function enigmailDeleteKey(key) {
   // Interrupt if the selected key is currently being used.
@@ -1115,7 +1193,7 @@ async function enigmailDeleteKey(key) {
 /**
  * Revoke the selected OpenPGP Key.
  *
- * @param {Object} key - The selected OpenPGP Key.
+ * @param {object} key - The selected OpenPGP Key.
  */
 async function openPgpRevokeKey(key) {
   // Interrupt if the selected key is currently being used.
@@ -1129,7 +1207,7 @@ async function openPgpRevokeKey(key) {
     return;
   }
 
-  EnigRevokeKey(key, function(success) {
+  EnigRevokeKey(key, function (success) {
     if (success) {
       document.l10n.setAttributes(
         document.getElementById("openPgpNotificationDescription"),
@@ -1143,11 +1221,25 @@ async function openPgpRevokeKey(key) {
   });
 }
 
+async function amE2eUploadKey(key) {
+  let ks = EnigmailKeyserverURIs.getUploadKeyServer();
+
+  let ok = await EnigmailKeyServer.upload(key.keyId, ks);
+  let msg = await document.l10n.formatValue(
+    ok ? "openpgp-key-publish-ok" : "openpgp-key-publish-fail",
+    {
+      keyserver: ks,
+    }
+  );
+
+  EnigmailDialog.alert(null, msg);
+}
+
 /**
  * Open the subdialog to enable the user to edit the expiration date of the
  * selected OpenPGP Key.
  *
- * @param {Object} key - The selected OpenPGP Key.
+ * @param {object} key - The selected OpenPGP Key.
  */
 async function enigmailEditKeyDate(key) {
   if (!key.iSimpleOneSubkeySameExpiry()) {
@@ -1227,7 +1319,10 @@ function updateUIForSelectedOpenPgpKey() {
   if (gKeyId) {
     let key = EnigmailKeyRing.getKeyById(gKeyId, true);
     if (key?.expiryTime && Math.round(Date.now() / 1000) > key.expiryTime) {
-      image.setAttribute("src", "chrome://messenger/skin/icons/stop.svg");
+      image.setAttribute(
+        "src",
+        "chrome://messenger/skin/icons/new/compact/close.svg"
+      );
       image.classList.add("status-error");
       document.l10n.setAttributes(
         statusLabel,
@@ -1235,7 +1330,10 @@ function updateUIForSelectedOpenPgpKey() {
         { key: `0x${gKeyId}` }
       );
     } else {
-      image.setAttribute("src", "chrome://global/skin/icons/check.svg");
+      image.setAttribute(
+        "src",
+        "chrome://messenger/skin/icons/new/compact/check.svg"
+      );
       image.classList.add("status-success");
       document.l10n.setAttributes(
         statusLabel,
@@ -1421,7 +1519,7 @@ async function openPgpExportSecretKey(keyId, keyFpr) {
  *
  * @param {string} password - The declared password to protect the keys.
  * @param {Array} fprArray - The array of fingerprint of the selected keys.
- * @param {Object} file - The file where the keys should be saved.
+ * @param {object} file - The file where the keys should be saved.
  * @param {boolean} confirmed - If the password was properly typed in the prompt.
  */
 async function exportSecretKey(password, fprArray, file, confirmed = false) {
@@ -1432,6 +1530,14 @@ async function exportSecretKey(password, fprArray, file, confirmed = false) {
   }
 
   let backupKeyBlock = await RNP.backupSecretKeys(fprArray, password);
+  if (!backupKeyBlock) {
+    Services.prompt.alert(
+      null,
+      await document.l10n.formatValue("save-keys-failed")
+    );
+    return;
+  }
+
   await IOUtils.writeUTF8(file.path, backupKeyBlock)
     .then(() => {
       document.l10n.setAttributes(
