@@ -6,19 +6,18 @@
 
 "use strict";
 
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+var { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
 
 function onLoad() {
   var dlg = document.getElementById("enigmailMsgBox");
-  dlg.getButton("help").setAttribute("hidden", "true");
   dlg.getButton("cancel").setAttribute("hidden", "true");
   dlg.getButton("extra1").setAttribute("hidden", "true");
   dlg.getButton("extra2").setAttribute("hidden", "true");
 
-  document.getElementById("filler").maxWidth = screen.availWidth - 50;
-  //dlg.maxHeight = screen.availHeight;
+  document.getElementById("filler").style.maxWidth =
+    screen.availWidth - 50 + "px";
 
   let args = window.arguments[0];
   let msgtext = args.msgtext;
@@ -102,7 +101,7 @@ function resizeDlg() {
       "height",
       boxHeight - btnHeight - (dlgHeight - availHeight)
     );
-    window.outerHeight = availHeight;
+    window.resizeTo(window.outerWidth, availHeight);
   }
 }
 
@@ -176,6 +175,6 @@ function onKeyPress(event) {
   }
 }
 
-document.addEventListener("dialogaccept", function(event) {
+document.addEventListener("dialogaccept", function (event) {
   dlgClose("accept");
 });
