@@ -6,20 +6,11 @@
 
 "use strict";
 
-/*globally available Thunderbird variables/object/functions: */
-/*global gMsgCompose: false, gComposeNotification: false */
-/*global UpdateAttachmentBucket: false, gContentChanged: true */
-/*global AddAttachments: false, AddAttachment: false, ChangeAttachmentBucketVisibility: false, GetResourceFromUri: false */
-/*global Recipients2CompFields: false, Attachments2CompFields: false, gWindowLocked: false */
-/*global CommandUpdate_MsgCompose: false, gSMFields: false, setSecuritySettings: false, getCurrentAccountKey: false */
-/*global Sendlater3Composing: false, gCurrentIdentity: false, showMessageComposeSecurityStatus: false */
-/*global gSendEncrypted: true, gOptionalEncryption: true, gSendSigned: true, gSelectedTechnologyIsPGP: true */
-/*global gIsRelatedToEncryptedOriginal: true, gIsRelatedToSignedOriginal: true, gAttachMyPublicPGPKey: true */
-/*global gEncryptSubject: true, setEncSigStatusUI: false, gEncryptedURIService: false */
-/* global updateE2eeOptions: true */
-/* global gAttachmentBucket: true */
+/* import-globals-from ../../../../components/compose/content/MsgComposeCommands.js */
+/* import-globals-from ../../../../components/compose/content/addressingWidgetOverlay.js */
+/* global MsgAccountManager */
+/* global gCurrentIdentity */
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
@@ -88,9 +79,7 @@ var l10nOpenPGP = new Localization(["messenger/openpgp/openpgp.ftl"]);
 // const kEncryptionPolicy_IfPossible = 1;
 var kEncryptionPolicy_Always = 2;
 
-if (!Enigmail) {
-  var Enigmail = {};
-}
+var Enigmail = {};
 
 const IOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
 const LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
@@ -150,7 +139,7 @@ Enigmail.msg = {
       true
     );
 
-    await OpenPGPAlias.load().catch(Cu.reportError);
+    await OpenPGPAlias.load().catch(console.error);
 
     Enigmail.msg.composeOpen();
     //Enigmail.msg.processFinalState();
@@ -353,12 +342,7 @@ Enigmail.msg = {
         msgUri = this.getOriginalMsgUri();
       }
       if (msgUri) {
-        let messenger = Cc["@mozilla.org/messenger;1"].getService(
-          Ci.nsIMessenger
-        );
-        return messenger
-          .messageServiceFromURI(msgUri)
-          .messageURIToMsgHdr(msgUri);
+        return gMessenger.msgHdrFromURI(msgUri);
       }
     } catch (ex) {
       // See also bug 1635648
@@ -386,11 +370,7 @@ Enigmail.msg = {
           if (self.getSavedDraftOptions(mimeMsg)) {
             obtainedDraftFlagsObj.value = true;
           }
-          if (self.draftSubjectEncrypted) {
-            self.setOriginalSubject(msgHdr.subject, msgHdr.flags, false);
-          }
-        } else if (EnigmailURIs.isEncryptedUri(msgUri)) {
-          self.setOriginalSubject(msgHdr.subject, msgHdr.flags, false);
+          updateEncryptionDependencies();
         }
       }
     } catch (ex) {
@@ -425,131 +405,102 @@ Enigmail.msg = {
 
     if (stat.substr(0, 1) == "N") {
       switch (Number(stat.substr(1, 1))) {
-        case EnigmailConstants.ENIG_UNDEF:
+        case 2:
+          // treat as "user decision to enable encryption, disable auto"
+          gUserTouchedSendEncrypted = true;
+          gSendEncrypted = true;
+          updateEncryptionDependencies();
           break;
-        case EnigmailConstants.ENIG_ALWAYS:
-          updateE2eeOptions(true);
+        case 0:
+          // treat as "user decision to disable encryption, disable auto"
+          gUserTouchedSendEncrypted = true;
+          gSendEncrypted = false;
+          updateEncryptionDependencies();
           break;
-        case EnigmailConstants.ENIG_NEVER:
+        case 1:
         default:
-          updateE2eeOptions(false);
+          // treat as "no user decision, automatic mode"
           break;
       }
-      gOptionalEncryption = false;
 
       switch (Number(stat.substr(2, 1))) {
-        case EnigmailConstants.ENIG_UNDEF:
-          break;
-        case EnigmailConstants.ENIG_ALWAYS:
+        case 2:
           gSendSigned = true;
+          gUserTouchedSendSigned = true;
           break;
-        case EnigmailConstants.ENIG_NEVER:
-        default:
+        case 0:
+          gUserTouchedSendSigned = true;
           gSendSigned = false;
           break;
+        case 1:
+        default:
+          // treat as "no user decision, automatic mode, based on encryption or other prefs"
+          break;
       }
 
       switch (Number(stat.substr(3, 1))) {
-        case EnigmailConstants.ENIG_UNDEF:
+        case 1:
           break;
         case EnigmailConstants.ENIG_FORCE_SMIME:
+          // 3
           gSelectedTechnologyIsPGP = false;
           break;
-        case EnigmailConstants.ENIG_ALWAYS: // pgp/mime
-        case EnigmailConstants.ENIG_NEVER: // inline
+        case 2: // pgp/mime
+        case 0: // inline
         default:
           gSelectedTechnologyIsPGP = true;
           break;
       }
 
-      gAttachMyPublicPGPKey = stat.substr(4, 1) == "1";
+      switch (Number(stat.substr(4, 1))) {
+        case 1:
+          gUserTouchedAttachMyPubKey = true;
+          gAttachMyPublicPGPKey = true;
+          break;
+        case 2:
+          gUserTouchedAttachMyPubKey = false;
+          break;
+        case 0:
+        default:
+          gUserTouchedAttachMyPubKey = true;
+          gAttachMyPublicPGPKey = false;
+          break;
+      }
 
-      Enigmail.msg.draftSubjectEncrypted = stat.substr(5, 1) == "1";
+      switch (Number(stat.substr(4, 1))) {
+        case 1:
+          gUserTouchedAttachMyPubKey = true;
+          gAttachMyPublicPGPKey = true;
+          break;
+        case 2:
+          gUserTouchedAttachMyPubKey = false;
+          break;
+        case 0:
+        default:
+          gUserTouchedAttachMyPubKey = true;
+          gAttachMyPublicPGPKey = false;
+          break;
+      }
+
+      switch (Number(stat.substr(5, 1))) {
+        case 1:
+          gUserTouchedEncryptSubject = true;
+          gEncryptSubject = true;
+          break;
+        case 2:
+          gUserTouchedEncryptSubject = false;
+          break;
+        case 0:
+        default:
+          gUserTouchedEncryptSubject = true;
+          gEncryptSubject = false;
+          break;
+      }
     }
     //Enigmail.msg.setOwnKeyStatus();
     return true;
   },
 
-  /**
-   * Updates the subject displayed in the #msgSubject element of the compose
-   * window. Sets prefixes such as "Re" depending on the value gMsgCompose.type.
-   *
-   * @param {string} subject The subject for the message.
-   * @param {number} flags The nsIMsgHdr flags for the message, used to detect
-   *   HasRe on drafts.
-   * @param {boolean} forceSetting If true, the subject is always updated
-   *   otherwise it is only updated for the following nsIMsgCompTypes: Draft,
-   *   Template,EditTemplate,ForwardInline,ForwardAttachement or EditAsNew.
-   */
-  setOriginalSubject(subject, flags, forceSetting) {
-    const CT = Ci.nsIMsgCompType;
-    let subjElem = document.getElementById("msgSubject");
-    let prefix = "";
-    let isReply = false;
-
-    if (!subjElem) {
-      return;
-    }
-
-    switch (gMsgCompose.type) {
-      case CT.ForwardInline:
-      case CT.ForwardAsAttachment:
-        prefix =
-          Services.prefs.getStringPref("mail.forward_subject_prefix") + ": ";
-        break;
-      case CT.Draft:
-        isReply = Boolean(flags & Ci.nsMsgMessageFlags.HasRe);
-        break;
-      case CT.Reply:
-      case CT.ReplyAll:
-      case CT.ReplyToSender:
-      case CT.ReplyToGroup:
-      case CT.ReplyToSenderAndGroup:
-      case CT.ReplyToList:
-        isReply = true;
-    }
-
-    if (isReply) {
-      if (!subject.startsWith("Re: ")) {
-        prefix = "Re: ";
-      }
-    }
-
-    let doSetSubject = forceSetting;
-    switch (gMsgCompose.type) {
-      case CT.Draft:
-      case CT.Template:
-      case CT.EditTemplate:
-      case CT.ForwardInline:
-      case CT.ForwardAsAttachment:
-      case CT.EditAsNew:
-        doSetSubject = true;
-        break;
-    }
-
-    if (doSetSubject) {
-      subject = EnigmailData.convertToUnicode(subject, "UTF-8");
-      subject = jsmime.headerparser.decodeRFC2047Words(subject, "utf-8");
-
-      if (subjElem.value == "Re: " + subject) {
-        return;
-      }
-
-      let newSubj = prefix + subject;
-
-      if (
-        newSubj !== subjElem.value ||
-        newSubj !== gMsgCompose.compFields.subject
-      ) {
-        gMsgCompose.compFields.subject = newSubj;
-        subjElem.value = newSubj;
-        if (typeof subjElem.oninput === "function") {
-          subjElem.oninput();
-        }
-      }
-    }
-  },
-
   composeOpen() {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.composeOpen\n"
@@ -637,7 +588,8 @@ Enigmail.msg = {
             EnigmailLog.DEBUG(
               "originalMsgURI=" + gMsgCompose.originalMsgURI + "\n"
             );
-            updateE2eeOptions(true);
+            gSendEncrypted = true;
+            updateEncryptionDependencies();
             gSelectedTechnologyIsPGP = true;
             useEncryptionUnlessWeHaveDraftInfo = false;
             usePGPUnlessWeKnowOtherwise = false;
@@ -648,7 +600,8 @@ Enigmail.msg = {
       }
 
       if (useEncryptionUnlessWeHaveDraftInfo) {
-        updateE2eeOptions(true);
+        gSendEncrypted = true;
+        updateEncryptionDependencies();
       }
       if (gSendEncrypted && !obtainedDraftFlagsObj.value) {
         gSendSigned = true;
@@ -682,6 +635,8 @@ Enigmail.msg = {
       UpdateAttachmentBucket(bucketList.hasChildNodes());
     }
 
+    this.warnUserIfSenderKeyExpired();
+
     //this.processFinalState();
     if (selectedElement) {
       selectedElement.focus();
@@ -982,6 +937,7 @@ Enigmail.msg = {
 
   /**
     key function to process the final encrypt/sign/pgpmime state from all settings
+   *
     @param sendFlags: contains the sendFlags if the message is really processed. Optional, can be null
       - uses as INPUT:
          - this.sendMode
@@ -1062,8 +1018,8 @@ Enigmail.msg = {
 
     if (inputObj.resetDefaults) {
       // reset everything to defaults
-      this.encryptForced = EnigmailConstants.ENIG_UNDEF;
-      this.signForced = EnigmailConstants.ENIG_UNDEF;
+      this.encryptForced = 1;
+      this.signForced = 1;
     }
     else {
       if (this.signForced != inputObj.sign) {
@@ -1106,20 +1062,32 @@ Enigmail.msg = {
     var draftStatus = "N";
 
     // Encryption:
-    // ENIG_ALWAYS == 2 -> required/enabled
-    // ENIG_NEVER == 0 -> disabled
-    // (gOptionalEncryption not yet handled)
-    draftStatus += gSendEncrypted ? "2" : "0";
+    // 2 -> required/enabled
+    // 0 -> disabled
 
-    // Signing:
-    // ENIG_ALWAYS == 2 -> enabled
-    // ENIG_NEVER == 0 -> disabled
-    draftStatus += gSendSigned ? "2" : "0";
+    if (!gUserTouchedSendEncrypted && !gIsRelatedToEncryptedOriginal) {
+      // After opening draft, it's allowed to use automatic decision.
+      draftStatus += "1";
+    } else {
+      // After opening draft, use the same state that is set now.
+      draftStatus += gSendEncrypted ? "2" : "0";
+    }
+
+    if (!gUserTouchedSendSigned) {
+      // After opening draft, it's allowed to use automatic decision.
+      draftStatus += "1";
+    } else {
+      // After opening draft, use the same state that is set now.
+      // Signing:
+      // 2 -> enabled
+      // 0 -> disabled
+      draftStatus += gSendSigned ? "2" : "0";
+    }
 
     // MIME/technology
     // ENIG_FORCE_SMIME == 3 -> S/MIME
     // ENIG_FORCE_ALWAYS == 2 -> PGP/MIME
-    // ENIG_NEVER == 0 -> PGP inline
+    // 0 -> PGP inline
     if (gSelectedTechnologyIsPGP) {
       // inline signing currently not implemented
       draftStatus += "2";
@@ -1127,15 +1095,23 @@ Enigmail.msg = {
       draftStatus += "3";
     }
 
-    draftStatus += gAttachMyPublicPGPKey ? "1" : "0";
+    if (!gUserTouchedAttachMyPubKey) {
+      draftStatus += "2";
+    } else {
+      draftStatus += gAttachMyPublicPGPKey ? "1" : "0";
+    }
 
-    draftStatus += gSendEncrypted && gEncryptSubject ? "1" : "0";
+    if (!gUserTouchedEncryptSubject) {
+      draftStatus += "2";
+    } else {
+      draftStatus += gSendEncrypted && gEncryptSubject ? "1" : "0";
+    }
 
     this.setAdditionalHeader("X-Enigmail-Draft-Status", draftStatus);
   },
 
   getSenderUserId() {
-    let keyId = gCurrentIdentity.getUnicharAttribute("openpgp_key_id");
+    let keyId = gCurrentIdentity?.getUnicharAttribute("openpgp_key_id");
     return keyId ? "0x" + keyId : null;
   },
 
@@ -1461,10 +1437,6 @@ Enigmail.msg = {
   getEncryptionFlags() {
     let f = 0;
 
-    console.debug(
-      `in getEncryptionFlags, gSendEncrypted=${gSendEncrypted}, gSendSigned=${gSendSigned}`
-    );
-
     if (gSendEncrypted) {
       f |= EnigmailConstants.SEND_ENCRYPTED;
     } else {
@@ -1478,7 +1450,9 @@ Enigmail.msg = {
     }
 
     if (gSendEncrypted && gSendSigned) {
-      f |= EnigmailConstants.SEND_TWO_MIME_LAYERS;
+      if (Services.prefs.getBoolPref("mail.openpgp.separate_mime_layers")) {
+        f |= EnigmailConstants.SEND_TWO_MIME_LAYERS;
+      }
     }
 
     if (gSendEncrypted && gEncryptSubject) {
@@ -2188,9 +2162,18 @@ Enigmail.msg = {
   // called just before sending
   modifyCompFields() {
     try {
-      if (Enigmail.msg.isEnigmailEnabledForIdentity()) {
-        this.setAutocryptHeader();
+      if (
+        !Enigmail.msg.isEnigmailEnabledForIdentity() ||
+        !gCurrentIdentity.sendAutocryptHeaders
+      ) {
+        return;
       }
+      if ((gSendSigned || gSendEncrypted) && !gSelectedTechnologyIsPGP) {
+        // If we're sending an S/MIME message, we don't want to send
+        // the OpenPGP autocrypt header.
+        return;
+      }
+      this.setAutocryptHeader();
     } catch (ex) {
       EnigmailLog.writeException(
         "enigmailMsgComposeOverlay.js: Enigmail.msg.modifyCompFields",
@@ -2207,10 +2190,6 @@ Enigmail.msg = {
   },
 
   setAutocryptHeader() {
-    if (!gAttachMyPublicPGPKey) {
-      return;
-    }
-
     let senderKeyId = gCurrentIdentity.getUnicharAttribute("openpgp_key_id");
     if (!senderKeyId) {
       return;
@@ -2241,10 +2220,6 @@ Enigmail.msg = {
       "enigmailMsgComposeOverlay.js: Enigmail.msg.sendMessageListener\n"
     );
 
-    if (!gSelectedTechnologyIsPGP) {
-      return;
-    }
-
     let msgcomposeWindow = document.getElementById("msgcomposeWindow");
     let sendMsgType = Number(msgcomposeWindow.getAttribute("msgtype"));
 
@@ -2254,11 +2229,15 @@ Enigmail.msg = {
         sendMsgType == Ci.nsIMsgCompDeliverMode.AutoSaveAsDraft
       )
     ) {
+      this.modifyCompFields();
+      if (!gSelectedTechnologyIsPGP) {
+        return;
+      }
+
       this.sendProcess = true;
       //let bc = document.getElementById("enigmail-bc-sendprocess");
 
       try {
-        this.modifyCompFields();
         const cApi = EnigmailCryptoAPI();
         let encryptResult = cApi.sync(this.prepareSendMsg(sendMsgType));
         if (!encryptResult) {
@@ -2267,7 +2246,7 @@ Enigmail.msg = {
           event.stopPropagation();
         }
       } catch (ex) {
-        Cu.reportError("GenericSendMessage FAILED: " + ex);
+        console.error("GenericSendMessage FAILED: " + ex);
         this.resetUpdatedFields();
         event.preventDefault();
         event.stopPropagation();
@@ -2436,8 +2415,8 @@ Enigmail.msg = {
       // automatic enabling encryption currently depends on
       // adjustSignEncryptAfterIdentityChanged to be always reached
       gIsRelatedToEncryptedOriginal = true;
-      updateE2eeOptions(gSendEncrypted);
-      gSendSigned = true;
+      gSendEncrypted = true;
+      updateEncryptionDependencies();
     }
     //}
 
@@ -2682,12 +2661,69 @@ Enigmail.msg = {
     return 0;
   },
 
+  isSenderKeyExpired() {
+    const senderKeyId = this.getSenderUserId();
+
+    if (senderKeyId) {
+      const key = EnigmailKeyRing.getKeyById(senderKeyId);
+      return key?.expiryTime && Math.round(Date.now() / 1000) > key.expiryTime;
+    }
+
+    return false;
+  },
+
+  removeNotificationIfPresent(name) {
+    const notif = gComposeNotification.getNotificationWithValue(name);
+    if (notif) {
+      gComposeNotification.removeNotification(notif);
+    }
+  },
+
+  warnUserThatSenderKeyExpired() {
+    const label = {
+      "l10n-id": "openpgp-selection-status-error",
+      "l10n-args": { key: this.getSenderUserId() },
+    };
+
+    const buttons = [
+      {
+        "l10n-id": "settings-context-open-account-settings-item2",
+        callback() {
+          MsgAccountManager(
+            "am-e2e.xhtml",
+            MailServices.accounts.getServersForIdentity(gCurrentIdentity)[0]
+          );
+          Services.wm.getMostRecentWindow("mail:3pane")?.focus();
+          return true;
+        },
+      },
+    ];
+
+    gComposeNotification.appendNotification(
+      "openpgpSenderKeyExpired",
+      {
+        label,
+        priority: gComposeNotification.PRIORITY_WARNING_MEDIUM,
+      },
+      buttons
+    );
+  },
+
+  warnUserIfSenderKeyExpired() {
+    if (!this.isSenderKeyExpired()) {
+      this.removeNotificationIfPresent("openpgpSenderKeyExpired");
+      return;
+    }
+
+    this.warnUserThatSenderKeyExpired();
+  },
+
   /**
    * Display a notification to the user at the bottom of the window
    *
-   * @param priority: Number    - Priority of the message [1 = high (error) ... 3 = low (info)]
-   * @param msgText: String     - Text to be displayed in notification bar
-   * @param messageId: String   - Unique message type identification
+   * @param priority: Number - Priority of the message [1 = high (error) ... 3 = low (info)]
+   * @param msgText: String - Text to be displayed in notification bar
+   * @param messageId: String - Unique message type identification
    * @param detailsText: String - optional text to be displayed by clicking on "Details" button.
    *                              if null or "", then the Detail button will no be displayed.
    */
@@ -2887,14 +2923,8 @@ Enigmail.composeStateListener = {
     */
 
     if (isEditable && !isEmpty) {
-      let msgHdr = Enigmail.msg.getMsgHdr();
-      if (msgHdr) {
-        Enigmail.msg.setOriginalSubject(msgHdr.subject, msgHdr.flags, true);
-      }
-      Enigmail.msg.fixMessageSubject();
-
       if (!Enigmail.msg.timeoutId && !Enigmail.msg.dirty) {
-        Enigmail.msg.timeoutId = setTimeout(function() {
+        Enigmail.msg.timeoutId = setTimeout(function () {
           Enigmail.msg.decryptQuote(false);
         }, 0);
       }
