@@ -1,26 +1,20 @@
-/*
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at https://mozilla.org/MPL/2.0/.
- */
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/. */
 
 "use strict";
 
-/* global setMessageEncryptionStateButton */
-/* global gFolderDisplay: false, currentAttachments: false */
-/* global gDBView: false, msgWindow: false, messageHeaderSink: false, gMessageListeners: false */
-/* global gExpandedHeaderView: false, CanDetachAttachments: true, gEncryptedURIService: false, FillAttachmentListPopup: false */
-/* global attachmentList: false, currentHeaderData: false */
+/* import-globals-from ../../../../base/content/aboutMessage.js */
+/* import-globals-from ../../../../base/content/msgHdrView.js */
+/* import-globals-from ../../../smime/content/msgHdrViewSMIMEOverlay.js */
 
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 XPCOMUtils.defineLazyModuleGetters(this, {
   EnigmailConstants: "chrome://openpgp/content/modules/constants.jsm",
   EnigmailCore: "chrome://openpgp/content/modules/core.jsm",
-  EnigmailData: "chrome://openpgp/content/modules/data.jsm",
   EnigmailDialog: "chrome://openpgp/content/modules/dialog.jsm",
   EnigmailFuncs: "chrome://openpgp/content/modules/funcs.jsm",
   EnigmailKey: "chrome://openpgp/content/modules/key.jsm",
@@ -35,10 +29,6 @@ XPCOMUtils.defineLazyModuleGetters(this, {
   // EnigmailWks: "chrome://openpgp/content/modules/webKey.jsm",
 });
 
-if (!Enigmail) {
-  var Enigmail = {};
-}
-
 Enigmail.hdrView = {
   lastEncryptedMsgKey: null,
   lastEncryptedUri: null,
@@ -49,11 +39,13 @@ Enigmail.hdrView = {
   msgSignatureState: EnigmailConstants.MSG_SIG_NONE,
   msgEncryptionState: EnigmailConstants.MSG_ENC_NONE,
   msgSignatureKeyId: "",
+  msgSignatureDate: null,
   msgEncryptionKeyId: null,
   msgEncryptionAllKeyIds: null,
   msgHasKeyAttached: false,
 
-  alreadyWrappedCDA: false,
+  ignoreStatusFromMimePart: "",
+  receivedStatusFromParts: new Set(),
 
   reset() {
     this.msgSignedStateString = null;
@@ -61,52 +53,22 @@ Enigmail.hdrView = {
     this.msgSignatureState = EnigmailConstants.MSG_SIG_NONE;
     this.msgEncryptionState = EnigmailConstants.MSG_ENC_NONE;
     this.msgSignatureKeyId = "";
+    this.msgSignatureDate = null;
     this.msgEncryptionKeyId = null;
     this.msgEncryptionAllKeyIds = null;
     this.msgHasKeyAttached = false;
     for (let value of ["decryptionFailed", "brokenExchange"]) {
       Enigmail.msg.removeNotification(value);
     }
+    this.ignoreStatusFromMimePart = "";
+    this.receivedStatusFromParts = new Set();
   },
 
   hdrViewLoad() {
     EnigmailLog.DEBUG("enigmailMsgHdrViewOverlay.js: this.hdrViewLoad\n");
 
-    // THE FOLLOWING OVERRIDES CODE IN msgHdrViewOverlay.js
-    // which wouldn't work otherwise
-
-    if (!this.alreadyWrappedCDA) {
-      this.alreadyWrappedCDA = true;
-      this.origCanDetachAttachments = CanDetachAttachments;
-      CanDetachAttachments = function() {
-        return (
-          Enigmail.hdrView.origCanDetachAttachments() &&
-          Enigmail.hdrView.enigCanDetachAttachments()
-        );
-      };
-    }
-
     this.msgHdrViewLoad();
 
-    /*
-    // Override SMIME ui
-    let signedHdrElement = document.getElementById("signedHdrIcon");
-    if (signedHdrElement) {
-      signedHdrElement.setAttribute(
-        "onclick",
-        "Enigmail.msg.viewSecurityInfo(event, true);"
-      );
-    }
-
-    let encryptedHdrElement = document.getElementById("encryptedHdrIcon");
-    if (encryptedHdrElement) {
-      encryptedHdrElement.setAttribute(
-        "onclick",
-        "Enigmail.msg.viewSecurityInfo(event, true);"
-      );
-    }
-    */
-
     let addrPopup = document.getElementById("emailAddressPopup");
     if (addrPopup) {
       addrPopup.addEventListener(
@@ -147,7 +109,7 @@ Enigmail.hdrView = {
     }
   },
 
-  updateHdrIcons(
+  updatePgpStatus(
     exitCode,
     statusFlags,
     extStatusFlags,
@@ -158,10 +120,10 @@ Enigmail.hdrView = {
     blockSeparation,
     encToDetails,
     xtraStatus,
-    encMimePartNumber
+    mimePartNumber
   ) {
     EnigmailLog.DEBUG(
-      "enigmailMsgHdrViewOverlay.js: this.updateHdrIcons: exitCode=" +
+      "enigmailMsgHdrViewOverlay.js: this.updatePgpStatus: exitCode=" +
         exitCode +
         ", statusFlags=" +
         statusFlags +
@@ -186,11 +148,8 @@ Enigmail.hdrView = {
     }
     */
 
-    if (
-      gFolderDisplay.selectedMessageUris &&
-      gFolderDisplay.selectedMessageUris.length > 0
-    ) {
-      this.lastEncryptedMsgKey = gFolderDisplay.selectedMessageUris[0];
+    if (gMessageURI) {
+      this.lastEncryptedMsgKey = gMessageURI;
     }
 
     if (!errorMsg) {
@@ -200,11 +159,8 @@ Enigmail.hdrView = {
     }
 
     var replaceUid = null;
-    if (keyId && gFolderDisplay.selectedMessage) {
-      replaceUid = EnigmailMsgRead.matchUidToSender(
-        keyId,
-        gFolderDisplay.selectedMessage.author
-      );
+    if (keyId && gMessage) {
+      replaceUid = EnigmailMsgRead.matchUidToSender(keyId, gMessage.author);
     }
 
     if (!replaceUid && userId) {
@@ -253,8 +209,9 @@ Enigmail.hdrView = {
       errorMsg = errorLines.join("\n") + "\n...\n" + lastLines;
     }
 
-    if (!(statusFlags & EnigmailConstants.PGP_MIME_ENCRYPTED)) {
-      encMimePartNumber = "";
+    let encryptedMimePart = "";
+    if (statusFlags & EnigmailConstants.PGP_MIME_ENCRYPTED) {
+      encryptedMimePart = mimePartNumber;
     }
 
     var msgSigned =
@@ -287,6 +244,8 @@ Enigmail.hdrView = {
       this.msgEncryptionAllKeyIds = encToDetails.allRecipKeys;
     }
 
+    this.msgSignatureDate = sigDetails?.sigDate;
+
     let tmp = {
       statusFlags,
       extStatusFlags,
@@ -295,7 +254,7 @@ Enigmail.hdrView = {
       msgSigned,
       blockSeparation,
       xtraStatus,
-      encryptedMimePart: encMimePartNumber,
+      encryptedMimePart,
     };
     Enigmail.msg.securityInfo = tmp;
 
@@ -307,7 +266,7 @@ Enigmail.hdrView = {
     }
     */
 
-    this.displayStatusBar();
+    this.updateStatusFlags(mimePartNumber);
     this.updateMsgDb();
   },
 
@@ -368,7 +327,14 @@ Enigmail.hdrView = {
   },
   */
 
-  async displayStatusBar() {
+  /**
+   * Update the various variables that track the OpenPGP status of
+   * the current message.
+   *
+   * @param {string} triggeredByMimePartNumber - the MIME part that
+   *   was processed and has triggered this status update request.
+   */
+  async updateStatusFlags(triggeredByMimePartNumber) {
     let secInfo = Enigmail.msg.securityInfo;
     let statusFlags = secInfo.statusFlags;
     let extStatusFlags =
@@ -469,7 +435,7 @@ Enigmail.hdrView = {
     if (signed) {
       this.msgSignedStateString = signed;
     }
-    this.updateEncryptionStateButton();
+    this.updateVisibleSecurityStatus(triggeredByMimePartNumber);
 
     /*
     // special handling after trying to fix buggy mail format (see buggyExchangeEmailContent in code)
@@ -506,20 +472,25 @@ Enigmail.hdrView = {
    */
   notifyHasKeyAttached() {
     this.msgHasKeyAttached = true;
-    this.updateEncryptionStateButton();
+    this.updateVisibleSecurityStatus();
   },
 
   /**
    * Should be called whenever more information about the OpenPGP
    * message state became available, such as encryption or signature
    * status, or the availability of an attached key.
+   *
+   * @param {string} triggeredByMimePartNumber - optional number of the
+   *   MIME part that was processed and has triggered this status update
+   *   request.
    */
-  updateEncryptionStateButton() {
-    setMessageEncryptionStateButton(
+  updateVisibleSecurityStatus(triggeredByMimePartNumber = undefined) {
+    setMessageCryptoBox(
       "OpenPGP",
       this.msgEncryptedStateString,
       this.msgSignedStateString,
-      this.msgHasKeyAttached
+      this.msgHasKeyAttached,
+      triggeredByMimePartNumber
     );
   },
 
@@ -529,7 +500,7 @@ Enigmail.hdrView = {
       [Enigmail.msg.securityInfo.userId],
       [Enigmail.msg.securityInfo.keyId]
     );
-    gDBView.reloadMessageWithAllParts();
+    ReloadMessage();
   },
 
   editKeyTrust() {
@@ -540,7 +511,7 @@ Enigmail.hdrView = {
       [Enigmail.msg.securityInfo.userId],
       [key.keyId]
     );
-    gDBView.reloadMessageWithAllParts();
+    ReloadMessage();
   },
 
   signKey() {
@@ -552,7 +523,7 @@ Enigmail.hdrView = {
       key.keyId,
       null
     );
-    gDBView.reloadMessageWithAllParts();
+    ReloadMessage();
   },
 
   msgHdrViewLoad() {
@@ -566,10 +537,7 @@ Enigmail.hdrView = {
 
         try {
           Enigmail.hdrView.statusBarHide();
-          EnigmailVerify.setMsgWindow(
-            msgWindow,
-            Enigmail.msg.getCurrentMsgUriSpec()
-          );
+          EnigmailVerify.setWindow(window, Enigmail.msg.getCurrentMsgUriSpec());
 
           let msgFrame = document.getElementById("messagepane").contentDocument;
 
@@ -597,10 +565,18 @@ Enigmail.hdrView = {
         EnigmailLog.DEBUG(
           "enigmailMsgHdrViewOverlay.js: _listener_onEndHeaders\n"
         );
+      },
+
+      onEndAttachments() {
+        EnigmailLog.DEBUG(
+          "enigmailMsgHdrViewOverlay.js: _listener_onEndAttachments\n"
+        );
 
         try {
-          Enigmail.hdrView.statusBarHide();
+          EnigmailVerify.setWindow(null, null);
         } catch (ex) {}
+
+        Enigmail.hdrView.messageLoad();
       },
 
       beforeStartHeaders() {
@@ -612,7 +588,7 @@ Enigmail.hdrView = {
 
     // fire the handlers since some windows open directly with a visible message
     this.messageListener.onStartHeaders();
-    this.messageListener.onEndHeaders();
+    this.messageListener.onEndAttachments();
   },
 
   messageUnload(event) {
@@ -736,7 +712,7 @@ Enigmail.hdrView = {
 
   updateMsgDb() {
     EnigmailLog.DEBUG("enigmailMsgHdrViewOverlay.js: this.updateMsgDb\n");
-    var msg = gFolderDisplay.selectedMessage;
+    var msg = gMessage;
     if (!msg || !msg.folder) {
       return;
     }
@@ -769,68 +745,37 @@ Enigmail.hdrView = {
     return canDetach;
   },
 
-  fillAttachmentListPopup(item) {
-    EnigmailLog.DEBUG(
-      "enigmailMsgHdrViewOverlay.js: Enigmail.hdrView.fillAttachmentListPopup\n"
-    );
-    FillAttachmentListPopup(item);
-
-    if (!this.enigCanDetachAttachments()) {
-      for (var i = 0; i < item.childNodes.length; i++) {
-        if (item.childNodes[i].className == "menu-iconic") {
-          var mnu = item.childNodes[i].firstChild.firstChild;
-          while (mnu) {
-            if (
-              mnu
-                .getAttribute("oncommand")
-                .search(/(detachAttachment|deleteAttachment)/) >= 0
-            ) {
-              mnu.setAttribute("disabled", true);
-            }
-            mnu = mnu.nextSibling;
-          }
-        }
-      }
-    }
-  },
-
   setSubject(subject) {
-    if (
-      gFolderDisplay.selectedMessages.length === 1 &&
-      gFolderDisplay.selectedMessage
-    ) {
-      // Strip multiple localised Re: prefixes. This emulates NS_MsgStripRE().
-      let newSubject = subject;
-      let prefixes = Services.prefs.getStringPref("mailnews.localizedRe", "Re");
-      prefixes = prefixes.split(",");
-      if (!prefixes.includes("Re")) {
-        prefixes.push("Re");
-      }
-      // Construct a regular expression like this: ^(Re: |Aw: )+
-      let regEx = new RegExp(`^(${prefixes.join(": |")}: )+`, "i");
-      newSubject = newSubject.replace(regEx, "");
-      let hadRe = newSubject != subject;
-
-      // Update the header pane.
-      this.updateHdrBox("subject", hadRe ? "Re: " + newSubject : newSubject);
-
-      // Update the thread pane.
-      let tree = gFolderDisplay.tree;
-      let msgHdr = gFolderDisplay.selectedMessage;
-      msgHdr.subject = EnigmailData.convertFromUnicode(newSubject, "utf-8");
-
-      // Set the corred HasRe flag and refresh the row.
-      let oldFlags = msgHdr.flags;
-      if (hadRe && !(oldFlags & Ci.nsMsgMessageFlags.HasRe)) {
-        let newFlags = oldFlags | Ci.nsMsgMessageFlags.HasRe;
-        msgHdr.flags = newFlags;
-        if (tree && tree.view) {
-          tree.view.db.NotifyHdrChangeAll(msgHdr, oldFlags, newFlags, {});
-        }
-      } else if (tree && tree.view && tree.view.selection) {
-        tree.invalidateRow(tree.view.selection.currentIndex);
-      }
+    // Strip multiple localised Re: prefixes. This emulates NS_MsgStripRE().
+    let prefixes = Services.prefs.getStringPref("mailnews.localizedRe", "Re");
+    prefixes = prefixes.split(",");
+    if (!prefixes.includes("Re")) {
+      prefixes.push("Re");
     }
+    // Construct a regular expression like this: ^(Re: |Aw: )+
+    let newSubject = subject.replace(
+      new RegExp(`^(${prefixes.join(": |")}: )+`, "i"),
+      ""
+    );
+    let hadRe = newSubject != subject;
+
+    // Update the message.
+    gMessage.subject = newSubject;
+    let oldFlags = gMessage.flags;
+    if (hadRe) {
+      gMessage.flags |= Ci.nsMsgMessageFlags.HasRe;
+      newSubject = "Re: " + newSubject;
+    }
+    document.title = newSubject;
+    currentHeaderData.subject.headerValue = newSubject;
+    document.getElementById("expandedsubjectBox").headerValue = newSubject;
+    // This even works if the flags haven't changed. Causes repaint in all thread trees.
+    gMessage.folder?.msgDatabase.notifyHdrChangeAll(
+      gMessage,
+      oldFlags,
+      gMessage.flags,
+      {}
+    );
   },
 
   updateHdrBox(header, value) {
@@ -862,45 +807,7 @@ Enigmail.hdrView = {
           "\n"
       );
 
-      if (!uriSpec) {
-        // We cannot compare if no URI, => assume it's the current message.
-        return true;
-      }
-
-      let msgUriSpec = Enigmail.msg.getCurrentMsgUriSpec();
-      let currUrl = EnigmailFuncs.getUrlFromUriSpec(msgUriSpec);
-      if (!currUrl) {
-        return false;
-      }
-
-      let currMsgId = EnigmailURIs.msgIdentificationFromUrl(currUrl);
-      let gotMsgId = EnigmailURIs.msgIdentificationFromUrl(uri);
-
-      if (!gotMsgId) {
-        return false;
-      }
-
-      EnigmailLog.DEBUG(
-        "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.isCurrentMessage: url=" +
-          currUrl.spec +
-          "\n"
-      );
-
-      if (
-        uri.host == currUrl.host &&
-        currMsgId.folder === gotMsgId.folder &&
-        currMsgId.msgNum === gotMsgId.msgNum
-      ) {
-        EnigmailLog.DEBUG(
-          "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.isCurrentMessage: true\n"
-        );
-        return true;
-      }
-
-      EnigmailLog.DEBUG(
-        "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.isCurrentMessage: false\n"
-      );
-      return false;
+      return true;
     },
 
     /**
@@ -969,19 +876,23 @@ Enigmail.hdrView = {
     },
 
     /**
-     * Determine if there are message parts that are not signed/encrypted
+     * Determine if there are message parts that are not encrypted
      *
      * @param mimePartNumber String - the MIME part number that was authenticated
      *
-     * @return Boolean: true: there are siblings / false: no siblings
+     * @returns Boolean: true: there are siblings / false: no siblings
      */
     hasUnauthenticatedParts(mimePartNumber) {
-      function hasSiblings(mimePart, searchPartNum, parentNum) {
-        if (mimePart.partNum === parentNum) {
-          // if we're a direct child of a PGP/MIME encrypted message, we know that everything
-          // is authenticated on this level
+      function hasUnauthenticatedSiblings(
+        mimeSubTree,
+        mimePartToCheck,
+        parentOfMimePartToCheck
+      ) {
+        if (mimeSubTree.partNum === parentOfMimePartToCheck) {
+          // If this is an encrypted message that is the parent of mimePartToCheck,
+          // then we know that all its childs (including mimePartToCheck) are authenticated.
           if (
-            mimePart.fullContentType.search(
+            mimeSubTree.fullContentType.search(
               /^multipart\/encrypted.{1,255}protocol="?application\/pgp-encrypted"?/i
             ) === 0
           ) {
@@ -989,14 +900,21 @@ Enigmail.hdrView = {
           }
         }
         if (
-          mimePart.partNum.indexOf(parentNum) == 0 &&
-          mimePart.partNum !== searchPartNum
+          mimeSubTree.partNum.indexOf(parentOfMimePartToCheck) == 0 &&
+          mimeSubTree.partNum !== mimePartToCheck
         ) {
+          // This is a sibling (same parent, different part number).
           return true;
         }
 
-        for (let i in mimePart.subParts) {
-          if (hasSiblings(mimePart.subParts[i], searchPartNum, parentNum)) {
+        for (let i in mimeSubTree.subParts) {
+          if (
+            hasUnauthenticatedSiblings(
+              mimeSubTree.subParts[i],
+              mimePartToCheck,
+              parentOfMimePartToCheck
+            )
+          ) {
             return true;
           }
         }
@@ -1004,18 +922,37 @@ Enigmail.hdrView = {
         return false;
       }
 
-      let parentNum = mimePartNumber.replace(/\.\d+$/, "");
-      if (mimePartNumber.search(/\./) < 0) {
-        parentNum = "";
+      if (!mimePartNumber || !Enigmail.msg.mimeParts) {
+        return false;
       }
 
-      if (mimePartNumber && Enigmail.msg.mimeParts) {
-        if (hasSiblings(Enigmail.msg.mimeParts, mimePartNumber, parentNum)) {
-          return true;
-        }
+      let parentNum = "";
+      if (mimePartNumber.includes(".")) {
+        parentNum = mimePartNumber.replace(/\.\d+$/, "");
       }
 
-      return false;
+      return hasUnauthenticatedSiblings(
+        Enigmail.msg.mimeParts,
+        mimePartNumber,
+        parentNum
+      );
+    },
+
+    /**
+     * Request that OpenPGP security status from the given MIME part
+     * shall be ignored (not shown in the UI). If status for that
+     * MIME part was already received, then reset the status.
+     *
+     * @param {string} originMimePartNumber - Ignore security status
+     *   of this MIME part.
+     */
+    ignoreStatusFrom(originMimePartNumber) {
+      Enigmail.hdrView.ignoreStatusFromMimePart = originMimePartNumber;
+      setIgnoreStatusFromMimePart(originMimePartNumber);
+      if (Enigmail.hdrView.receivedStatusFromParts.has(originMimePartNumber)) {
+        Enigmail.hdrView.reset();
+        Enigmail.hdrView.ignoreStatusFromMimePart = originMimePartNumber;
+      }
     },
 
     async updateSecurityStatus(
@@ -1032,6 +969,15 @@ Enigmail.hdrView = {
       extraDetails,
       mimePartNumber
     ) {
+      if (
+        Enigmail.hdrView.ignoreStatusFromMimePart != "" &&
+        mimePartNumber == Enigmail.hdrView.ignoreStatusFromMimePart
+      ) {
+        return;
+      }
+
+      Enigmail.hdrView.receivedStatusFromParts.add(mimePartNumber);
+
       // uriSpec is not used for Enigmail anymore. It is here because other addons and pEp rely on it
 
       EnigmailLog.DEBUG(
@@ -1046,8 +992,7 @@ Enigmail.hdrView = {
         if (statusFlags & EnigmailConstants.DECRYPTION_OKAY) {
           if (gEncryptedURIService) {
             // remember encrypted message URI to enable TB prevention against EFAIL attack
-            Enigmail.hdrView.lastEncryptedUri =
-              gFolderDisplay.selectedMessageUris[0];
+            Enigmail.hdrView.lastEncryptedUri = gMessageURI;
             gEncryptedURIService.rememberEncrypted(
               Enigmail.hdrView.lastEncryptedUri
             );
@@ -1077,7 +1022,7 @@ Enigmail.hdrView = {
           }
         }
 
-        Enigmail.hdrView.updateHdrIcons(
+        Enigmail.hdrView.updatePgpStatus(
           exitCode,
           statusFlags,
           extStatusFlags,
@@ -1105,14 +1050,11 @@ Enigmail.hdrView = {
           "\n"
       );
 
-      let msg = gFolderDisplay.selectedMessage;
+      let msg = gMessage;
       if (!msg) {
         return;
       }
-      if (
-        !this.isCurrentMessage(uri) ||
-        gFolderDisplay.selectedMessages.length !== 1
-      ) {
+      if (!this.isCurrentMessage(uri)) {
         return;
       }
 
@@ -1151,7 +1093,7 @@ Enigmail.hdrView = {
         return;
       }
 
-      let msg = gFolderDisplay.selectedMessage;
+      let msg = gMessage;
 
       if ("subject" in hdr) {
         Enigmail.hdrView.setSubject(hdr.subject);
@@ -1196,10 +1138,13 @@ Enigmail.hdrView = {
         Enigmail.hdrView.msgSignedStateString != null ||
         Enigmail.hdrView.msgEncryptedStateString != null
       ) {
+        // If we already processed an OpenPGP part, then we are handling
+        // a message with an inner S/MIME part. We must not reload
+        // the message here, because we'd run into an endless loop.
         return;
       }
       if (this.isCurrentMessage(uri)) {
-        EnigmailVerify.unregisterContentTypeHandler();
+        EnigmailVerify.unregisterPGPMimeHandler();
         Enigmail.msg.messageReload(false);
       }
     },
@@ -1256,8 +1201,6 @@ Enigmail.hdrView = {
       );
       msgFrame.removeEventListener("load", Enigmail.hdrView.messageLoad);
     }
-
-    CanDetachAttachments = Enigmail.hdrView.origCanDetachAttachments;
   },
   */
 };
