@@ -7,11 +7,14 @@ const {
   be_in_folder,
   create_folder,
   create_message,
+  get_about_message,
   select_click_row,
 } = ChromeUtils.import(
   "resource://testing-common/mozmill/FolderDisplayHelpers.jsm"
 );
 
+let aboutMessage = get_about_message();
+
 const mimeService = Cc["@mozilla.org/mime;1"].getService(Ci.nsIMIMEService);
 const handlerService = Cc[
   "@mozilla.org/uriloader/handler-service;1"
@@ -42,9 +45,9 @@ function getNsIFileFromPath(path) {
   return file;
 }
 
-add_setup(async function() {
+add_setup(async function () {
   folder = await create_folder("OpenAttachment");
-  be_in_folder(folder);
+  await be_in_folder(folder);
 
   // @see logic for tmpD in msgHdrView.js
   tmpD = PathUtils.join(
@@ -78,7 +81,7 @@ add_setup(async function() {
   });
 });
 
-registerCleanupFunction(async function() {
+registerCleanupFunction(async function () {
   MockFilePicker.cleanup();
 
   await IOUtils.remove(savePath, { recursive: true });
@@ -98,6 +101,8 @@ registerCleanupFunction(async function() {
 
   // Remove created folders.
   folder.deleteSelf(null);
+
+  Services.focus.focusedWindow = window;
 });
 
 function createMockedHandler(type, preferredAction, alwaysAskBeforeHandling) {
@@ -152,9 +157,8 @@ async function createAndLoadMessage(
 
     // Append relevant Thunderbird headers to indicate a detached file.
     attachment.extraHeaders = {
-      "X-Mozilla-External-Attachment-URL": fileHandler.getURLSpecFromActualFile(
-        attachmentFile
-      ),
+      "X-Mozilla-External-Attachment-URL":
+        fileHandler.getURLSpecFromActualFile(attachmentFile),
       "X-Mozilla-Altered":
         'AttachmentDetached; date="Mon Apr 04 13:59:42 2022"',
     };
@@ -206,29 +210,26 @@ async function singleClickAttachmentAndWaitForDialog(
           );
         }
 
-        dialogDocument
-          .querySelector("dialog")
-          .getButton(button)
-          .click();
+        dialogDocument.querySelector("dialog").getButton(button).click();
       },
     }
   );
 
-  info(document.getElementById("attachmentName").value);
+  info(aboutMessage.document.getElementById("attachmentName").value);
   EventUtils.synthesizeMouseAtCenter(
-    document.getElementById("attachmentName"),
+    aboutMessage.document.getElementById("attachmentName"),
     {},
-    window
+    aboutMessage
   );
   await dialogPromise;
 }
 
 async function singleClickAttachment() {
-  info(document.getElementById("attachmentName").value);
+  info(aboutMessage.document.getElementById("attachmentName").value);
   EventUtils.synthesizeMouseAtCenter(
-    document.getElementById("attachmentName"),
+    aboutMessage.document.getElementById("attachmentName"),
     {},
-    window
+    aboutMessage
   );
 }
 
@@ -269,10 +270,13 @@ function checkHandler(type, preferredAction, alwaysAskBeforeHandling) {
 }
 
 function promiseFileOpened() {
-  let __openFile = window.AttachmentInfo.prototype._openFile;
+  let __openFile = aboutMessage.AttachmentInfo.prototype._openFile;
   return new Promise(resolve => {
-    window.AttachmentInfo.prototype._openFile = function(mimeInfo, file) {
-      window.AttachmentInfo.prototype._openFile = __openFile;
+    aboutMessage.AttachmentInfo.prototype._openFile = function (
+      mimeInfo,
+      file
+    ) {
+      aboutMessage.AttachmentInfo.prototype._openFile = __openFile;
       resolve({ mimeInfo, file });
     };
   });
@@ -374,7 +378,7 @@ add_task(async function saveToDiskAlwaysAskPromptLocation() {
 
   let expectedFile = getNsIFileFromPath(tmpD);
   expectedFile.append(`attachment${messageIndex}.test${messageIndex}`);
-  MockFilePicker.showCallback = function(instance) {
+  MockFilePicker.showCallback = function (instance) {
     Assert.equal(instance.defaultString, expectedFile.leafName);
     Assert.equal(instance.defaultExtension, `test${messageIndex}`);
   };
@@ -500,7 +504,7 @@ add_task(async function saveToDiskPromptLocation() {
 
   let expectedFile = getNsIFileFromPath(tmpD);
   expectedFile.append(`attachment${messageIndex}.test${messageIndex}`);
-  MockFilePicker.showCallback = function(instance) {
+  MockFilePicker.showCallback = function (instance) {
     Assert.equal(instance.defaultString, expectedFile.leafName);
     Assert.equal(instance.defaultExtension, `test${messageIndex}`);
   };
@@ -569,10 +573,14 @@ add_task(async function useHelperApp() {
   let { file } = await openedPromise;
   Assert.ok(file.path);
 
-  // In the temp dir, files should be read only.
+  // In the temp dir, files should be read-only.
   if (AppConstants.platform != "win") {
     let fileInfo = await IOUtils.stat(file.path);
-    Assert.equal(fileInfo.permissions, 0o400);
+    Assert.equal(
+      fileInfo.permissions,
+      0o400,
+      `file ${file.path} should be read-only`
+    );
   }
   attachmentFile.permissions = 0o755;
   attachmentFile.remove(false);
@@ -623,10 +631,14 @@ add_task(async function useSystemDefault() {
   let { file } = await openedPromise;
   Assert.ok(file.path);
 
-  // In the temp dir, files should be read only.
+  // In the temp dir, files should be read-only.
   if (AppConstants.platform != "win") {
     let fileInfo = await IOUtils.stat(file.path);
-    Assert.equal(fileInfo.permissions, 0o400);
+    Assert.equal(
+      fileInfo.permissions,
+      0o400,
+      `file ${file.path} should be read-only`
+    );
   }
   attachmentFile.permissions = 0o755;
   attachmentFile.remove(false);
@@ -701,10 +713,14 @@ add_task(async function filenameSanitisedOpen() {
     "f i_le_123.bar"
   );
   Assert.equal(file.leafName, "f i_le_123.bar");
-  // In the temp dir, files should be read only.
+  // In the temp dir, files should be read-only.
   if (AppConstants.platform != "win") {
     let fileInfo = await IOUtils.stat(file.path);
-    Assert.equal(fileInfo.permissions, 0o400);
+    Assert.equal(
+      fileInfo.permissions,
+      0o400,
+      `file ${file.path} should be read-only`
+    );
   }
   attachmentFile.permissions = 0o755;
   attachmentFile.remove(false);
