@@ -8,11 +8,8 @@
 
 "use strict";
 
-var {
-  add_attachments,
-  close_compose_window,
-  open_compose_new_mail,
-} = ChromeUtils.import("resource://testing-common/mozmill/ComposeHelpers.jsm");
+var { add_attachments, close_compose_window, open_compose_new_mail } =
+  ChromeUtils.import("resource://testing-common/mozmill/ComposeHelpers.jsm");
 var { mc } = ChromeUtils.import(
   "resource://testing-common/mozmill/FolderDisplayHelpers.jsm"
 );
@@ -23,7 +20,7 @@ requestLongerTimeout(3);
  * Test the cycling of focus in the composition window through (Shift+)F6.
  *
  * @param {MozMillController} controller - Controller for the compose window.
- * @param {Object} options - Options to set for the test.
+ * @param {object} options - Options to set for the test.
  * @param {boolean} options.useTab - Whether to use Ctrl+Tab instead of F6.
  * @param {boolean} options.attachment - Whether to add an attachment.
  * @param {boolean} options.notifications - Whether to show notifications.
@@ -65,7 +62,7 @@ async function checkFocusCycling(controller, options) {
     );
 
   if (options.attachment) {
-    add_attachments(controller, "http://www.mozilla.org/");
+    add_attachments(controller, "https://www.mozilla.org/");
   }
 
   if (options.contacts) {
