@@ -31,7 +31,7 @@ var testFolder = null;
 var identity1Email = "carl@example.com";
 var identity2Email = "lenny@springfield.invalid";
 
-add_setup(async function() {
+add_setup(async function () {
   addIdentitiesAndFolder();
   // Msg #0
   await add_message_to_folder(
@@ -41,8 +41,7 @@ add_setup(async function() {
       to: "workers@springfield.invalid",
       subject: "no matching identity, like bcc/list",
       body: {
-        body:
-          "Alcohol is a way of life, alcohol is my way of life, and I aim to keep it.",
+        body: "Alcohol is a way of life, alcohol is my way of life, and I aim to keep it.",
       },
       clobberHeaders: {},
     })
@@ -136,7 +135,7 @@ function addIdentitiesAndFolder() {
 }
 
 function checkReply(replyWin, expectedFromEmail) {
-  let identityList = replyWin.e("msgIdentity");
+  let identityList = replyWin.window.document.getElementById("msgIdentity");
   if (!identityList.selectedItem.label.includes(expectedFromEmail)) {
     throw new Error(
       "The From address is not correctly selected! Expected: " +
@@ -147,8 +146,8 @@ function checkReply(replyWin, expectedFromEmail) {
   }
 }
 
-add_task(function test_reply_no_matching_identity() {
-  be_in_folder(testFolder);
+add_task(async function test_reply_no_matching_identity() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(0);
   assert_selected_and_displayed(mc, msg);
@@ -159,8 +158,8 @@ add_task(function test_reply_no_matching_identity() {
   close_compose_window(replyWin);
 });
 
-add_task(function test_reply_matching_only_deliveredto() {
-  be_in_folder(testFolder);
+add_task(async function test_reply_matching_only_deliveredto() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(1);
   assert_selected_and_displayed(mc, msg);
@@ -169,10 +168,10 @@ add_task(function test_reply_matching_only_deliveredto() {
   // Should have selected the second id, which is listed in Delivered-To:.
   checkReply(replyWin, identity2Email);
   close_compose_window(replyWin);
-});
+}).skip();
 
-add_task(function test_reply_matching_subaddress() {
-  be_in_folder(testFolder);
+add_task(async function test_reply_matching_subaddress() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(2);
   assert_selected_and_displayed(mc, msg);
@@ -184,8 +183,8 @@ add_task(function test_reply_matching_subaddress() {
   close_compose_window(replyWin);
 });
 
-add_task(function test_reply_to_matching_second_id() {
-  be_in_folder(testFolder);
+add_task(async function test_reply_to_matching_second_id() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(3);
   assert_selected_and_displayed(mc, msg);
@@ -196,8 +195,8 @@ add_task(function test_reply_to_matching_second_id() {
   close_compose_window(replyWin);
 });
 
-add_task(function test_deliveredto_to_matching_only_parlty() {
-  be_in_folder(testFolder);
+add_task(async function test_deliveredto_to_matching_only_parlty() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(4);
   assert_selected_and_displayed(mc, msg);
@@ -212,8 +211,8 @@ add_task(function test_deliveredto_to_matching_only_parlty() {
  * A reply from self is treated as a follow-up. And this self
  * was the second identity, so the reply should also be from the second identity.
  */
-add_task(function test_reply_to_self_second_id() {
-  be_in_folder(testFolder);
+add_task(async function test_reply_to_self_second_id() {
+  await be_in_folder(testFolder);
 
   let msg = select_click_row(5);
   assert_selected_and_displayed(mc, msg);
