@@ -8,6 +8,7 @@
 
 "use strict";
 
+var utils = ChromeUtils.import("resource://testing-common/mozmill/utils.jsm");
 var {
   be_in_folder,
   close_message_window,
@@ -25,7 +26,7 @@ var {
 var menuFolder;
 var menuState;
 
-add_setup(async function() {
+add_setup(async function () {
   menuFolder = await create_folder("menuFolder");
   await make_message_sets_in_folders([menuFolder], [{ count: 1 }]);
 
@@ -79,51 +80,42 @@ async function help_test_autohide(controller, menubar) {
       return (menubar.getBoundingClientRect().height != 0) != aHidden;
     };
   }
+  await TestUtils.waitForCondition(
+    () => Services.focus.activeWindow == controller.window
+  );
   await set_autohide_menubar(controller, menubar, true);
-  controller.waitFor(hiddenChecker(true), "Menubar should be hidden");
+  utils.waitFor(hiddenChecker(true), "Menubar should be hidden");
 
   menubar.focus();
   EventUtils.synthesizeKey("VK_ALT", {}, controller.window);
-  controller.waitFor(
+  utils.waitFor(
     hiddenChecker(false),
     "Menubar should be shown after pressing ALT!"
   );
 
   info("Menubar showing or not should toggle for ALT.");
   await set_autohide_menubar(controller, menubar, false);
-  controller.waitFor(hiddenChecker(false), "Menubar should be shown");
+  utils.waitFor(hiddenChecker(false), "Menubar should be shown");
   Assert.ok("help_test_autohide success");
 }
 
 add_task(async function test_autohidden_menubar_3pane() {
-  let menubar = mc.e("toolbar-menubar");
+  let menubar = mc.window.document.getElementById("toolbar-menubar");
   await help_test_autohide(mc, menubar);
 });
 
 add_task(async function test_autohidden_menubar_message_window() {
-  be_in_folder(menuFolder);
+  await be_in_folder(menuFolder);
   select_click_row(0);
   let msgc = await open_selected_message_in_new_window();
-  msgc.window.focus();
-  let menubar = msgc.e("toolbar-menubar");
+  let menubar = msgc.window.document.getElementById("toolbar-menubar");
 
   await help_test_autohide(msgc, menubar);
   close_message_window(msgc);
 });
 
-// We don't have an autohidable menu bar item for compose.
-/*
-add_task(async function test_autohidden_menubar_compose_window() {
-  let cwc = open_compose_new_mail();
-  let menubar = cwc.e("compose-toolbar-menubar2");
-
-  await help_test_autohide(cwc, menubar);
-  close_compose_window(cwc);
-});
-*/
-
-registerCleanupFunction(function() {
+registerCleanupFunction(async function () {
   toggle_main_menu(menuState);
-  be_in_folder(inboxFolder);
+  await be_in_folder(inboxFolder);
   menuFolder.deleteSelf(null);
 });
