@@ -2,20 +2,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
 import React from "react";
+import PropTypes from "prop-types";
 
 import AccessibleImage from "../AccessibleImage";
 
 import "./styles/CloseButton.css";
 
-type Props = {
-  handleClick: Function,
-  buttonClass?: string,
-  tooltip?: string,
-};
-
-function CloseButton({ handleClick, buttonClass, tooltip }: Props) {
+function CloseButton({ handleClick, buttonClass, tooltip }) {
   return (
     <button
       className={buttonClass ? `close-btn ${buttonClass}` : "close-btn"}
@@ -27,4 +21,10 @@ function CloseButton({ handleClick, buttonClass, tooltip }: Props) {
   );
 }
 
+CloseButton.propTypes = {
+  buttonClass: PropTypes.string,
+  handleClick: PropTypes.func.isRequired,
+  tooltip: PropTypes.string,
+};
+
 export default CloseButton;
