@@ -24,7 +24,7 @@ cases(
   "parseLineColumn utility",
   ({ query, location }) => expect(parseLineColumn(query)).toEqual(location),
   [
-    { name: "empty query", query: "", location: undefined },
+    { name: "empty query", query: "", location: null },
     { name: "just line", query: ":30", location: { line: 30 } },
     {
       name: "line and column",
