@@ -36,7 +36,7 @@ const TEST_ARRAY = [
 /**
  * Basic test that checks content of the DOM panel.
  */
-add_task(async function() {
+add_task(async function () {
   info("Test DOM Panel Array Expansion started");
 
   const { panel } = await addTestTab(TEST_PAGE_URL);
@@ -56,7 +56,7 @@ add_task(async function() {
     const row = childRows[name];
 
     is(
-      name,
+      parseInt(name, 10),
       i++,
       `index ${name} is correct and sorted into the correct position`
     );
