@@ -4,16 +4,16 @@
 
 "use strict";
 
-const DevToolsUtils = require("devtools/shared/DevToolsUtils");
-const { LongStringFront } = require("devtools/client/fronts/string");
 const {
   FrontClassWithSpec,
   registerFront,
-} = require("devtools/shared/protocol");
-const { webconsoleSpec } = require("devtools/shared/specs/webconsole");
+} = require("resource://devtools/shared/protocol.js");
+const {
+  webconsoleSpec,
+} = require("resource://devtools/shared/specs/webconsole.js");
 const {
   getAdHocFrontOrPrimitiveGrip,
-} = require("devtools/client/fronts/object");
+} = require("resource://devtools/client/fronts/object.js");
 
 /**
  * A WebConsoleFront is used as a front end for the WebConsoleActor that is
@@ -26,8 +26,6 @@ class WebConsoleFront extends FrontClassWithSpec(webconsoleSpec) {
   constructor(client, targetFront, parentFront) {
     super(client, targetFront, parentFront);
     this._client = client;
-    this.traits = {};
-    this._longStrings = {};
     this.events = [];
 
     // Attribute name from which to retrieve the actorID out of the target actor's form
@@ -295,78 +293,6 @@ class WebConsoleFront extends FrontClassWithSpec(webconsoleSpec) {
     return this._client.request(packet, onResponse);
   }
 
-  /**
-   * Start the given Web Console listeners.
-   * TODO: remove once the front is retrieved via getFront, and we use form()
-   *
-   * @see this.LISTENERS
-   * @param array listeners
-   *        Array of listeners you want to start. See this.LISTENERS for
-   *        known listeners.
-   * @return request
-   *         Request object that implements both Promise and EventEmitter interfaces
-   */
-  async startListeners(listeners) {
-    const response = await super.startListeners(listeners);
-    this.hasNativeConsoleAPI = response.nativeConsoleAPI;
-    this.traits = response.traits;
-    return response;
-  }
-
-  /**
-   * Return an instance of LongStringFront for the given long string grip.
-   *
-   * @param object grip
-   *        The long string grip returned by the protocol.
-   * @return {LongStringFront} the front for the given long string grip.
-   */
-  longString(grip) {
-    if (grip.actor in this._longStrings) {
-      return this._longStrings[grip.actor];
-    }
-
-    const front = new LongStringFront(this._client, this.targetFront, this);
-    front.form(grip);
-    this.manage(front);
-    this._longStrings[grip.actor] = front;
-    return front;
-  }
-
-  /**
-   * Fetches the full text of a LongString.
-   *
-   * @param object | string stringGrip
-   *        The long string grip containing the corresponding actor.
-   *        If you pass in a plain string (by accident or because you're lazy),
-   *        then a promise of the same string is simply returned.
-   * @return object Promise
-   *         A promise that is resolved when the full string contents
-   *         are available, or rejected if something goes wrong.
-   */
-  async getString(stringGrip) {
-    // Make sure this is a long string.
-    if (typeof stringGrip !== "object" || stringGrip.type !== "longString") {
-      // Go home string, you're drunk.
-      return stringGrip;
-    }
-
-    // Fetch the long string only once.
-    if (stringGrip._fullText) {
-      return stringGrip._fullText;
-    }
-
-    const { initial, length } = stringGrip;
-    const longStringFront = this.longString(stringGrip);
-
-    try {
-      const response = await longStringFront.substring(initial.length, length);
-      return initial + response;
-    } catch (e) {
-      DevToolsUtils.reportException("getString", e.message);
-      throw e;
-    }
-  }
-
   /**
    * Close the WebConsoleFront.
    *
@@ -381,7 +307,6 @@ class WebConsoleFront extends FrontClassWithSpec(webconsoleSpec) {
     // at the top of the function.
     this._client = null;
 
-    this._longStrings = null;
     return super.destroy();
   }
 }
