@@ -35,7 +35,7 @@ EditingSession.prototype = {
    *         The name of the property.
    * @return {String} the value.
    */
-  getPropertyFromRule: function(rule, property) {
+  getPropertyFromRule(rule, property) {
     // Use the parsed declarations in the StyleRuleFront object if available.
     const index = this.getPropertyIndex(property, rule);
     if (index !== -1) {
@@ -55,7 +55,7 @@ EditingSession.prototype = {
    * @param  {String} property
    *         The name of the property as a string
    */
-  getProperty: function(property) {
+  getProperty(property) {
     // Create a hidden element for getPropertyFromRule to use
     const div = this._doc.createElement("div");
     div.setAttribute("style", "display: none");
@@ -86,13 +86,12 @@ EditingSession.prototype = {
    *         Optional, defaults to the element style rule.
    * @return {Number} The property index in the rule.
    */
-  getPropertyIndex: function(name, rule = this._rules[0]) {
-    const elementStyleRule = this._rules[0];
-    if (!elementStyleRule.declarations.length) {
+  getPropertyIndex(name, rule = this._rules[0]) {
+    if (!rule.declarations.length) {
       return -1;
     }
 
-    return elementStyleRule.declarations.findIndex(p => p.name === name);
+    return rule.declarations.findIndex(p => p.name === name);
   },
 
   /**
@@ -175,7 +174,7 @@ EditingSession.prototype = {
     }
   },
 
-  destroy: function() {
+  destroy() {
     this._modifications.clear();
 
     this._cssProperties = null;
