@@ -11,7 +11,7 @@ const PREF = "devtools.source-map.client-service.enabled";
 const SCSS_LOC = "doc_sourcemaps.scss:4";
 const CSS_LOC = "doc_sourcemaps2.css:1";
 
-add_task(async function() {
+add_task(async function () {
   info("Setting the " + PREF + " pref to true");
   Services.prefs.setBoolPref(PREF, true);
 
@@ -31,7 +31,18 @@ add_task(async function() {
   Services.prefs.setBoolPref(PREF, true);
 
   await testClickingLink(toolbox, view);
-  await checkDisplayedStylesheet(toolbox);
+  const selectedEditor = await waitForOriginalStyleSheetEditorSelection(
+    toolbox
+  );
+  const href = selectedEditor.styleSheet.href;
+  ok(
+    href.endsWith("doc_sourcemaps.scss"),
+    "selected stylesheet is correct one"
+  );
+  await selectedEditor.getSourceEditor();
+
+  const { line } = selectedEditor.sourceEditor.getCursor();
+  is(line, 3, "cursor is at correct line number in original source");
 
   info("Clearing the " + PREF + " pref");
   Services.prefs.clearUserPref(PREF);
@@ -39,7 +50,7 @@ add_task(async function() {
 
 async function testClickingLink(toolbox, view) {
   info("Listening for switch to the style editor");
-  const onStyleEditorReady = toolbox.once("styleeditor-ready");
+  const onStyleEditorReady = toolbox.once("styleeditor-selected");
 
   info("Finding the stylesheet link and clicking it");
   const link = getRuleViewLinkByIndex(view, 1);
@@ -48,40 +59,31 @@ async function testClickingLink(toolbox, view) {
   await onStyleEditorReady;
 }
 
-function checkDisplayedStylesheet(toolbox) {
+function waitForOriginalStyleSheetEditorSelection(toolbox) {
   const panel = toolbox.getCurrentPanel();
   return new Promise((resolve, reject) => {
-    panel.UI.on("editor-selected", editor => {
+    const maybeContinue = editor => {
       // The style editor selects the first sheet at first load before
       // selecting the desired sheet.
       if (editor.styleSheet.href.endsWith("scss")) {
         info("Original source editor selected");
-        editor
-          .getSourceEditor()
-          .then(editorSelected)
-          .then(resolve, reject);
+        off();
+        resolve(editor);
       }
-    });
+    };
+    const off = panel.UI.on("editor-selected", maybeContinue);
+    if (panel.UI.selectedEditor) {
+      maybeContinue(panel.UI.selectedEditor);
+    }
   });
 }
 
-function editorSelected(editor) {
-  const href = editor.styleSheet.href;
-  ok(
-    href.endsWith("doc_sourcemaps.scss"),
-    "selected stylesheet is correct one"
-  );
-
-  const { line } = editor.sourceEditor.getCursor();
-  is(line, 3, "cursor is at correct line number in original source");
-}
-
 function verifyLinkText(text, view) {
   info("Verifying that the rule-view stylesheet link is " + text);
   const label = getRuleViewLinkByIndex(view, 1).querySelector(
     ".ruleview-rule-source-label"
   );
-  return waitForSuccess(function() {
+  return waitForSuccess(function () {
     return label.textContent == text;
   }, "Link text changed to display correct location: " + text);
 }
