@@ -10,28 +10,35 @@ const TEST_URI =
   "browser_inspector_invalidate.js\n" +
   '<div style="width: 100px; height: 100px; background:yellow;"></div>';
 
-add_task(async function() {
-  const { inspector, testActor } = await openInspectorForURL(TEST_URI);
+add_task(async function () {
+  const { inspector, highlighterTestFront } = await openInspectorForURL(
+    TEST_URI
+  );
   const divFront = await getNodeFront("div", inspector);
 
   info("Waiting for highlighter to activate");
-  await inspector.highlighter.showBoxModel(divFront);
+  await inspector.highlighters.showHighlighterTypeForNode(
+    inspector.highlighters.TYPES.BOXMODEL,
+    divFront
+  );
 
-  let rect = await testActor.getSimpleBorderRect();
+  let rect = await highlighterTestFront.getSimpleBorderRect();
   is(rect.width, 100, "The highlighter has the right width.");
 
   info(
     "Changing the test element's size and waiting for the highlighter " +
       "to update"
   );
-  await testActor.changeHighlightedNodeWaitForUpdate(
+  await highlighterTestFront.changeHighlightedNodeWaitForUpdate(
     "style",
     "width: 200px; height: 100px; background:yellow;"
   );
 
-  rect = await testActor.getSimpleBorderRect();
+  rect = await highlighterTestFront.getSimpleBorderRect();
   is(rect.width, 200, "The highlighter has the right width after update");
 
   info("Waiting for highlighter to hide");
-  await inspector.highlighter.hideBoxModel();
+  await inspector.highlighters.hideHighlighterType(
+    inspector.highlighters.TYPES.BOXMODEL
+  );
 });
