@@ -3,18 +3,20 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const { Preferences } = require("resource://gre/modules/Preferences.jsm");
+const { Preferences } = ChromeUtils.importESModule(
+  "resource://gre/modules/Preferences.sys.mjs"
+);
 const {
   assert,
   reportException,
   isSet,
-} = require("devtools/shared/DevToolsUtils");
+} = require("resource://devtools/shared/DevToolsUtils.js");
 const {
   censusIsUpToDate,
   getSnapshot,
   createSnapshot,
   dominatorTreeIsComputed,
-} = require("devtools/client/memory/utils");
+} = require("resource://devtools/client/memory/utils.js");
 const {
   actions,
   snapshotState: states,
@@ -23,11 +25,11 @@ const {
   treeMapState,
   dominatorTreeState,
   individualsState,
-} = require("devtools/client/memory/constants");
-const view = require("devtools/client/memory/actions/view");
-const refresh = require("devtools/client/memory/actions/refresh");
-const diffing = require("devtools/client/memory/actions/diffing");
-const TaskCache = require("devtools/client/memory/actions/task-cache");
+} = require("resource://devtools/client/memory/constants.js");
+const view = require("resource://devtools/client/memory/actions/view.js");
+const refresh = require("resource://devtools/client/memory/actions/refresh.js");
+const diffing = require("resource://devtools/client/memory/actions/diffing.js");
+const TaskCache = require("resource://devtools/client/memory/actions/task-cache.js");
 
 /**
  * A series of actions are fired from this task to save, read and generate the
@@ -37,8 +39,8 @@ const TaskCache = require("devtools/client/memory/actions/task-cache");
  * @param {HeapAnalysesClient}
  * @param {Object}
  */
-exports.takeSnapshotAndCensus = function(front, heapWorker) {
-  return async function(dispatch, getState) {
+exports.takeSnapshotAndCensus = function (front, heapWorker) {
+  return async function ({ dispatch, getState }) {
     const id = await dispatch(takeSnapshot(front));
     if (id === null) {
       return;
@@ -61,11 +63,11 @@ exports.takeSnapshotAndCensus = function(front, heapWorker) {
  * @param {HeapAnalysesClient} heapWorker
  * @param {snapshotId} id
  */
-const computeSnapshotData = (exports.computeSnapshotData = function(
+const computeSnapshotData = (exports.computeSnapshotData = function (
   heapWorker,
   id
 ) {
-  return async function(dispatch, getState) {
+  return async function ({ dispatch, getState }) {
     if (getSnapshot(getState(), id).state !== states.READ) {
       return;
     }
@@ -90,8 +92,8 @@ const computeSnapshotData = (exports.computeSnapshotData = function(
  * @param {HeapAnalysesClient} heapWorker
  * @param {snapshotId} id
  */
-exports.selectSnapshotAndRefresh = function(heapWorker, id) {
-  return async function(dispatch, getState) {
+exports.selectSnapshotAndRefresh = function (heapWorker, id) {
+  return async function ({ dispatch, getState }) {
     if (getState().diffing || getState().individuals) {
       dispatch(view.changeView(viewState.CENSUS));
     }
@@ -107,8 +109,8 @@ exports.selectSnapshotAndRefresh = function(heapWorker, id) {
  * @param {MemoryFront} front
  * @returns {Number|null}
  */
-const takeSnapshot = (exports.takeSnapshot = function(front) {
-  return async function(dispatch, getState) {
+const takeSnapshot = (exports.takeSnapshot = function (front) {
+  return async function ({ dispatch, getState }) {
     if (getState().diffing || getState().individuals) {
       dispatch(view.changeView(viewState.CENSUS));
     }
@@ -144,7 +146,7 @@ const readSnapshot = (exports.readSnapshot = TaskCache.declareCacheableTask({
     return id;
   },
 
-  task: async function(heapWorker, id, removeFromCache, dispatch, getState) {
+  async task(heapWorker, id, removeFromCache, dispatch, getState) {
     const snapshot = getSnapshot(getState(), id);
     assert(
       [states.SAVED, states.IMPORTING].includes(snapshot.state),
@@ -205,7 +207,7 @@ function makeTakeCensusTask({
       return `take-census-task-${thisTakeCensusTaskId}-${id}`;
     },
 
-    task: async function(heapWorker, id, removeFromCache, dispatch, getState) {
+    async task(heapWorker, id, removeFromCache, dispatch, getState) {
       const snapshot = getSnapshot(getState(), id);
       if (!snapshot) {
         removeFromCache();
@@ -324,7 +326,7 @@ const defaultCensusTaker = takeTreeMap;
  *
  * @param {string} value from viewState
  */
-const getCurrentCensusTaker = (exports.getCurrentCensusTaker = function(
+const getCurrentCensusTaker = (exports.getCurrentCensusTaker = function (
   currentView
 ) {
   switch (currentView) {
@@ -342,7 +344,7 @@ const getCurrentCensusTaker = (exports.getCurrentCensusTaker = function(
  *
  * @param {DominatorTreeNode} node.
  */
-exports.focusIndividual = function(node) {
+exports.focusIndividual = function (node) {
   return {
     type: actions.FOCUS_INDIVIDUAL,
     node,
@@ -358,13 +360,13 @@ exports.focusIndividual = function(node) {
  * @param {Object} censusBreakdown
  * @param {Set<Number> | Number} reportLeafIndex
  */
-const fetchIndividuals = (exports.fetchIndividuals = function(
+const fetchIndividuals = (exports.fetchIndividuals = function (
   heapWorker,
   id,
   censusBreakdown,
   reportLeafIndex
 ) {
-  return async function(dispatch, getState) {
+  return async function ({ dispatch, getState }) {
     if (getState().view.state !== viewState.INDIVIDUALS) {
       dispatch(view.changeView(viewState.INDIVIDUALS));
     }
@@ -445,8 +447,8 @@ const fetchIndividuals = (exports.fetchIndividuals = function(
  *
  * @param {HeapAnalysesClient} heapWorker
  */
-exports.refreshIndividuals = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.refreshIndividuals = function (heapWorker) {
+  return async function ({ dispatch, getState }) {
     assert(
       getState().view.state === viewState.INDIVIDUALS,
       "Should be in INDIVIDUALS view."
@@ -492,8 +494,8 @@ exports.refreshIndividuals = function(heapWorker) {
  *
  * @param {HeapAnalysesClient} heapWorker
  */
-exports.refreshSelectedCensus = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.refreshSelectedCensus = function (heapWorker) {
+  return async function ({ dispatch, getState }) {
     const snapshot = getState().snapshots.find(s => s.selected);
     if (!snapshot || snapshot.state !== states.READ) {
       return;
@@ -520,8 +522,8 @@ exports.refreshSelectedCensus = function(heapWorker) {
  *
  * @param {HeapAnalysesClient} heapWorker
  */
-exports.refreshSelectedTreeMap = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.refreshSelectedTreeMap = function (heapWorker) {
+  return async function ({ dispatch, getState }) {
     const snapshot = getState().snapshots.find(s => s.selected);
     if (!snapshot || snapshot.state !== states.READ) {
       return;
@@ -551,13 +553,13 @@ exports.refreshSelectedTreeMap = function(heapWorker) {
  *
  * @returns {Promise<DominatorTreeId>}
  */
-const computeDominatorTree = (exports.computeDominatorTree = TaskCache.declareCacheableTask(
-  {
+const computeDominatorTree = (exports.computeDominatorTree =
+  TaskCache.declareCacheableTask({
     getCacheKey(_, id) {
       return id;
     },
 
-    task: async function(heapWorker, id, removeFromCache, dispatch, getState) {
+    async task(heapWorker, id, removeFromCache, dispatch, getState) {
       const snapshot = getSnapshot(getState(), id);
       assert(
         !snapshot.dominatorTree?.dominatorTreeId,
@@ -584,8 +586,7 @@ const computeDominatorTree = (exports.computeDominatorTree = TaskCache.declareCa
       });
       return dominatorTreeId;
     },
-  }
-));
+  }));
 
 /**
  * Get the partial subtree, starting from the root, of the
@@ -596,13 +597,13 @@ const computeDominatorTree = (exports.computeDominatorTree = TaskCache.declareCa
  *
  * @returns {Promise<DominatorTreeNode>}
  */
-const fetchDominatorTree = (exports.fetchDominatorTree = TaskCache.declareCacheableTask(
-  {
+const fetchDominatorTree = (exports.fetchDominatorTree =
+  TaskCache.declareCacheableTask({
     getCacheKey(_, id) {
       return id;
     },
 
-    task: async function(heapWorker, id, removeFromCache, dispatch, getState) {
+    async task(heapWorker, id, removeFromCache, dispatch, getState) {
       const snapshot = getSnapshot(getState(), id);
       assert(
         dominatorTreeIsComputed(snapshot),
@@ -642,8 +643,7 @@ const fetchDominatorTree = (exports.fetchDominatorTree = TaskCache.declareCachea
       dispatch({ type: actions.FETCH_DOMINATOR_TREE_END, id, root });
       return root;
     },
-  }
-));
+  }));
 
 /**
  * Fetch the immediately dominated children represented by the placeholder
@@ -658,7 +658,7 @@ exports.fetchImmediatelyDominated = TaskCache.declareCacheableTask({
     return `${id}-${lazyChildren.key()}`;
   },
 
-  task: async function(
+  async task(
     heapWorker,
     id,
     lazyChildren,
@@ -722,13 +722,13 @@ exports.fetchImmediatelyDominated = TaskCache.declareCacheableTask({
  *
  * @returns {Promise<DominatorTreeNode>}
  */
-const computeAndFetchDominatorTree = (exports.computeAndFetchDominatorTree = TaskCache.declareCacheableTask(
-  {
+const computeAndFetchDominatorTree = (exports.computeAndFetchDominatorTree =
+  TaskCache.declareCacheableTask({
     getCacheKey(_, id) {
       return id;
     },
 
-    task: async function(heapWorker, id, removeFromCache, dispatch, getState) {
+    async task(heapWorker, id, removeFromCache, dispatch, getState) {
       const dominatorTreeId = await dispatch(
         computeDominatorTree(heapWorker, id)
       );
@@ -746,16 +746,15 @@ const computeAndFetchDominatorTree = (exports.computeAndFetchDominatorTree = Tas
 
       return root;
     },
-  }
-));
+  }));
 
 /**
  * Update the currently selected snapshot's dominator tree.
  *
  * @param {HeapAnalysesClient} heapWorker
  */
-exports.refreshSelectedDominatorTree = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.refreshSelectedDominatorTree = function (heapWorker) {
+  return async function ({ dispatch, getState }) {
     const snapshot = getState().snapshots.find(s => s.selected);
     if (!snapshot) {
       return;
@@ -791,7 +790,7 @@ exports.refreshSelectedDominatorTree = function(heapWorker) {
  * @param {snapshotId} id
  * @see {Snapshot} model defined in devtools/client/memory/models.js
  */
-const selectSnapshot = (exports.selectSnapshot = function(id) {
+const selectSnapshot = (exports.selectSnapshot = function (id) {
   return {
     type: actions.SELECT_SNAPSHOT,
     id,
@@ -803,8 +802,8 @@ const selectSnapshot = (exports.selectSnapshot = function(id) {
  *
  * @param {HeapAnalysesClient} heapWorker
  */
-exports.clearSnapshots = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.clearSnapshots = function (heapWorker) {
+  return async function ({ dispatch, getState }) {
     const snapshots = getState().snapshots.filter(s => {
       const snapshotReady = s.state === states.READ || s.state === states.ERROR;
       const censusReady =
@@ -844,8 +843,8 @@ exports.clearSnapshots = function(heapWorker) {
  * @param {HeapAnalysesClient} heapWorker
  * @param {snapshotModel} snapshot
  */
-exports.deleteSnapshot = function(heapWorker, snapshot) {
-  return async function(dispatch, getState) {
+exports.deleteSnapshot = function (heapWorker, snapshot) {
+  return async function ({ dispatch, getState }) {
     dispatch({ type: actions.DELETE_SNAPSHOTS_START, ids: [snapshot.id] });
 
     try {
@@ -864,7 +863,7 @@ exports.deleteSnapshot = function(heapWorker, snapshot) {
  *
  * @param {CensusTreeNode} node
  */
-exports.expandCensusNode = function(id, node) {
+exports.expandCensusNode = function (id, node) {
   return {
     type: actions.EXPAND_CENSUS_NODE,
     id,
@@ -877,7 +876,7 @@ exports.expandCensusNode = function(id, node) {
  *
  * @param {CensusTreeNode} node
  */
-exports.collapseCensusNode = function(id, node) {
+exports.collapseCensusNode = function (id, node) {
   return {
     type: actions.COLLAPSE_CENSUS_NODE,
     id,
@@ -891,7 +890,7 @@ exports.collapseCensusNode = function(id, node) {
  * @param {SnapshotId} id
  * @param {DominatorTreeNode} node
  */
-exports.focusCensusNode = function(id, node) {
+exports.focusCensusNode = function (id, node) {
   return {
     type: actions.FOCUS_CENSUS_NODE,
     id,
@@ -904,7 +903,7 @@ exports.focusCensusNode = function(id, node) {
  *
  * @param {DominatorTreeTreeNode} node
  */
-exports.expandDominatorTreeNode = function(id, node) {
+exports.expandDominatorTreeNode = function (id, node) {
   return {
     type: actions.EXPAND_DOMINATOR_TREE_NODE,
     id,
@@ -917,7 +916,7 @@ exports.expandDominatorTreeNode = function(id, node) {
  *
  * @param {DominatorTreeTreeNode} node
  */
-exports.collapseDominatorTreeNode = function(id, node) {
+exports.collapseDominatorTreeNode = function (id, node) {
   return {
     type: actions.COLLAPSE_DOMINATOR_TREE_NODE,
     id,
@@ -931,7 +930,7 @@ exports.collapseDominatorTreeNode = function(id, node) {
  * @param {SnapshotId} id
  * @param {DominatorTreeNode} node
  */
-exports.focusDominatorTreeNode = function(id, node) {
+exports.focusDominatorTreeNode = function (id, node) {
   return {
     type: actions.FOCUS_DOMINATOR_TREE_NODE,
     id,
