@@ -6,24 +6,30 @@
 
 const {
   connect,
-} = require("devtools/client/shared/redux/visibility-handler-connect");
-const { Component } = require("devtools/client/shared/vendor/react");
-const PropTypes = require("devtools/client/shared/vendor/react-prop-types");
-const dom = require("devtools/client/shared/vendor/react-dom-factories");
-const { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
+} = require("resource://devtools/client/shared/redux/visibility-handler-connect.js");
+const {
+  Component,
+} = require("resource://devtools/client/shared/vendor/react.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
+const {
+  L10N,
+} = require("resource://devtools/client/netmonitor/src/utils/l10n.js");
 const {
   getNetMonitorTimingsURL,
-} = require("devtools/client/netmonitor/src/utils/mdn-utils");
+} = require("resource://devtools/client/netmonitor/src/utils/doc-utils.js");
 const {
   fetchNetworkUpdatePacket,
-} = require("devtools/client/netmonitor/src/utils/request-utils");
+} = require("resource://devtools/client/netmonitor/src/utils/request-utils.js");
 const {
   getFormattedTime,
-} = require("devtools/client/netmonitor/src/utils/format-utils");
-const { TIMING_KEYS } = require("devtools/client/netmonitor/src/constants");
+} = require("resource://devtools/client/netmonitor/src/utils/format-utils.js");
+const {
+  TIMING_KEYS,
+} = require("resource://devtools/client/netmonitor/src/constants.js");
 
 // Components
-const MDNLink = require("devtools/client/shared/components/MdnLink");
+const MDNLink = require("resource://devtools/client/shared/components/MdnLink.js");
 
 const { div, span } = dom;
 
@@ -47,7 +53,8 @@ class TimingsPanel extends Component {
     fetchNetworkUpdatePacket(connector.requestData, request, ["eventTimings"]);
   }
 
-  componentWillReceiveProps(nextProps) {
+  // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
+  UNSAFE_componentWillReceiveProps(nextProps) {
     const { connector, request } = nextProps;
     fetchNetworkUpdatePacket(connector.requestData, request, ["eventTimings"]);
   }
@@ -82,15 +89,17 @@ class TimingsPanel extends Component {
             span({
               className: "requests-list-timings-offset",
               style: {
-                width: `calc(${(totalTime - duration) /
-                  totalTime} * (100% - ${TIMINGS_END_PADDING})`,
+                width: `calc(${
+                  (totalTime - duration) / totalTime
+                } * (100% - ${TIMINGS_END_PADDING})`,
               },
             }),
             span({
               className: `requests-list-timings-box server-timings-color-${color}`,
               style: {
-                width: `calc(${duration /
-                  totalTime} * (100% - ${TIMINGS_END_PADDING}))`,
+                width: `calc(${
+                  duration / totalTime
+                } * (100% - ${TIMINGS_END_PADDING}))`,
               },
             }),
             span(
