@@ -13,15 +13,16 @@ const HAR_PATH = ["har", "logs", HAR_FILENAME];
  * Note that the `enableAutoExportToFile` is set from browser-harautomation.ini
  * because the preference needs to be set before starting the browser.
  */
-add_task(async function() {
+add_task(async function () {
   // Set a simple test filename for the exported HAR.
   await pushPref("devtools.netmonitor.har.defaultFileName", "test_filename");
 
   const tab = await addTab(SIMPLE_URL);
-  const target = await TargetFactory.forTab(tab);
-  const toolbox = await gDevTools.showToolbox(target, "inspector");
+  const toolbox = await gDevTools.showToolboxForTab(tab, {
+    toolId: "inspector",
+  });
 
-  tab.linkedBrowser.reload();
+  await reloadBrowser();
 
   info("Wait until the HAR file is created in the profile directory");
   await waitUntil(() => FileUtils.getFile("ProfD", HAR_PATH).exists());
