@@ -1,5 +1,3 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -37,8 +35,6 @@
 
 "use strict";
 
-const Services = require("Services");
-
 const Curl = {
   /**
    * Generates a cURL command string which can be used from the command line etc.
@@ -52,82 +48,122 @@ const Curl = {
    *        - httpVersion:string, http protocol version rfc2616 formatted. Eg. "HTTP/1.1"
    *        - postDataText:string, optional - the request payload.
    *
+   * @param string platform
+   *        Optional parameter to override platform,
+   *        Fallbacks to current platform if not defined.
+   *
    * @return string
    *         A cURL command.
    */
-  generateCommand: function (data) {
-    let utils = CurlUtils;
+  generateCommand(data, platform) {
+    const utils = CurlUtils;
 
     let command = ["curl"];
-    let ignoredHeaders = new Set();
+
+    // Make sure to use the following helpers to sanitize arguments before execution.
+    const addParam = value => {
+      const safe = /^[a-zA-Z-]+$/.test(value) ? value : escapeString(value);
+      command.push(safe);
+    };
+
+    const addPostData = value => {
+      const safe = /^[a-zA-Z-]+$/.test(value) ? value : escapeString(value);
+      postData.push(safe);
+    };
+
+    const ignoredHeaders = new Set();
+    const currentPlatform = platform || Services.appinfo.OS;
 
     // The cURL command is expected to run on the same platform that Firefox runs
     // (it may be different from the inspected page platform).
-    let escapeString = Services.appinfo.OS == "WINNT" ?
-                       utils.escapeStringWin : utils.escapeStringPosix;
+    const escapeString =
+      currentPlatform == "WINNT"
+        ? utils.escapeStringWin
+        : utils.escapeStringPosix;
 
     // Add URL.
-    command.push(escapeString(data.url));
+    addParam(data.url);
 
-    let postDataText = null;
-    let multipartRequest = utils.isMultipartRequest(data);
-
-    // Create post data.
-    let postData = [];
-    if (utils.isUrlEncodedRequest(data) ||
-          ["PUT", "POST", "PATCH"].includes(data.method)) {
-      postDataText = data.postDataText;
-      postData.push("--data");
-      postData.push(escapeString(utils.writePostDataTextParams(postDataText)));
-      ignoredHeaders.add("content-length");
-    } else if (multipartRequest) {
-      postDataText = data.postDataText;
-      postData.push("--data-binary");
-      let boundary = utils.getMultipartBoundary(data);
-      let text = utils.removeBinaryDataFromMultipartText(postDataText, boundary);
-      postData.push(escapeString(text));
-      ignoredHeaders.add("content-length");
+    // Disable globbing if the URL contains brackets.
+    // cURL also globs braces but they are already percent-encoded.
+    if (data.url.includes("[") || data.url.includes("]")) {
+      addParam("--globoff");
     }
 
-    // Add method.
-    // For GET and POST requests this is not necessary as GET is the
-    // default. If --data or --binary is added POST is the default.
-    if (!(data.method == "GET" || data.method == "POST")) {
-      command.push("-X");
-      command.push(data.method);
+    let postDataText = null;
+    const multipartRequest = utils.isMultipartRequest(data);
+
+    // Create post data.
+    const postData = [];
+    if (multipartRequest) {
+      // WINDOWS KNOWN LIMITATIONS: Due to the specificity of running curl on
+      // cmd.exe even correctly escaped windows newline \r\n will be
+      // treated by curl as plain local newline. It corresponds in unix
+      // to single \n and that's what curl will send in payload.
+      // It may be particularly hurtful for multipart/form-data payloads
+      // which composed using \n only, not \r\n, may be not parsable for
+      // peers which split parts of multipart payload using \r\n.
+      postDataText = data.postDataText;
+      addPostData("--data-binary");
+      const boundary = utils.getMultipartBoundary(data);
+      const text = utils.removeBinaryDataFromMultipartText(
+        postDataText,
+        boundary
+      );
+      addPostData(text);
+      ignoredHeaders.add("content-length");
+    } else if (
+      data.postDataText &&
+      (utils.isUrlEncodedRequest(data) ||
+        ["PUT", "POST", "PATCH"].includes(data.method))
+    ) {
+      // When no postData exists, --data-raw should not be set
+      postDataText = data.postDataText;
+      addPostData("--data-raw");
+      addPostData(utils.writePostDataTextParams(postDataText));
+      ignoredHeaders.add("content-length");
+    }
+    // curl generates the host header itself based on the given URL
+    ignoredHeaders.add("host");
+
+    // Add --compressed if the response is compressed
+    if (utils.isContentEncodedResponse(data)) {
+      addParam("--compressed");
     }
 
     // Add -I (HEAD)
     // For servers that supports HEAD.
     // This will fetch the header of a document only.
-    if (data.method == "HEAD") {
-      command.push("-I");
+    if (data.method === "HEAD") {
+      addParam("-I");
+    } else if (data.method !== "GET") {
+      // Add method.
+      // For HEAD and GET requests this is not necessary. GET is the
+      // default, -I implies HEAD.
+      addParam("-X");
+      addParam(data.method);
     }
 
     // Add request headers.
     let headers = data.headers;
     if (multipartRequest) {
-      let multipartHeaders = utils.getHeadersFromMultipartText(postDataText);
+      const multipartHeaders = utils.getHeadersFromMultipartText(postDataText);
       headers = headers.concat(multipartHeaders);
     }
     for (let i = 0; i < headers.length; i++) {
-      let header = headers[i];
-      if (header.name.toLowerCase() === "accept-encoding") {
-        command.push("--compressed");
-        continue;
-      }
+      const header = headers[i];
       if (ignoredHeaders.has(header.name.toLowerCase())) {
         continue;
       }
-      command.push("-H");
-      command.push(escapeString(header.name + ": " + header.value));
+      addParam("-H");
+      addParam(header.name + ": " + header.value);
     }
 
     // Add post data.
     command = command.concat(postData);
 
     return command.join(" ");
-  }
+  },
 };
 
 exports.Curl = Curl;
@@ -144,21 +180,25 @@ const CurlUtils = {
    * @return boolean
    *         True if the request is URL encoded, false otherwise.
    */
-  isUrlEncodedRequest: function (data) {
+  isUrlEncodedRequest(data) {
     let postDataText = data.postDataText;
     if (!postDataText) {
       return false;
     }
 
     postDataText = postDataText.toLowerCase();
-    if (postDataText.includes("content-type: application/x-www-form-urlencoded")) {
+    if (
+      postDataText.includes("content-type: application/x-www-form-urlencoded")
+    ) {
       return true;
     }
 
-    let contentType = this.findHeader(data.headers, "content-type");
+    const contentType = this.findHeader(data.headers, "content-type");
 
-    return (contentType &&
-      contentType.toLowerCase().includes("application/x-www-form-urlencoded"));
+    return (
+      contentType &&
+      contentType.toLowerCase().includes("application/x-www-form-urlencoded")
+    );
   },
 
   /**
@@ -169,7 +209,7 @@ const CurlUtils = {
    * @return boolean
    *         True if the request is multipart reqeust, false otherwise.
    */
-  isMultipartRequest: function (data) {
+  isMultipartRequest(data) {
     let postDataText = data.postDataText;
     if (!postDataText) {
       return false;
@@ -180,10 +220,23 @@ const CurlUtils = {
       return true;
     }
 
-    let contentType = this.findHeader(data.headers, "content-type");
+    const contentType = this.findHeader(data.headers, "content-type");
 
-    return (contentType &&
-      contentType.toLowerCase().includes("multipart/form-data;"));
+    return (
+      contentType && contentType.toLowerCase().includes("multipart/form-data;")
+    );
+  },
+
+  /**
+   * Check if the response of an URL has content encoding header.
+   *
+   * @param object data
+   *        The data source. See the description in the Curl object.
+   * @return boolean
+   *         True if the response is compressed, false otherwise.
+   */
+  isContentEncodedResponse(data) {
+    return !!this.findHeader(data.responseHeaders, "content-encoding");
   },
 
   /**
@@ -194,11 +247,11 @@ const CurlUtils = {
    * @return string
    *         Post data parameters.
    */
-  writePostDataTextParams: function (postDataText) {
+  writePostDataTextParams(postDataText) {
     if (!postDataText) {
       return "";
     }
-    let lines = postDataText.split("\r\n");
+    const lines = postDataText.split("\r\n");
     return lines[lines.length - 1];
   },
 
@@ -212,13 +265,13 @@ const CurlUtils = {
    * @return string
    *         The found header value or null if not found.
    */
-  findHeader: function (headers, name) {
+  findHeader(headers, name) {
     if (!headers) {
       return null;
     }
 
     name = name.toLowerCase();
-    for (let header of headers) {
+    for (const header of headers) {
       if (name == header.name.toLowerCase()) {
         return header.value;
       }
@@ -235,18 +288,18 @@ const CurlUtils = {
    * @return string
    *         The boundary string for the request.
    */
-  getMultipartBoundary: function (data) {
-    let boundaryRe = /\bboundary=(-{3,}\w+)/i;
+  getMultipartBoundary(data) {
+    const boundaryRe = /\bboundary=(-{3,}\w+)/i;
 
     // Get the boundary string from the Content-Type request header.
-    let contentType = this.findHeader(data.headers, "Content-Type");
+    const contentType = this.findHeader(data.headers, "Content-Type");
     if (boundaryRe.test(contentType)) {
       return contentType.match(boundaryRe)[1];
     }
     // Temporary workaround. As of 2014-03-11 the requestHeaders array does not
     // always contain the Content-Type header for mulitpart requests. See bug 978144.
     // Find the header from the request payload.
-    let boundaryString = data.postDataText.match(boundaryRe)[1];
+    const boundaryString = data.postDataText.match(boundaryRe)[1];
     if (boundaryString) {
       return boundaryString;
     }
@@ -264,11 +317,11 @@ const CurlUtils = {
    * @return string
    *         The multipart text without the binary data.
    */
-  removeBinaryDataFromMultipartText: function (multipartText, boundary) {
+  removeBinaryDataFromMultipartText(multipartText, boundary) {
     let result = "";
     boundary = "--" + boundary;
-    let parts = multipartText.split(boundary);
-    for (let part of parts) {
+    const parts = multipartText.split(boundary);
+    for (const part of parts) {
       // Each part is expected to have a content disposition line.
       let contentDispositionLine = part.trimLeft().split("\r\n")[0];
       if (!contentDispositionLine) {
@@ -279,10 +332,10 @@ const CurlUtils = {
         if (contentDispositionLine.includes("filename=")) {
           // The header lines and the binary blob is separated by 2 CRLF's.
           // Add only the headers to the result.
-          let headers = part.split("\r\n\r\n")[0];
-          result += boundary + "\r\n" + headers + "\r\n\r\n";
+          const headers = part.split("\r\n\r\n")[0];
+          result += boundary + headers + "\r\n\r\n";
         } else {
-          result += boundary + "\r\n" + part;
+          result += boundary + part;
         }
       }
     }
@@ -299,24 +352,24 @@ const CurlUtils = {
    * @return array
    *         An array of header objects {name:x, value:x}
    */
-  getHeadersFromMultipartText: function (multipartText) {
-    let headers = [];
+  getHeadersFromMultipartText(multipartText) {
+    const headers = [];
     if (!multipartText || multipartText.startsWith("---")) {
       return headers;
     }
 
     // Get the header section.
-    let index = multipartText.indexOf("\r\n\r\n");
+    const index = multipartText.indexOf("\r\n\r\n");
     if (index == -1) {
       return headers;
     }
 
     // Parse the header lines.
-    let headersText = multipartText.substring(0, index);
-    let headerLines = headersText.split("\r\n");
+    const headersText = multipartText.substring(0, index);
+    const headerLines = headersText.split("\r\n");
     let lastHeaderName = null;
 
-    for (let line of headerLines) {
+    for (const line of headerLines) {
       // Create a header for each line in fields that spans across multiple lines.
       // Subsquent lines always begins with at least one space or tab character.
       // (rfc2616)
@@ -325,12 +378,15 @@ const CurlUtils = {
         continue;
       }
 
-      let indexOfColon = line.indexOf(":");
+      const indexOfColon = line.indexOf(":");
       if (indexOfColon == -1) {
         continue;
       }
 
-      let header = [line.slice(0, indexOfColon), line.slice(indexOfColon + 1)];
+      const header = [
+        line.slice(0, indexOfColon),
+        line.slice(indexOfColon + 1),
+      ];
       if (header.length != 2) {
         continue;
       }
@@ -345,12 +401,14 @@ const CurlUtils = {
    * Escape util function for POSIX oriented operating systems.
    * Credit: Google DevTools
    */
-  escapeStringPosix: function (str) {
+  escapeStringPosix(str) {
     function escapeCharacter(x) {
       let code = x.charCodeAt(0);
       if (code < 256) {
         // Add leading zero when needed to not care about the next character.
-        return code < 16 ? "\\x0" + code.toString(16) : "\\x" + code.toString(16);
+        return code < 16
+          ? "\\x0" + code.toString(16)
+          : "\\x" + code.toString(16);
       }
       code = code.toString(16);
       return "\\u" + ("0000" + code).substr(code.length, 4);
@@ -358,11 +416,17 @@ const CurlUtils = {
 
     if (/[^\x20-\x7E]|\'/.test(str)) {
       // Use ANSI-C quoting syntax.
-      return "$\'" + str.replace(/\\/g, "\\\\")
-                        .replace(/\'/g, "\\\'")
-                        .replace(/\n/g, "\\n")
-                        .replace(/\r/g, "\\r")
-                        .replace(/[^\x20-\x7E]/g, escapeCharacter) + "'";
+      return (
+        "$'" +
+        str
+          .replace(/\\/g, "\\\\")
+          .replace(/\'/g, "\\'")
+          .replace(/\n/g, "\\n")
+          .replace(/\r/g, "\\r")
+          .replace(/!/g, "\\041")
+          .replace(/[^\x20-\x7E]/g, escapeCharacter) +
+        "'"
+      );
     }
 
     // Use single quote syntax.
@@ -373,26 +437,53 @@ const CurlUtils = {
    * Escape util function for Windows systems.
    * Credit: Google DevTools
    */
-  escapeStringWin: function (str) {
-    /* Replace quote by double quote (but not by \") because it is
-       recognized by both cmd.exe and MS Crt arguments parser.
-
-       Replace % by "%" because it could be expanded to an environment
-       variable value. So %% becomes "%""%". Even if an env variable ""
-       (2 doublequotes) is declared, the cmd.exe will not
-       substitute it with its value.
-
-       Replace each backslash with double backslash to make sure
-       MS Crt arguments parser won't collapse them.
-
-       Replace new line outside of quotes since cmd.exe doesn't let
-       to do it inside.
+  escapeStringWin(str) {
+    /*
+      Because cmd.exe parser and MS Crt arguments parsers use some of the
+      same escape characters, they can interact with each other in
+      horrible ways, the order of operations is critical.
     */
-    return "\"" + str.replace(/"/g, "\"\"")
-                     .replace(/%/g, "\"%\"")
-                     .replace(/\\/g, "\\\\")
-                     .replace(/[\r\n]+/g, "\"^$&\"") + "\"";
-  }
+    const encapsChars = '"';
+    return (
+      encapsChars +
+      str
+
+        //  Replace \ with \\ first because it is an escape character for certain
+        // conditions in both parsers.
+        .replace(/\\/g, "\\\\")
+
+        // Replace double quote chars with two double quotes (not by escaping with \") because it is
+        // recognized by both cmd.exe and MS Crt arguments parser.
+        .replace(/"/g, '""')
+
+        // Escape ` and $ so commands do not get executed e.g $(calc.exe) or `\$(calc.exe)
+        .replace(/[`$]/g, "\\$&")
+
+        // Then escape all characters we are not sure about with ^ to ensure it
+        // gets to MS Crt parser safely.
+        .replace(/[^a-zA-Z0-9\s_\-:=+~\/.',?;()*\$&\\{}\"`]/g, "^$&")
+
+        // The % character is special because MS Crt parser will try and look for
+        // ENV variables and fill them in its place. We cannot escape them with %
+        // and cannot escape them with ^ (because it's cmd.exe's escape not MS Crt
+        // parser); So we can get cmd.exe parser to escape the character after it,
+        // if it is followed by a valid beginning character of an ENV variable.
+        // This ensures we do not try and double escape another ^ if it was placed
+        // by the previous replace.
+        .replace(/%(?=[a-zA-Z0-9_])/g, "%^")
+
+        // We replace \r and \r\n with \n, this allows to consistently escape all new
+        // lines in the next replace
+        .replace(/\r\n?/g, "\n")
+
+        // Lastly we replace new lines with ^ and TWO new lines because the first
+        // new line is there to enact the escape command the second is the character
+        // to escape (in this case new line).
+        // The extra " enables escaping new lines with ^ within quotes in cmd.exe.
+        .replace(/\n/g, '"^\r\n\r\n"') +
+      encapsChars
+    );
+  },
 };
 
 exports.CurlUtils = CurlUtils;
