@@ -6,47 +6,55 @@
 const {
   Component,
   createElement,
-} = require("devtools/client/shared/vendor/react");
-const dom = require("devtools/client/shared/vendor/react-dom-factories");
+  createRef,
+} = require("resource://devtools/client/shared/vendor/react.js");
+const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
 const {
   connect,
-} = require("devtools/client/shared/redux/visibility-handler-connect");
-const { initialize } = require("devtools/client/webconsole/actions/ui");
+} = require("resource://devtools/client/shared/redux/visibility-handler-connect.js");
+const {
+  initialize,
+} = require("resource://devtools/client/webconsole/actions/ui.js");
+const LazyMessageList = require("resource://devtools/client/webconsole/components/Output/LazyMessageList.js");
 
 const {
-  getAllMessagesById,
+  getMutableMessagesById,
   getAllMessagesUiById,
-  getAllMessagesPayloadById,
+  getAllDisabledMessagesById,
+  getAllCssMessagesMatchingElements,
   getAllNetworkMessagesUpdateById,
+  getLastMessageId,
   getVisibleMessages,
   getAllRepeatById,
   getAllWarningGroupsById,
   isMessageInWarningGroup,
-} = require("devtools/client/webconsole/selectors/messages");
+} = require("resource://devtools/client/webconsole/selectors/messages.js");
 
 loader.lazyRequireGetter(
   this,
   "PropTypes",
-  "devtools/client/shared/vendor/react-prop-types"
+  "resource://devtools/client/shared/vendor/react-prop-types.js"
 );
 loader.lazyRequireGetter(
   this,
   "MessageContainer",
-  "devtools/client/webconsole/components/Output/MessageContainer",
+  "resource://devtools/client/webconsole/components/Output/MessageContainer.js",
   true
 );
+loader.lazyRequireGetter(this, "flags", "resource://devtools/shared/flags.js");
 
-const { MESSAGE_TYPE } = require("devtools/client/webconsole/constants");
 const {
-  getInitialMessageCountForViewport,
-} = require("devtools/client/webconsole/utils/messages.js");
+  MESSAGE_TYPE,
+} = require("resource://devtools/client/webconsole/constants.js");
 
 class ConsoleOutput extends Component {
   static get propTypes() {
     return {
       initialized: PropTypes.bool.isRequired,
-      messages: PropTypes.object.isRequired,
+      mutableMessages: PropTypes.object.isRequired,
+      messageCount: PropTypes.number.isRequired,
       messagesUi: PropTypes.array.isRequired,
+      disabledMessages: PropTypes.array.isRequired,
       serviceContainer: PropTypes.shape({
         attachRefToWebConsoleUI: PropTypes.func.isRequired,
         openContextMenu: PropTypes.func.isRequired,
@@ -54,7 +62,7 @@ class ConsoleOutput extends Component {
       }),
       dispatch: PropTypes.func.isRequired,
       timestampsVisible: PropTypes.bool,
-      messagesPayload: PropTypes.object.isRequired,
+      cssMatchingElements: PropTypes.object.isRequired,
       messagesRepeat: PropTypes.object.isRequired,
       warningGroups: PropTypes.object.isRequired,
       networkMessagesUpdate: PropTypes.object.isRequired,
@@ -62,6 +70,9 @@ class ConsoleOutput extends Component {
       networkMessageActiveTabId: PropTypes.string.isRequired,
       onFirstMeaningfulPaint: PropTypes.func.isRequired,
       editorMode: PropTypes.bool.isRequired,
+      cacheGeneration: PropTypes.number.isRequired,
+      disableVirtualization: PropTypes.bool,
+      lastMessageId: PropTypes.string.isRequired,
     };
   }
 
@@ -69,6 +80,9 @@ class ConsoleOutput extends Component {
     super(props);
     this.onContextMenu = this.onContextMenu.bind(this);
     this.maybeScrollToBottom = this.maybeScrollToBottom.bind(this);
+    this.messageIdsToKeepAlive = new Set();
+    this.ref = createRef();
+    this.lazyMessageListRef = createRef();
 
     this.resizeObserver = new ResizeObserver(entries => {
       // If we don't have the outputNode reference, or if the outputNode isn't connected
@@ -86,25 +100,32 @@ class ConsoleOutput extends Component {
   }
 
   componentDidMount() {
-    if (this.props.visibleMessages.length > 0) {
+    if (this.props.disableVirtualization) {
+      return;
+    }
+
+    if (this.props.visibleMessages.length) {
       this.scrollToBottom();
     }
 
-    this.lastMessageIntersectionObserver = new IntersectionObserver(
+    this.scrollDetectionIntersectionObserver = new IntersectionObserver(
       entries => {
         for (const entry of entries) {
-          // Consider that we're not pinned to the bottom anymore if the last message is
-          // less than half-visible.
-          this.scrolledToBottom = entry.intersectionRatio >= 0.5;
+          // Consider that we're not pinned to the bottom anymore if the bottom of the
+          // scrollable area is within 10px of visible (half the typical element height.)
+          this.scrolledToBottom = entry.intersectionRatio > 0;
         }
       },
-      { root: this.outputNode, threshold: [0.5] }
+      { root: this.outputNode, rootMargin: "10px" }
     );
 
     this.resizeObserver.observe(this.getElementToObserve());
 
     const { serviceContainer, onFirstMeaningfulPaint, dispatch } = this.props;
-    serviceContainer.attachRefToWebConsoleUI("outputScroller", this.outputNode);
+    serviceContainer.attachRefToWebConsoleUI(
+      "outputScroller",
+      this.ref.current
+    );
 
     // Waiting for the next paint.
     new Promise(res => requestAnimationFrame(res)).then(() => {
@@ -119,7 +140,13 @@ class ConsoleOutput extends Component {
     });
   }
 
-  componentWillUpdate(nextProps, nextState) {
+  // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
+  UNSAFE_componentWillUpdate(nextProps, nextState) {
+    this.isUpdating = true;
+    if (nextProps.cacheGeneration !== this.props.cacheGeneration) {
+      this.messageIdsToKeepAlive = new Set();
+    }
+
     if (nextProps.editorMode !== this.props.editorMode) {
       this.resizeObserver.disconnect();
     }
@@ -130,11 +157,12 @@ class ConsoleOutput extends Component {
       // This makes the console stay pinned to the bottom if a batch of messages
       // are added after a page refresh (Bug 1402237).
       this.shouldScrollBottom = true;
+      this.scrolledToBottom = true;
       return;
     }
 
-    const { lastChild } = outputNode;
-    this.lastMessageIntersectionObserver.unobserve(lastChild);
+    const bottomBuffer = this.lazyMessageListRef.current.bottomBuffer;
+    this.scrollDetectionIntersectionObserver.unobserve(bottomBuffer);
 
     // We need to scroll to the bottom if:
     // - we are reacting to "initialize" action, and we are already scrolled to the bottom
@@ -145,10 +173,13 @@ class ConsoleOutput extends Component {
 
     const visibleMessagesDelta =
       nextProps.visibleMessages.length - this.props.visibleMessages.length;
-    const messagesDelta = nextProps.messages.size - this.props.messages.size;
+    const messagesDelta = nextProps.messageCount - this.props.messageCount;
+    // Evaluation results are never filtered out, so if it's in the store, it will be
+    // visible in the output.
     const isNewMessageEvaluationResult =
       messagesDelta > 0 &&
-      [...nextProps.messages.values()][nextProps.messages.size - 1].type ===
+      nextProps.lastMessageId &&
+      nextProps.mutableMessages.get(nextProps.lastMessageId)?.type ===
         MESSAGE_TYPE.RESULT;
 
     const messagesUiDelta =
@@ -172,9 +203,11 @@ class ConsoleOutput extends Component {
   }
 
   componentDidUpdate(prevProps) {
+    this.isUpdating = false;
     this.maybeScrollToBottom();
     if (this?.outputNode?.lastChild) {
-      this.lastMessageIntersectionObserver.observe(this.outputNode.lastChild);
+      const bottomBuffer = this.lazyMessageListRef.current.bottomBuffer;
+      this.scrollDetectionIntersectionObserver.observe(bottomBuffer);
     }
 
     if (prevProps.editorMode !== this.props.editorMode) {
@@ -182,13 +215,35 @@ class ConsoleOutput extends Component {
     }
   }
 
+  get outputNode() {
+    return this.ref.current;
+  }
+
   maybeScrollToBottom() {
     if (this.outputNode && this.shouldScrollBottom) {
       this.scrollToBottom();
     }
   }
 
+  // The maybeScrollToBottom callback we provide to messages needs to be a little bit more
+  // strict than the one we normally use, because they can potentially interrupt a user
+  // scroll (between when the intersection observer registers the scroll break and when
+  // a componentDidUpdate comes through to reconcile it.)
+  maybeScrollToBottomMessageCallback(index) {
+    if (
+      this.outputNode &&
+      this.shouldScrollBottom &&
+      this.scrolledToBottom &&
+      this.lazyMessageListRef.current?.isItemNearBottom(index)
+    ) {
+      this.scrollToBottom();
+    }
+  }
+
   scrollToBottom() {
+    if (flags.testing && this.outputNode.hasAttribute("disable-autoscroll")) {
+      return;
+    }
     if (this.outputNode.scrollHeight > this.outputNode.clientHeight) {
       this.outputNode.scrollTop = this.outputNode.scrollHeight;
     }
@@ -212,77 +267,106 @@ class ConsoleOutput extends Component {
   }
 
   render() {
-    let {
+    const {
+      cacheGeneration,
       dispatch,
       visibleMessages,
-      messages,
+      disabledMessages,
+      mutableMessages,
       messagesUi,
-      messagesPayload,
+      cssMatchingElements,
       messagesRepeat,
       warningGroups,
       networkMessagesUpdate,
       networkMessageActiveTabId,
       serviceContainer,
       timestampsVisible,
-      initialized,
     } = this.props;
 
-    if (!initialized) {
-      const numberMessagesFitViewport = getInitialMessageCountForViewport(
-        window
-      );
-      if (numberMessagesFitViewport < visibleMessages.length) {
-        visibleMessages = visibleMessages.slice(
-          visibleMessages.length - numberMessagesFitViewport
-        );
-      }
-    }
-
-    const messageNodes = visibleMessages.map(messageId =>
-      createElement(MessageContainer, {
+    const renderMessage = (messageId, index) => {
+      return createElement(MessageContainer, {
         dispatch,
         key: messageId,
         messageId,
         serviceContainer,
         open: messagesUi.includes(messageId),
-        payload: messagesPayload.get(messageId),
+        cssMatchingElements: cssMatchingElements.get(messageId),
         timestampsVisible,
+        disabled: disabledMessages.includes(messageId),
         repeat: messagesRepeat[messageId],
         badge: warningGroups.has(messageId)
           ? warningGroups.get(messageId).length
           : null,
         inWarningGroup:
           warningGroups && warningGroups.size > 0
-            ? isMessageInWarningGroup(messages.get(messageId), visibleMessages)
+            ? isMessageInWarningGroup(
+                mutableMessages.get(messageId),
+                visibleMessages
+              )
             : false,
         networkMessageUpdate: networkMessagesUpdate[messageId],
         networkMessageActiveTabId,
-        getMessage: () => messages.get(messageId),
-        maybeScrollToBottom: this.maybeScrollToBottom,
-      })
-    );
+        getMessage: () => mutableMessages.get(messageId),
+        maybeScrollToBottom: () =>
+          this.maybeScrollToBottomMessageCallback(index),
+        // Whenever a node is expanded, we want to make sure we keep the
+        // message node alive so as to not lose the expanded state.
+        setExpanded: () => this.messageIdsToKeepAlive.add(messageId),
+      });
+    };
 
+    // scrollOverdrawCount tells the list to draw extra elements above and
+    // below the scrollport so that we can avoid flashes of blank space
+    // when scrolling. When `disableVirtualization` is passed we make it as large as the
+    // number of messages to render them all and effectively disabling virtualization (this
+    // should only be used for some actions that requires all the messages to be rendered
+    // in the DOM, like "Copy All Messages").
+    const scrollOverdrawCount = this.props.disableVirtualization
+      ? visibleMessages.length
+      : 20;
+
+    const attrs = {
+      className: "webconsole-output",
+      role: "main",
+      onContextMenu: this.onContextMenu,
+      ref: this.ref,
+    };
+    if (flags.testing) {
+      attrs["data-visible-messages"] = JSON.stringify(visibleMessages);
+    }
     return dom.div(
-      {
-        className: "webconsole-output",
-        role: "main",
-        onContextMenu: this.onContextMenu,
-        ref: node => {
-          this.outputNode = node;
-        },
-      },
-      messageNodes
+      attrs,
+      createElement(LazyMessageList, {
+        viewportRef: this.ref,
+        items: visibleMessages,
+        itemDefaultHeight: 21,
+        editorMode: this.props.editorMode,
+        scrollOverdrawCount,
+        ref: this.lazyMessageListRef,
+        renderItem: renderMessage,
+        itemsToKeepAlive: this.messageIdsToKeepAlive,
+        serviceContainer,
+        cacheGeneration,
+        shouldScrollBottom: () => this.shouldScrollBottom && this.isUpdating,
+      })
     );
   }
 }
 
 function mapStateToProps(state, props) {
+  const mutableMessages = getMutableMessagesById(state);
   return {
     initialized: state.ui.initialized,
-    messages: getAllMessagesById(state),
+    cacheGeneration: state.ui.cacheGeneration,
+    // We need to compute this so lifecycle methods can compare the global message count
+    // on state change (since we can't do it with mutableMessagesById).
+    messageCount: mutableMessages.size,
+    mutableMessages,
+    lastMessageId: getLastMessageId(state),
     visibleMessages: getVisibleMessages(state),
+    disabledMessages: getAllDisabledMessagesById(state),
     messagesUi: getAllMessagesUiById(state),
-    messagesPayload: getAllMessagesPayloadById(state),
+    cssMatchingElements: getAllCssMessagesMatchingElements(state),
     messagesRepeat: getAllRepeatById(state),
     warningGroups: getAllWarningGroupsById(state),
     networkMessagesUpdate: getAllNetworkMessagesUpdateById(state),
