@@ -1,7 +1,6 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 /* eslint no-unused-vars: [2, {"vars": "local"}] */
-/* import-globals-from ../../../shared/test/telemetry-test-helpers.js */
 
 "use strict";
 
@@ -9,31 +8,27 @@
 
 // shared-head.js handles imports, constants, and utility functions
 // Load the shared-head file first.
-/* import-globals-from ../../../shared/test/shared-head.js */
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/shared/test/shared-head.js",
   this
 );
 
 // Import helpers for the new debugger
-/* import-globals-from ../../../debugger/test/mochitest/helpers/context.js */
 Services.scriptloader.loadSubScript(
-  "chrome://mochitests/content/browser/devtools/client/debugger/test/mochitest/helpers/context.js",
+  "chrome://mochitests/content/browser/devtools/client/debugger/test/mochitest/shared-head.js",
   this
 );
 
-// Import helpers for the new debugger
-/* import-globals-from ../../../debugger/test/mochitest/helpers.js*/
 Services.scriptloader.loadSubScript(
-  "chrome://mochitests/content/browser/devtools/client/debugger/test/mochitest/helpers.js",
+  "chrome://mochitests/content/browser/devtools/client/webconsole/test/browser/shared-head.js",
   this
 );
 
 var {
   BrowserConsoleManager,
-} = require("devtools/client/webconsole/browser-console-manager");
+} = require("resource://devtools/client/webconsole/browser-console-manager.js");
 
-var WCUL10n = require("devtools/client/webconsole/utils/l10n");
+var WCUL10n = require("resource://devtools/client/webconsole/utils/l10n.js");
 const DOCS_GA_PARAMS = `?${new URLSearchParams({
   utm_source: "mozilla",
   utm_medium: "firefox-console-errors",
@@ -45,9 +40,13 @@ const GA_PARAMS = `?${new URLSearchParams({
   utm_campaign: "default",
 })}`;
 
-const wcActions = require("devtools/client/webconsole/actions/index");
+const wcActions = require("resource://devtools/client/webconsole/actions/index.js");
+
+registerCleanupFunction(async function () {
+  // Reset all cookies, tests loading sjs_slow-response-test-server.sjs will
+  // set a foo cookie which might have side effects on other tests.
+  Services.cookies.removeAll();
 
-registerCleanupFunction(async function() {
   Services.prefs.clearUserPref("devtools.webconsole.ui.filterbar");
 
   // Reset all filter prefs between tests. First flushPrefEnv in case one of the
@@ -99,22 +98,24 @@ async function openNewTabWithIframesAndConsole(tabUrl, iframes) {
   // to handle remote frames (we don't support creating frames target when the toolbox
   // is already open).
   await addTab(tabUrl);
-  await ContentTask.spawn(gBrowser.selectedBrowser, iframes, async function(
-    urls
-  ) {
-    const iframesLoadPromises = urls.map((url, i) => {
-      const iframe = content.document.createElement("iframe");
-      iframe.classList.add(`iframe-${i + 1}`);
-      const onLoadIframe = new Promise(resolve => {
-        iframe.addEventListener("load", resolve, { once: true });
+  await ContentTask.spawn(
+    gBrowser.selectedBrowser,
+    iframes,
+    async function (urls) {
+      const iframesLoadPromises = urls.map((url, i) => {
+        const iframe = content.document.createElement("iframe");
+        iframe.classList.add(`iframe-${i + 1}`);
+        const onLoadIframe = new Promise(resolve => {
+          iframe.addEventListener("load", resolve, { once: true });
+        });
+        content.document.body.append(iframe);
+        iframe.src = url;
+        return onLoadIframe;
       });
-      content.document.body.append(iframe);
-      iframe.src = url;
-      return onLoadIframe;
-    });
 
-    await Promise.all(iframesLoadPromises);
-  });
+      await Promise.all(iframesLoadPromises);
+    }
+  );
 
   return openConsole();
 }
@@ -148,7 +149,9 @@ function logAllStoreChanges(hud) {
   // Adding logging each time the store is modified in order to check
   // the store state in case of failure.
   store.subscribe(() => {
-    const messages = [...store.getState().messages.messagesById.values()];
+    const messages = [
+      ...store.getState().messages.mutableMessagesById.values(),
+    ];
     const debugMessages = messages.map(
       ({ id, type, parameters, messageText }) => {
         return { id, type, parameters, messageText };
@@ -156,7 +159,7 @@ function logAllStoreChanges(hud) {
     );
     info(
       "messages : " +
-        JSON.stringify(debugMessages, function(key, value) {
+        JSON.stringify(debugMessages, function (key, value) {
           if (value && value.getGrip) {
             return value.getGrip();
           }
@@ -167,17 +170,20 @@ function logAllStoreChanges(hud) {
 }
 
 /**
- * Wait for messages in the web console output, resolving once they are received.
+ * Wait for messages with given message type in the web console output,
+ * resolving once they are received.
  *
  * @param object options
  *        - hud: the webconsole
  *        - messages: Array[Object]. An array of messages to match.
-            Current supported options:
- *            - text: Partial text match in .message-body
- *        - selector: {String} a selector that should match the message node. Defaults to
- *                             ".message".
+ *          Current supported options:
+ *            - text: {String} Partial text match in .message-body
+ *            - typeSelector: {String} A part of selector for the message, to
+ *                                     specify the message type.
+ * @return promise
+ *         A promise that is resolved to an array of the message nodes
  */
-function waitForMessages({ hud, messages, selector = ".message" }) {
+function waitForMessagesByType({ hud, messages }) {
   return new Promise(resolve => {
     const matchedMessages = [];
     hud.ui.on("new-messages", function messagesReceived(newMessages) {
@@ -186,6 +192,17 @@ function waitForMessages({ hud, messages, selector = ".message" }) {
           continue;
         }
 
+        const typeSelector = message.typeSelector;
+        if (!typeSelector) {
+          throw new Error("typeSelector property is required");
+        }
+        if (!typeSelector.startsWith(".")) {
+          throw new Error(
+            "typeSelector property start with a dot e.g. `.result`"
+          );
+        }
+        const selector = ".message" + typeSelector;
+
         for (const newMessage of newMessages) {
           const messageBody = newMessage.node.querySelector(`.message-body`);
           if (
@@ -221,12 +238,14 @@ function waitForMessages({ hud, messages, selector = ".message" }) {
  *
  * @param {Object} hud : the webconsole
  * @param {String} text : text included in .message-body
+ * @param {String} typeSelector : A part of selector for the message, to
+ *                                specify the message type.
  * @param {Number} repeat : expected repeat count in .message-repeats
  */
-function waitForRepeatedMessage(hud, text, repeat) {
+function waitForRepeatedMessageByType(hud, text, typeSelector, repeat) {
   return waitFor(() => {
     // Wait for a message matching the provided text.
-    const node = findMessage(hud, text);
+    const node = findMessageByType(hud, text, typeSelector);
     if (!node) {
       return false;
     }
@@ -242,17 +261,20 @@ function waitForRepeatedMessage(hud, text, repeat) {
 }
 
 /**
- * Wait for a single message in the web console output, resolving once it is received.
+ * Wait for a single message with given message type in the web console output,
+ * resolving with the first message that matches the query once it is received.
  *
  * @param {Object} hud : the webconsole
  * @param {String} text : text included in .message-body
- * @param {String} selector : A selector that should match the message node.
+ * @param {String} typeSelector : A part of selector for the message, to
+ *                                specify the message type.
+ * @return promise
+ *         A promise that is resolved to the message node
  */
-async function waitForMessage(hud, text, selector) {
-  const messages = await waitForMessages({
+async function waitForMessageByType(hud, text, typeSelector) {
+  const messages = await waitForMessagesByType({
     hud,
-    messages: [{ text }],
-    selector,
+    messages: [{ text, typeSelector }],
   });
   return messages[0];
 }
@@ -268,44 +290,63 @@ function execute(hud, input) {
 }
 
 /**
- * Execute an input expression and wait for a message with the expected text (and an
- * optional selector) to be displayed in the output.
+ * Execute an input expression and wait for a message with the expected text
+ * with given message type to be displayed in the output.
  *
  * @param {Object} hud : The webconsole.
  * @param {String} input : The input expression to execute.
- * @param {String} matchingText : A string that should match the message body content.
- * @param {String} selector : A selector that should match the message node.
+ * @param {String} matchingText : A string that should match the message body content.
+ * @param {String} typeSelector : A part of selector for the message, to
+ *                                specify the message type.
  */
-function executeAndWaitForMessage(
+function executeAndWaitForMessageByType(
   hud,
   input,
   matchingText,
-  selector = ".message"
+  typeSelector
 ) {
-  const onMessage = waitForMessage(hud, matchingText, selector);
+  const onMessage = waitForMessageByType(hud, matchingText, typeSelector);
   execute(hud, input);
   return onMessage;
 }
 
 /**
- * Set the input value, simulates the right keyboard event to evaluate it, depending on
- * if the console is in editor mode or not, and wait for a message with the expected text
- * (and an optional selector) to be displayed in the output.
+ * Type-specific wrappers for executeAndWaitForMessageByType
  *
  * @param {Object} hud : The webconsole.
  * @param {String} input : The input expression to execute.
- * @param {String} matchingText : A string that should match the message body content.
- * @param {String} selector : A selector that should match the message node.
+ * @param {String} matchingText : A string that should match the message body
+ *                                content.
  */
-function keyboardExecuteAndWaitForMessage(
+function executeAndWaitForResultMessage(hud, input, matchingText) {
+  return executeAndWaitForMessageByType(hud, input, matchingText, ".result");
+}
+
+function executeAndWaitForErrorMessage(hud, input, matchingText) {
+  return executeAndWaitForMessageByType(hud, input, matchingText, ".error");
+}
+
+/**
+ * Set the input value, simulates the right keyboard event to evaluate it,
+ * depending on if the console is in editor mode or not, and wait for a message
+ * with the expected text with given message type to be displayed in the output.
+ *
+ * @param {Object} hud : The webconsole.
+ * @param {String} input : The input expression to execute.
+ * @param {String} matchingText : A string that should match the message body
+ *                                content.
+ * @param {String} typeSelector : A part of selector for the message, to
+ *                                specify the message type.
+ */
+function keyboardExecuteAndWaitForMessageByType(
   hud,
   input,
   matchingText,
-  selector = ".message"
+  typeSelector
 ) {
   hud.jsterm.focus();
   setInputValue(hud, input);
-  const onMessage = waitForMessage(hud, matchingText, selector);
+  const onMessage = waitForMessageByType(hud, matchingText, typeSelector);
   if (isEditorModeEnabled(hud)) {
     EventUtils.synthesizeKey("KEY_Enter", {
       [Services.appinfo.OS === "Darwin" ? "metaKey" : "ctrlKey"]: true,
@@ -317,38 +358,22 @@ function keyboardExecuteAndWaitForMessage(
 }
 
 /**
- * Find a message in the output.
+ * Type-specific wrappers for keyboardExecuteAndWaitForMessageByType
  *
- * @param object hud
- *        The web console.
- * @param string text
- *        A substring that can be found in the message.
- * @param selector [optional]
- *        The selector to use in finding the message.
- * @return {Node} the node corresponding the found message
+ * @param {Object} hud : The webconsole.
+ * @param {String} input : The input expression to execute.
+ * @param {String} matchingText : A string that should match the message body
+ *                                content.
  */
-function findMessage(hud, text, selector = ".message") {
-  const elements = findMessages(hud, text, selector);
-  return elements.pop();
+function keyboardExecuteAndWaitForResultMessage(hud, input, matchingText) {
+  return keyboardExecuteAndWaitForMessageByType(
+    hud,
+    input,
+    matchingText,
+    ".result"
+  );
 }
 
-/**
- * Find multiple messages in the output.
- *
- * @param object hud
- *        The web console.
- * @param string text
- *        A substring that can be found in the message.
- * @param selector [optional]
- *        The selector to use in finding the message.
- */
-function findMessages(hud, text, selector = ".message") {
-  const messages = hud.ui.outputNode.querySelectorAll(selector);
-  const elements = Array.prototype.filter.call(messages, el =>
-    el.textContent.includes(text)
-  );
-  return elements;
-}
 /**
  * Wait for a message to be logged and ensure it is logged only once.
  *
@@ -356,27 +381,32 @@ function findMessages(hud, text, selector = ".message") {
  *        The web console.
  * @param string text
  *        A substring that can be found in the message.
- * @param selector [optional]
- *        The selector to use in finding the message.
+ * @param string typeSelector
+ *        A part of selector for the message, to specify the message type.
  * @return {Node} the node corresponding the found message
  */
-async function checkUniqueMessageExists(hud, msg, selector) {
+async function checkUniqueMessageExists(hud, msg, typeSelector) {
   info(`Checking "${msg}" was logged`);
   let messages;
   try {
-    messages = await waitFor(() => {
-      const msgs = findMessages(hud, msg, selector);
-      return msgs.length > 0 ? msgs : null;
+    messages = await waitFor(async () => {
+      const msgs = await findMessagesVirtualizedByType({
+        hud,
+        text: msg,
+        typeSelector,
+      });
+      return msgs.length ? msgs : null;
     });
   } catch (e) {
     ok(false, `Message "${msg}" wasn't logged\n`);
-    return;
+    return null;
   }
 
   is(messages.length, 1, `"${msg}" was logged once`);
   const [messageEl] = messages;
   const repeatNode = messageEl.querySelector(".message-repeats");
   is(repeatNode, null, `"${msg}" wasn't repeated`);
+  return messageEl;
 }
 
 /**
@@ -406,7 +436,7 @@ async function openContextMenu(hud, element) {
  */
 function hideContextMenu(hud) {
   const popup = _getContextMenu(hud);
-  if (!popup) {
+  if (!popup || popup.state == "hidden") {
     return Promise.resolve();
   }
 
@@ -421,6 +451,27 @@ function _getContextMenu(hud) {
   return doc.getElementById("webconsole-menu");
 }
 
+/**
+ * Toggle Enable network monitoring setting
+ *
+ *  @param object hud
+ *         The web console.
+ *  @param boolean shouldBeSwitchedOn
+ *         The expected state the setting should be in after the toggle.
+ */
+async function toggleNetworkMonitoringConsoleSetting(hud, shouldBeSwitchedOn) {
+  const selector =
+    ".webconsole-console-settings-menu-item-enableNetworkMonitoring";
+  const settingChanged = waitFor(() => {
+    const el = getConsoleSettingElement(hud, selector);
+    return shouldBeSwitchedOn
+      ? el.getAttribute("aria-checked") === "true"
+      : el.getAttribute("aria-checked") !== "true";
+  });
+  await toggleConsoleSetting(hud, selector);
+  await settingChanged;
+}
+
 async function toggleConsoleSetting(hud, selector) {
   const toolbox = hud.toolbox;
   const doc = toolbox ? toolbox.doc : hud.chromeWindow.document;
@@ -473,40 +524,42 @@ function waitForNodeMutation(node, observeConfig = {}) {
  *
  * @param {Object} hud
  *        The webconsole
- * @param {Object} toolbox
- *        The toolbox
- * @param {String} text
- *        The text to search for. This should be contained in the
- *        message. The searching is done with @see findMessage.
- * @param {boolean} expectUrl
- *        Whether the URL in the opened source should match the link, or whether
- *        it is expected to be null.
- * @param {boolean} expectLine
- *        It indicates if there is the need to check the line.
- * @param {boolean} expectColumn
- *        It indicates if there is the need to check the column.
- * @param {String} logPointExpr
- *        The logpoint expression
+ * @param {Object} options
+ *        - text: {String} The text to search for. This should be contained in
+ *                         the message. The searching is done with
+ *                         @see findMessageByType.
+ *        - typeSelector: {string} A part of selector for the message, to
+ *                                 specify the message type.
+ *        - expectUrl: {boolean} Whether the URL in the opened source should
+ *                               match the link, or whether it is expected to
+ *                               be null.
+ *        - expectLine: {boolean} It indicates if there is the need to check
+ *                                the line.
+ *        - expectColumn: {boolean} It indicates if there is the need to check
+ *                                the column.
+ *        - logPointExpr: {String} The logpoint expression
  */
 async function testOpenInDebugger(
   hud,
-  toolbox,
-  text,
-  expectUrl = true,
-  expectLine = true,
-  expectColumn = true,
-  logPointExpr = undefined
+  {
+    text,
+    typeSelector,
+    expectUrl = true,
+    expectLine = true,
+    expectColumn = true,
+    logPointExpr = undefined,
+  }
 ) {
   info(`Finding message for open-in-debugger test; text is "${text}"`);
-  const messageNode = await waitFor(() => findMessage(hud, text));
-  const frameLinkNode = messageNode.querySelector(
-    ".message-location .frame-link"
+  const messageNode = await waitFor(() =>
+    findMessageByType(hud, text, typeSelector)
   );
-  ok(frameLinkNode, "The message does have a location link");
+  const locationNode = messageNode.querySelector(".message-location");
+  ok(locationNode, "The message does have a location link");
   await checkClickOnNode(
     hud,
-    toolbox,
-    frameLinkNode,
+    hud.toolbox,
+    locationNode,
     expectUrl,
     expectLine,
     expectColumn,
@@ -901,86 +954,6 @@ async function closeConsole(tab = gBrowser.selectedTab) {
   }
 }
 
-/**
- * Fake clicking a link and return the URL we would have navigated to.
- * This function should be used to check external links since we can't access
- * network in tests.
- * This can also be used to test that a click will not be fired.
- *
- * @param ElementNode element
- *        The <a> element we want to simulate click on.
- * @param Object clickEventProps
- *        The custom properties which would be used to dispatch a click event
- * @returns Promise
- *          A Promise that is resolved when the link click simulation occured or
- *          when the click is not dispatched.
- *          The promise resolves with an object that holds the following properties
- *          - link: url of the link or null(if event not fired)
- *          - where: "tab" if tab is active or "tabshifted" if tab is inactive
- *            or null(if event not fired)
- */
-function simulateLinkClick(element, clickEventProps) {
-  return overrideOpenLink(() => {
-    if (clickEventProps) {
-      // Click on the link using the event properties.
-      element.dispatchEvent(clickEventProps);
-    } else {
-      // Click on the link.
-      element.click();
-    }
-  });
-}
-
-/**
- * Override the browserWindow open*Link function, executes the passed function and either
- * wait for:
- * - the link to be "opened"
- * - 1s before timing out
- * Then it puts back the original open*Link functions in browserWindow.
- *
- * @returns {Promise<Object>}: A promise resolving with an object of the following shape:
- * - link: The link that was "opened"
- * - where: If the link was opened in the background (null) or not ("tab").
- */
-function overrideOpenLink(fn) {
-  const browserWindow = Services.wm.getMostRecentWindow(
-    gDevTools.chromeWindowType
-  );
-
-  // Override LinkIn methods to prevent navigating.
-  const oldOpenTrustedLinkIn = browserWindow.openTrustedLinkIn;
-  const oldOpenWebLinkIn = browserWindow.openWebLinkIn;
-
-  const onOpenLink = new Promise(resolve => {
-    const openLinkIn = function(link, where) {
-      browserWindow.openTrustedLinkIn = oldOpenTrustedLinkIn;
-      browserWindow.openWebLinkIn = oldOpenWebLinkIn;
-      resolve({ link: link, where });
-    };
-    browserWindow.openWebLinkIn = browserWindow.openTrustedLinkIn = openLinkIn;
-    fn();
-  });
-
-  // Declare a timeout Promise that we can use to make sure openTrustedLinkIn or
-  // openWebLinkIn was not called.
-  let timeoutId;
-  const onTimeout = new Promise(function(resolve) {
-    timeoutId = setTimeout(() => {
-      browserWindow.openTrustedLinkIn = oldOpenTrustedLinkIn;
-      browserWindow.openWebLinkIn = oldOpenWebLinkIn;
-      timeoutId = null;
-      resolve({ link: null, where: null });
-    }, 1000);
-  });
-
-  onOpenLink.then(() => {
-    if (timeoutId) {
-      clearTimeout(timeoutId);
-    }
-  });
-  return Promise.race([onOpenLink, onTimeout]);
-}
-
 /**
  * Open a network request logged in the webconsole in the netmonitor panel.
  *
@@ -996,7 +969,7 @@ async function openMessageInNetmonitor(toolbox, hud, url, urlInConsole) {
   urlInConsole = urlInConsole || url;
 
   const message = await waitFor(() =>
-    findMessage(hud, urlInConsole, ".network")
+    findMessageByType(hud, urlInConsole, ".network")
   );
 
   const onNetmonitorSelected = toolbox.once(
@@ -1476,7 +1449,7 @@ async function selectFrame(dbg, frame) {
 async function pauseDebugger(dbg) {
   info("Waiting for debugger to pause");
   const onPaused = waitForPaused(dbg);
-  SpecialPowers.spawn(gBrowser.selectedBrowser, [], function() {
+  SpecialPowers.spawn(gBrowser.selectedBrowser, [], function () {
     content.wrappedJSObject.firstCall();
   }).catch(() => {});
   await onPaused;
@@ -1519,8 +1492,8 @@ function isScrolledToBottom(container) {
  *                        Start the string with "|︎ " to indicate that the message is
  *                        inside a group and should be indented.
  */
-function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
-  const messages = findMessages(hud, "");
+async function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
+  const messages = await findAllMessagesVirtualized(hud);
   is(
     messages.length,
     expectedMessages.length,
@@ -1540,11 +1513,15 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
       ok(false, "Unexpected structure: an indented message isn't in a group");
     }
 
-    return groups[0].startsWith("▶︎⚠") || groups[0].startsWith("▼⚠");
+    return groups[0].startsWith("▼︎⚠");
   };
 
-  expectedMessages.forEach((expectedMessage, i) => {
-    const message = messages[i];
+  for (let [i, expectedMessage] of expectedMessages.entries()) {
+    // Refresh the reference to the message, as it may have been scrolled out of existence.
+    const message = await findMessageVirtualizedById({
+      hud,
+      messageId: messages[i].getAttribute("data-message-id"),
+    });
     info(`Checking "${expectedMessage}"`);
 
     // Collapsed Warning group
@@ -1555,7 +1532,7 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
         "There's a collapsed arrow"
       );
       is(
-        message.querySelector(".indent").getAttribute("data-indent"),
+        message.getAttribute("data-indent"),
         "0",
         "The warningGroup has the expected indent"
       );
@@ -1570,7 +1547,7 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
         "There's an expanded arrow"
       );
       is(
-        message.querySelector(".indent").getAttribute("data-indent"),
+        message.getAttribute("data-indent"),
         "0",
         "The warningGroup has the expected indent"
       );
@@ -1600,11 +1577,8 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
     // In-group message
     if (expectedMessage.startsWith("|")) {
       if (isInWarningGroup(i)) {
-        is(
-          message
-            .querySelector(".indent.warning-indent")
-            .getAttribute("data-indent"),
-          "1",
+        ok(
+          message.querySelector(".warning-indent"),
           "The message has the expected indent"
         );
       }
@@ -1612,7 +1586,7 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
       expectedMessage = expectedMessage.replace("| ", "");
     } else {
       is(
-        message.querySelector(".indent").getAttribute("data-indent"),
+        message.getAttribute("data-indent"),
         "0",
         "The message has the expected indent"
       );
@@ -1623,7 +1597,7 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
       `Message includes ` +
         `the expected "${expectedMessage}" content - "${message.textContent.trim()}"`
     );
-  });
+  }
 }
 
 /**
@@ -1638,7 +1612,7 @@ function checkConsoleOutputForWarningGroup(hud, expectedMessages) {
 async function checkMessageStack(hud, text, expectedFrameLines) {
   info(`Checking message stack for "${text}"`);
   const msgNode = await waitFor(
-    () => findMessage(hud, text),
+    () => findErrorMessage(hud, text),
     `Couln't find message including "${text}"`
   );
   ok(!msgNode.classList.contains("open"), `Error logged not expanded`);
@@ -1781,11 +1755,8 @@ function getContextSelectorItems(hud) {
  * state.
  *
  * @param {WebConsole} hud
- * @param {Array<Object>} expected: An array of object which can have the following shape:
- *         - {String} label: The label of the target
- *         - {String} tooltip: The tooltip of the target element in the menu
- *         - {Boolean} checked: if the target should be selected or not
- *         - {Boolean} separator: if the element is a simple separator
+ * @param {Array<Object>} expected: An array of object (see checkContextSelectorMenuItemAt
+ *                        for expected properties)
  */
 function checkContextSelectorMenu(hud, expected) {
   const items = getContextSelectorItems(hud);
@@ -1796,32 +1767,43 @@ function checkContextSelectorMenu(hud, expected) {
     "The context selector menu has the expected number of items"
   );
 
-  expected.forEach(({ label, tooltip, checked, separator }, i) => {
-    const el = items[i];
-
-    if (separator === true) {
-      is(
-        el.getAttribute("role"),
-        "menuseparator",
-        "The element is a separator"
-      );
-      return;
-    }
-
-    const elChecked = el.getAttribute("aria-checked") === "true";
-    const elTooltip = el.getAttribute("title");
-    const elLabel = el.querySelector(".label").innerText;
-
-    is(elLabel, label, `The item has the expected label`);
-    is(elTooltip, tooltip, `Item "${label}" has the expected tooltip`);
-    is(
-      elChecked,
-      checked,
-      `Item "${label}" is ${checked ? "checked" : "unchecked"}`
-    );
+  expected.forEach((expectedItem, i) => {
+    checkContextSelectorMenuItemAt(hud, i, expectedItem);
   });
 }
 
+/**
+ * Check that the evaluation context selector menu has the expected item at the specified index.
+ *
+ * @param {WebConsole} hud
+ * @param {Number} index
+ * @param {Object} expected
+ * @param {String} expected.label: The label of the target
+ * @param {String} expected.tooltip: The tooltip of the target element in the menu
+ * @param {Boolean} expected.checked: if the target should be selected or not
+ * @param {Boolean} expected.separator: if the element is a simple separator
+ */
+function checkContextSelectorMenuItemAt(hud, index, expected) {
+  const el = getContextSelectorItems(hud).at(index);
+
+  if (expected.separator === true) {
+    is(el.getAttribute("role"), "menuseparator", "The element is a separator");
+    return;
+  }
+
+  const elChecked = el.getAttribute("aria-checked") === "true";
+  const elTooltip = el.getAttribute("title");
+  const elLabel = el.querySelector(".label").innerText;
+
+  is(elLabel, expected.label, `The item has the expected label`);
+  is(elTooltip, expected.tooltip, `Item "${elLabel}" has the expected tooltip`);
+  is(
+    elChecked,
+    expected.checked,
+    `Item "${elLabel}" is ${expected.checked ? "checked" : "unchecked"}`
+  );
+}
+
 /**
  * Select a target in the context selector.
  *
@@ -1896,33 +1878,35 @@ async function getImageSizeFromClipboard() {
   // (which is value of the global `document` here). Doing so might push the
   // toolbox upwards, shrink the content page and fail the fullpage screenshot
   // test.
-  return SpecialPowers.spawn(gBrowser.selectedBrowser, [buffer], async function(
-    _buffer
-  ) {
-    const img = content.document.createElement("img");
-    const loaded = new Promise(r => {
-      img.addEventListener("load", r, { once: true });
-    });
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [buffer],
+    async function (_buffer) {
+      const img = content.document.createElement("img");
+      const loaded = new Promise(r => {
+        img.addEventListener("load", r, { once: true });
+      });
 
-    // Build a URL from the buffer passed to the ContentTask
-    const url = content.URL.createObjectURL(
-      new Blob([_buffer], { type: "image/png" })
-    );
+      // Build a URL from the buffer passed to the ContentTask
+      const url = content.URL.createObjectURL(
+        new Blob([_buffer], { type: "image/png" })
+      );
 
-    // Load the image
-    img.src = url;
-    content.document.documentElement.appendChild(img);
+      // Load the image
+      img.src = url;
+      content.document.documentElement.appendChild(img);
 
-    info("Waiting for the clipboard image to load in the content page");
-    await loaded;
+      info("Waiting for the clipboard image to load in the content page");
+      await loaded;
 
-    // Remove the image and revoke the URL.
-    img.remove();
-    content.URL.revokeObjectURL(url);
+      // Remove the image and revoke the URL.
+      img.remove();
+      content.URL.revokeObjectURL(url);
 
-    return {
-      width: img.width,
-      height: img.height,
-    };
-  });
+      return {
+        width: img.width,
+        height: img.height,
+      };
+    }
+  );
 }
