@@ -3,9 +3,6 @@
 
 "use strict";
 
-const ChromeUtils = require("ChromeUtils");
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
-
 const {
   getAdHocFrontOrPrimitiveGrip,
 } = require("devtools/client/fronts/object");
@@ -191,7 +188,9 @@ function getCleanedPacket(key, packet) {
     res.securityState = existingPacket.securityState;
   }
 
-  if (res.waitingTime && existingPacket.waitingTime) {
+  // waitingTime can be very small and rounded to 0. However this is still a
+  // valid waiting time, so check isNaN instead of a simple truthy check.
+  if (!isNaN(res.waitingTime) && existingPacket.waitingTime) {
     res.waitingTime = existingPacket.waitingTime;
   }
 
@@ -244,13 +243,12 @@ function cleanTimeStamp(packet) {
 /**
  * Write stubs to a given file
  *
- * @param {Object} env
  * @param {String} fileName: The file to write the stubs in.
  * @param {Map} packets: A Map of the packets.
  * @param {Boolean} isNetworkMessage: Is the packets are networkMessage packets
  */
-async function writeStubsToFile(env, fileName, packets, isNetworkMessage) {
-  const mozRepo = env.get("MOZ_DEVELOPER_REPO_DIR");
+async function writeStubsToFile(fileName, packets, isNetworkMessage) {
+  const mozRepo = Services.env.get("MOZ_DEVELOPER_REPO_DIR");
   const filePath = `${mozRepo}/${STUBS_FOLDER + fileName}`;
 
   const serializedPackets = Array.from(packets.entries()).map(
@@ -267,17 +265,17 @@ async function writeStubsToFile(env, fileName, packets, isNetworkMessage) {
 "use strict";
 
 /*
- * THIS FILE IS AUTOGENERATED. DO NOT MODIFY BY HAND. RUN TESTS IN FIXTURES/ TO UPDATE.
+ * THIS FILE IS AUTOGENERATED. DO NOT MODIFY BY HAND. SEE devtools/client/webconsole/test/README.md.
  */
 
 const {
   parsePacketsWithFronts,
-} = require("chrome://mochitests/content/browser/devtools/client/webconsole/test/browser/stub-generator-helpers");
-const { prepareMessage } = require("devtools/client/webconsole/utils/messages");
+} = require("chrome://mochitests/content/browser/devtools/client/webconsole/test/browser/stub-generator-helpers.js");
+const { prepareMessage } = require("resource://devtools/client/webconsole/utils/messages.js");
 const {
   ConsoleMessage,
   NetworkEventMessage,
-} = require("devtools/client/webconsole/types");
+} = require("resource://devtools/client/webconsole/types.js");
 
 const rawPackets = new Map();
 ${serializedPackets.join("\n\n")}
@@ -305,7 +303,7 @@ module.exports = {
 };
 `;
 
-  await OS.File.writeAtomic(filePath, fileContent);
+  await IOUtils.write(filePath, new TextEncoder().encode(fileContent));
 }
 
 function getStubFile(fileName) {
@@ -354,7 +352,7 @@ function getSerializedPacket(
 
   return JSON.stringify(
     packet,
-    function(key, value) {
+    function (key, value) {
       // The message can have fronts that we need to serialize
       if (value && value._grip) {
         return {
@@ -371,6 +369,10 @@ function getSerializedPacket(
         return actorIdPlaceholder;
       }
 
+      if (key === "resourceId") {
+        return undefined;
+      }
+
       return value;
     },
     2
