@@ -6,15 +6,9 @@
 
 // Checks for the AccessibleActor
 
-add_task(async function() {
-  const {
-    target,
-    walker,
-    a11yWalker,
-    parentAccessibility,
-  } = await initAccessibilityFrontsForUrl(
-    MAIN_DOMAIN + "doc_accessibility.html"
-  );
+add_task(async function () {
+  const { target, walker, a11yWalker, parentAccessibility } =
+    await initAccessibilityFrontsForUrl(MAIN_DOMAIN + "doc_accessibility.html");
   const modifiers =
     Services.appinfo.OS === "Darwin" ? "\u2303\u2325" : "Alt+Shift+";
 
@@ -23,7 +17,7 @@ add_task(async function() {
 
   checkA11yFront(accessibleFront, {
     name: "Accessible Button",
-    role: "pushbutton",
+    role: "button",
     childCount: 1,
   });
 
@@ -31,7 +25,7 @@ add_task(async function() {
 
   checkA11yFront(accessibleFront, {
     name: "Accessible Button",
-    role: "pushbutton",
+    role: "button",
     value: "",
     description: "Accessibility Test",
     keyboardShortcut: modifiers + "b",
@@ -92,7 +86,7 @@ add_task(async function() {
   const snapshot = await controlAccessibleFront.snapshot();
   Assert.deepEqual(snapshot, {
     name: "Label",
-    role: "entry",
+    role: "textbox",
     actions: ["Activate"],
     value: "",
     nodeCssSelector: "#control",
@@ -126,6 +120,30 @@ add_task(async function() {
     },
   });
 
+  // Check that we're using ARIA role tokens for landmarks implicit in native
+  // markup.
+  const headerNode = await walker.querySelector(walker.rootNode, "#header");
+  const headerAccessibleFront = await a11yWalker.getAccessibleFor(headerNode);
+  checkA11yFront(headerAccessibleFront, {
+    name: null,
+    role: "banner",
+    childCount: 1,
+  });
+  const navNode = await walker.querySelector(walker.rootNode, "#nav");
+  const navAccessibleFront = await a11yWalker.getAccessibleFor(navNode);
+  checkA11yFront(navAccessibleFront, {
+    name: null,
+    role: "navigation",
+    childCount: 1,
+  });
+  const footerNode = await walker.querySelector(walker.rootNode, "#footer");
+  const footerAccessibleFront = await a11yWalker.getAccessibleFor(footerNode);
+  checkA11yFront(footerAccessibleFront, {
+    name: null,
+    role: "contentinfo",
+    childCount: 1,
+  });
+
   await waitForA11yShutdown(parentAccessibility);
   await target.destroy();
   gBrowser.removeCurrentTab();
