@@ -22,12 +22,9 @@ class nsChildView;
 namespace mozilla {
 
 enum class VibrancyType {
-  LIGHT,
-  DARK,
   TOOLTIP,
   MENU,
   HIGHLIGHTED_MENUITEM,
-  SHEET,
   SOURCE_LIST,
   SOURCE_LIST_SELECTION,
   ACTIVE_SOURCE_LIST_SELECTION
@@ -57,9 +54,7 @@ class VibrancyManager {
    *   NSVisualEffectViews which will be created for vibrant regions.
    */
   VibrancyManager(const nsChildView& aCoordinateConverter, NSView* aContainerView)
-      : mCoordinateConverter(aCoordinateConverter), mContainerView(aContainerView) {
-    MOZ_ASSERT(SystemSupportsVibrancy(), "Don't instantiate this if !SystemSupportsVibrancy()");
-  }
+      : mCoordinateConverter(aCoordinateConverter), mContainerView(aContainerView) {}
 
   /**
    * Update the placement of the NSVisualEffectViews inside the container
@@ -67,25 +62,13 @@ class VibrancyManager {
    * or remove existing ones as needed.
    * @param aType   The vibrancy type to use in the region.
    * @param aRegion The vibrant area, in device pixels.
+   * @return Whether the region changed.
    */
-  void UpdateVibrantRegion(VibrancyType aType, const LayoutDeviceIntRegion& aRegion);
+  bool UpdateVibrantRegion(VibrancyType aType, const LayoutDeviceIntRegion& aRegion);
 
   bool HasVibrantRegions() { return !mVibrantRegions.IsEmpty(); }
 
-  /**
-   * Return the fill color that should be drawn on top of the cleared window
-   * parts. Usually this would be drawn by -[NSVisualEffectView drawRect:].
-   * The returned color is opaque if the system-wide "Reduce transparency"
-   * preference is set.
-   */
-  NSColor* VibrancyFillColorForType(VibrancyType aType);
-
-  /**
-   * Check whether the operating system supports vibrancy at all.
-   * You may only create a VibrancyManager instance if this returns true.
-   * @return Whether VibrancyManager can be used on this OS.
-   */
-  static bool SystemSupportsVibrancy();
+  LayoutDeviceIntRegion GetUnionOfVibrantRegions() const;
 
   /**
    * Create an NSVisualEffectView for the specified vibrancy type. The return
