@@ -45,9 +45,11 @@ class nsFilePicker : public nsBaseFilePicker {
  protected:
   virtual ~nsFilePicker();
 
-  nsresult Show(int16_t* aReturn) override;
+  nsresult Show(nsIFilePicker::ResultCode* aReturn) override;
   void ReadValuesFromFileChooser(void* file_chooser);
 
+  bool WarnForNonReadableFile(void* file_chooser);
+
   static void OnResponse(void* file_chooser, gint response_id,
                          gpointer user_data);
   static void OnDestroy(GtkWidget* file_chooser, gpointer user_data);
@@ -58,7 +60,7 @@ class nsFilePicker : public nsBaseFilePicker {
   nsCOMArray<nsIFile> mFiles;
 
   int16_t mSelectedType;
-  int16_t mResult;
+  nsIFilePicker::ResultCode mResult;
   bool mRunning;
   bool mAllowURLs;
   nsCString mFileURL;
@@ -80,9 +82,7 @@ class nsFilePicker : public nsBaseFilePicker {
   void GtkFileChooserSetModal(void* file_chooser, GtkWindow* parent_widget,
                               gboolean modal);
 
-#ifdef MOZ_WIDGET_GTK
   GtkFileChooserWidget* mFileChooserDelegate;
-#endif
   bool mUseNativeFileChooser;
 };
 
